include("spcm_drv/spcm_drv.jl")
include("spcm_drv/regs.jl")
include("spcm_drv/spcerr.jl")
include("common/spcm_common.jl")

# set the duration of the fifo recording
dFifoRunTime_Seconds = 5.0

# set true to stream data to file
bWriteDataToFile = false

# settings for the fifo mode buffer handling
llBufferSize = Int64(64*1024*1024) # 16 MB
lNotifySize = Int32(1024*1024) # 1 MB

# cards are locked if script aborts without card close
# for debug purposes call spcm_vClose(-1) to free all locked cards
# spcm_vClose(-1)

# open card or digitizerNETBOX
# uncomment the second line and replace the IP address to use remote
# cards like a digitizerNETBOX
hDevice=spcm_hOpen("/dev/spcm0")
# hDevice=spcm_hOpen("TCPIP::192.168.1.10::inst0::INSTR")
if (hDevice == 0)
    println("\nNo card found !!!\n")
    exit(1)
end

# read card type name
qwLen = UInt64(20)
strBuffer = Vector{UInt8}(undef, qwLen)
spcm_dwGetParam_ptr(hDevice, SPC_PCITYP, strBuffer, qwLen)
sCardName = String(strBuffer[1:findfirst(==(0), strBuffer)-1])

# read type, function and sn and check for A/D card
dwError, lCardType     = spcm_dwGetParam_i32(hDevice, SPC_PCITYP)
dwError, lSerialNumber = spcm_dwGetParam_i32(hDevice, SPC_PCISERIALNO)
dwError, lFncType      = spcm_dwGetParam_i32(hDevice, SPC_FNCTYPE)

if lFncType == SPCM_TYPE_AI
    println("")
    println(string("\nFound: ", sCardName, " sn", lSerialNumber))
else
    println("")
    println(string("\nCard: ", sCardName, " sn", lSerialNumber, " not supported by example"))
    spcm_vClose(hDevice)
    exit(1)
end

# do a simple standard setup
dwError = spcm_dwSetParam_i32(hDevice, SPC_CARDMODE,     SPC_REC_FIFO_SINGLE)  # single standard mode
dwError = spcm_dwSetParam_i32(hDevice, SPC_CHENABLE,     CHANNEL0)             # just 1 channel enabled
dwError = spcm_dwSetParam_i32(hDevice, SPC_PRETRIGGER,   Int32(1024))          # 1k of pretrigger data at start of FIFO mode
dwError = spcm_dwSetParam_i32(hDevice, SPC_TRIG_ORMASK,  SPC_TMASK_SOFTWARE)   # trigger set to software
dwError = spcm_dwSetParam_i32(hDevice, SPC_TRIG_ANDMASK, Int32(0))             # ...
dwError = spcm_dwSetParam_i32(hDevice, SPC_CLOCKMODE,    SPC_CM_INTPLL)        # clock mode internal PLL
dwError = spcm_dwSetParam_i32(hDevice, SPC_CLOCKOUT,     Int32(0))             # no clock output
dwError = spcm_dwSetParam_i32(hDevice, SPC_TIMEOUT,      Int32(5000))          # timeout 5 s

# we try to set the samplerate to 100 kHz (M2i), 10 MHz (M2p) or 20 MHz (M3i, M4i)
llSamplingRate = Int64(0)
if (((lCardType & TYP_SERIESMASK) == TYP_M2ISERIES) || ((lCardType & TYP_SERIESMASK) == TYP_M2IEXPSERIES))
    llSamplingRate = 100000
elseif ((lCardType & TYP_SERIESMASK) == TYP_M2PEXPSERIES)
    llSamplingRate = 10000000
else
    llSamplingRate = 20000000
end

dwError = spcm_dwSetParam_i64(hDevice, SPC_SAMPLERATE, llSamplingRate)

# open file
if (bWriteDataToFile)
    fileStream = open("stream.bin","w")
end

dwError, lBytesPerSample = spcm_dwGetParam_i64(hDevice, SPC_MIINST_BYTESPERSAMPLE)

if (lBytesPerSample == 1)
    pBuffer = zeros(Int8, llBufferSize)
else
    pBuffer = zeros(Int16, Int64(llBufferSize / 2))
end

dwError = spcm_dwDefTransfer_i64(hDevice, SPCM_BUF_DATA, SPCM_DIR_CARDTOPC, UInt32(lNotifySize), pBuffer, UInt64(0), UInt64(llBufferSize))

qwTotalMem = UInt64(0)

dTime_Start = time()

# start everything
dwError = spcm_dwSetParam_i32(hDevice, SPC_M2CMD, M2CMD_CARD_START | M2CMD_CARD_ENABLETRIGGER | M2CMD_DATA_STARTDMA)

# check for error
if (dwError != ERR_OK)
    dwError, dwErrorReg, lErrorVal, sErrorText = spcm_dwGetErrorInfo_i32(hDevice)
    println(sErrorText)

# run the FIFO mode and loop through the data
else
    #while (qwTotalMem < qwToTransfer)
    while (time() - dTime_Start < dFifoRunTime_Seconds)

        dwError = spcm_dwSetParam_i32(hDevice, SPC_M2CMD, M2CMD_DATA_WAITDMA)
        if (dwError != ERR_OK)
            if (dwError == ERR_TIMEOUT)
                println("... Timeout")
            else
                global dwError, dwErrorReg, lErrorVal, sErrorText = spcm_dwGetErrorInfo_i32(hDevice)
                println(sErrorText)
            end
            break
        else
            global dwError, lStatus = spcm_dwGetParam_i32(hDevice, SPC_M2STATUS)
            global dwError, lFillSize = spcm_dwGetParam_i32(hDevice, SPC_FILLSIZEPROMILLE)

            global dwError, llAvailUser = spcm_dwGetParam_i64(hDevice, SPC_DATA_AVAIL_USER_LEN)
            global dwError, llPCPos = spcm_dwGetParam_i64(hDevice, SPC_DATA_AVAIL_USER_POS)

            if (llAvailUser >= lNotifySize)
                global qwTotalMem += lNotifySize
                println("Transferred: ", qwTotalMem / Int32(1024*1024), " MB", ", HW Buffer Fillsize ", lFillSize / 10, " %")

                ####################################################
                # this is the point to do anything with the data
                ####################################################

                # write data to file
                if (bWriteDataToFile)
                    llPos_Start = Int64(llPCPos / lBytesPerSample) + 1;
                    llPos_End   = Int64((llPCPos / lBytesPerSample) + (lNotifySize / lBytesPerSample))

                    if (llPos_End > (llBufferSize / lBytesPerSample))
                        llPos_End = (llBufferSize / lBytesPerSample)
                    end

                    write(fileStream,pBuffer[llPos_Start:llPos_End])
                end

                # free fifo block
                dwError = spcm_dwSetParam_i32(hDevice, SPC_DATA_AVAIL_CARD_LEN, lNotifySize)
            end
        end
    end
end

# send the stop command
dwError = spcm_dwSetParam_i32(hDevice, SPC_M2CMD, M2CMD_CARD_STOP | M2CMD_DATA_STOPDMA)

println(string("\nFifo transfer stopped after ", round(time() - dTime_Start), " seconds\n"))

if (dwError != ERR_OK)
    dwError, dwErrorReg, lErrorVal, sErrorText = spcm_dwGetErrorInfo_i32(hDevice);
    println(sErrorText)
end

# close file
if (bWriteDataToFile)
    close(fileStream)
end

# close card or digitizerNETBOX
spcm_vClose(hDevice)
