include("spcm_drv/spcm_drv.jl")
include("spcm_drv/regs.jl")
include("spcm_drv/spcerr.jl")
include("common/spcm_common.jl")

# set to true to plot signals (requires installation of Plots package)
bUsePlots = false

if (bUsePlots)
    using Plots
end

# cards are locked if script aborts without card close
# for debug purposes call spcm_vClose(-1) to free all locked cards
# spcm_vClose(-1)

# open card or digitizerNETBOX
# uncomment the second line and replace the IP address to use remote
# cards like a digitizerNETBOX
hDevice=spcm_hOpen("/dev/spcm0")
# hDevice=spcm_hOpen("TCPIP::192.168.1.10::inst0::INSTR")
if (hDevice == 0)
    println("\nNo card found !!!\n")
    exit(1)
end

# read card type name
qwLen = UInt64(20)
strBuffer = Vector{UInt8}(undef, qwLen)
spcm_dwGetParam_ptr(hDevice, SPC_PCITYP, strBuffer, qwLen)
sCardName = String(strBuffer[1:findfirst(==(0), strBuffer)-1])

# read type, function and sn and check for A/D card
dwError, lCardType     = spcm_dwGetParam_i32(hDevice, SPC_PCITYP)
dwError, lSerialNumber = spcm_dwGetParam_i32(hDevice, SPC_PCISERIALNO)
dwError, lFncType      = spcm_dwGetParam_i32(hDevice, SPC_FNCTYPE)
dwError, lNumModules   = spcm_dwGetParam_i32(hDevice, SPC_MIINST_MODULES)
dwError, lChPerModule  = spcm_dwGetParam_i32(hDevice, SPC_MIINST_CHPERMODULE)

lMaxChannels = lNumModules * lChPerModule

if lFncType == SPCM_TYPE_AI
    println("")
    println(string("\nFound: ", sCardName, " sn", lSerialNumber))
else
    println("")
    println(string("\nCard: ", sCardName, " sn", lSerialNumber, " not supported by example"))
    spcm_vClose(hDevice)
    exit(1)
end

# activate all available channels
lChEnabledMask = Int32((1 << lMaxChannels) - 1)

# set memsize to 16 kS
llMemsize=Int64(16384)

# do a simple standard setup
dwError = spcm_dwSetParam_i32(hDevice, SPC_CARDMODE,     SPC_REC_STD_SINGLE)    # single standard mode
dwError = spcm_dwSetParam_i32(hDevice, SPC_CHENABLE,     lChEnabledMask)        # channel enabled mask
dwError = spcm_dwSetParam_i64(hDevice, SPC_MEMSIZE,      llMemsize)             # set memsize
dwError = spcm_dwSetParam_i64(hDevice, SPC_POSTTRIGGER,  Int64(llMemsize/2))    # half of the total number of samples after trigger event
dwError = spcm_dwSetParam_i32(hDevice, SPC_TRIG_ORMASK,  SPC_TMASK_SOFTWARE)    # trigger set to software
dwError = spcm_dwSetParam_i32(hDevice, SPC_TRIG_ANDMASK, Int32(0))              # ...
dwError = spcm_dwSetParam_i32(hDevice, SPC_CLOCKMODE,    SPC_CM_INTPLL)         # clock mode internal PLL
dwError = spcm_dwSetParam_i32(hDevice, SPC_CLOCKOUT,     Int32(0))              # no clock output
dwError = spcm_dwSetParam_i32(hDevice, SPC_TIMEOUT,      Int32(5000))           # timeout 5 s

# set card to maximum sampling rate
dwError, llMaxSamplingrate = spcm_dwGetParam_i64(hDevice, SPC_PCISAMPLERATE)
dwError = spcm_dwSetParam_i64(hDevice, SPC_SAMPLERATE, llMaxSamplingrate)

dwError, lNumCh = spcm_dwGetParam_i32(hDevice, SPC_CHCOUNT)

print("\nStart acquisition ... ")

#start card and wait for card ready
dwError = spcm_dwSetParam_i32(hDevice, SPC_M2CMD, M2CMD_CARD_START | M2CMD_CARD_ENABLETRIGGER | M2CMD_CARD_WAITREADY)

# check for error
if (dwError != ERR_OK)
    println("failed")
    dwError, dwErrorReg, lErrorVal, sErrorText = spcm_dwGetErrorInfo_i32(hDevice)
    println(sErrorText)
else
    dwError, lBytesPerSample = spcm_dwGetParam_i64(hDevice, SPC_MIINST_BYTESPERSAMPLE)

    if (lBytesPerSample == 1)
        pBuffer = zeros(Int8, llMemsize * lNumCh)
    else
        pBuffer = zeros(Int16, llMemsize * lNumCh)
    end

    llBufferSize=sizeof(pBuffer)

    dwError = spcm_dwDefTransfer_i64(hDevice, SPCM_BUF_DATA, SPCM_DIR_CARDTOPC, UInt32(0), pBuffer, UInt64(0), UInt64(llBufferSize))

    # start DMA and wait for DMA transfer ready state
    dwError = spcm_dwSetParam_i32(hDevice, SPC_M2CMD, M2CMD_DATA_STARTDMA | M2CMD_DATA_WAITDMA)

    if (dwError != ERR_OK)
        println("failed\n")
        dwError, dwErrorReg, lErrorVal, sErrorText = spcm_dwGetErrorInfo_i32(hDevice)
        println(sErrorText)
    else
        println("done\n")

        Ch_Data = zeros(Int16, lNumCh, llMemsize)

        # demultiplex channel data from buffer
        for lIdx = 1 : lNumCh
            Ch_Data[lIdx,1:llMemsize] = pBuffer[lIdx:lNumCh:llMemsize * lNumCh]
        end

        # calculate minimum and maximum sample values for each channel
        aMaxValues = Int16[]
        aMinValues = Int16[]
        for lChIdx = 1 : lNumCh
            push!(aMaxValues, Int16(-32768))
            push!(aMinValues, Int16(32767))

            for lDataIdx = 1 : llMemsize
                if (Ch_Data[lChIdx, lDataIdx] > aMaxValues[lChIdx])
                    aMaxValues[lChIdx] = Ch_Data[lChIdx, lDataIdx]
                end
                if (Ch_Data[lChIdx, lDataIdx] < aMinValues[lChIdx])
                    aMinValues[lChIdx] = Ch_Data[lChIdx, lDataIdx]
                end
            end
        end

        # print minimum and maximum sample values
        for lIdx = 1 : lNumCh
            println(string("Ch", lIdx - 1, " => Min: ", aMinValues[lIdx], ", Max: ", aMaxValues[lIdx]))
        end

        # plot signals
        if (bUsePlots)
            if (lNumCh == 1)
                display(plot(Dat_Ch0, label = "Ch0"))
            end
            if (lNumCh == 2)
                display(plot([Ch_Data[1,:], Ch_Data[2,:]], label = ["Ch0" "Ch1"]))
            end
            if (lNumCh == 4)
                display(plot([Ch_Data[1,:], Ch_Data[2,:], Ch_Data[3,:], Ch_Data[4,:]], label = ["Ch0" "Ch1" "Ch2" "Ch3"]))
            end
            if (lNumCh == 8)
                display(plot([Ch_Data[1,:], Ch_Data[2,:], Ch_Data[3,:], Ch_Data[4,:], Ch_Data[5,:], Ch_Data[6,:], Ch_Data[7,:], Ch_Data[8,:]], label = ["Ch0" "Ch1" "Ch2" "Ch3" "Ch4" "Ch5" "Ch6" "Ch7"]))
            end
        end
    end
end

# close card or digitizerNETBOX
spcm_vClose(hDevice)
