include("spcm_drv/spcm_drv.jl")
include("spcm_drv/regs.jl")
include("spcm_drv/spcerr.jl")
include("common/spcm_common.jl")

lReplayTimeout_Seconds = 10

# cards are locked if script aborts without card close
# for debug purposes call spcm_vClose(-1) to free all locked cards
# spcm_vClose(-1)

# open card or digitizerNETBOX
# uncomment the second line and replace the IP address to use remote
# cards like a digitizerNETBOX
hDevice=spcm_hOpen("/dev/spcm0")
# hDevice=spcm_hOpen("TCPIP::192.168.1.10::inst0::INSTR")
if (hDevice == 0)
    println("\nNo card found !!!\n")
    exit(1)
end

# read card type name
qwLen = UInt64(20)
strBuffer = Vector{UInt8}(undef, qwLen)
spcm_dwGetParam_ptr(hDevice, SPC_PCITYP, strBuffer, qwLen)
sCardName = String(strBuffer[1:findfirst(==(0), strBuffer)-1])

# read type, function and sn and check for A/D card
dwError, lCardType      = spcm_dwGetParam_i32(hDevice, SPC_PCITYP)
dwError, lSerialNumber  = spcm_dwGetParam_i32(hDevice, SPC_PCISERIALNO)
dwError, lFncType       = spcm_dwGetParam_i32(hDevice, SPC_FNCTYPE)
dwError, lNumModules    = spcm_dwGetParam_i32(hDevice, SPC_MIINST_MODULES)
dwError, lChPerModule   = spcm_dwGetParam_i32(hDevice, SPC_MIINST_CHPERMODULE)
dwError, lBitsPerSample = spcm_dwGetParam_i32(hDevice, SPC_MIINST_BITSPERSAMPLE)

lMaxChannels = lNumModules * lChPerModule

if (lFncType == SPCM_TYPE_AO)
    println("")
    println(string("Found: ", sCardName, " sn", lSerialNumber))
else
    println("")
    println(string("Card: ", sCardName, " sn", lSerialNumber, " not supported by example"))
    spcm_vClose(hDevice)
    exit(1)
end

# activate all available channels
lChEnabledMask = Int32((1 << lMaxChannels) - 1)

# set memsize to 16 kS
llMemsize=Int64(16384)
llLoops=Int64(0) # continuously

# do a simple standard setup
dwError = spcm_dwSetParam_i32(hDevice, SPC_CARDMODE,     SPC_REP_STD_CONTINUOUS) # continuously replay mode
dwError = spcm_dwSetParam_i32(hDevice, SPC_CHENABLE,     lChEnabledMask)         # channel enabled mask
dwError = spcm_dwSetParam_i64(hDevice, SPC_MEMSIZE,      llMemsize)              # set memsize
dwError = spcm_dwSetParam_i64(hDevice, SPC_LOOPS,        llLoops)                # set number of replay loops
dwError = spcm_dwSetParam_i32(hDevice, SPC_TRIG_ORMASK,  SPC_TMASK_SOFTWARE)     # trigger set to software
dwError = spcm_dwSetParam_i32(hDevice, SPC_TRIG_ANDMASK, Int32(0))               # ...
dwError = spcm_dwSetParam_i32(hDevice, SPC_CLOCKMODE,    SPC_CM_INTPLL)          # clock mode internal PLL
dwError = spcm_dwSetParam_i32(hDevice, SPC_CLOCKOUT,     Int32(0))               # no clock output

# set card to maximum sampling rate
dwError, llMaxSamplingrate = spcm_dwGetParam_i64(hDevice, SPC_PCISAMPLERATE)
dwError = spcm_dwSetParam_i64(hDevice, SPC_SAMPLERATE, llMaxSamplingrate)

dwError, lNumCh = spcm_dwGetParam_i32(hDevice, SPC_CHCOUNT)

# do setup for each channel
for lIdx = 1 : lMaxChannels
    global dwError = spcm_dwSetParam_i32(hDevice, SPC_AMP0 + Int32((lIdx - 1) * 100), Int32(1000))    # set output level to 1000 mV
    global dwError = spcm_dwSetParam_i32(hDevice, SPC_ENABLEOUT0 + Int32((lIdx - 1) * 100), Int32(1)) # enable channel output
end

# calculate sine waveforms for all channels
aWaveforms = zeros(Int16, lNumCh, llMemsize)

fullScale = 2^lBitsPerSample / 2 - 1

for lChIdx = 1 : lNumCh
    for lDataIdx = 1 : llMemsize
        aWaveforms[lChIdx, lDataIdx] = Int16(floor(fullScale * sin(2 * π * (lDataIdx - 1) / llMemsize * lChIdx)))
    end
end

# allocate buffer memory and copy waveforms to buffer
pBuffer = zeros(Int16, llMemsize * lNumCh)

for lChIdx = 1 : lNumCh
    pBuffer[lChIdx:lNumCh:llMemsize * lNumCh] = aWaveforms[lChIdx,:]
end

dwError = spcm_dwDefTransfer_i64(hDevice, SPCM_BUF_DATA, SPCM_DIR_PCTOCARD, UInt32(0), pBuffer, UInt64(0), UInt64(sizeof(pBuffer)))
dwError = spcm_dwSetParam_i32(hDevice, SPC_M2CMD, M2CMD_DATA_STARTDMA | M2CMD_DATA_WAITDMA)
println("\nData has been transferred to board memory\n")

print(string("Start replay for ", lReplayTimeout_Seconds, " seconds ... "))

# set timeout to abort continuously replay
dwError = spcm_dwSetParam_i64(hDevice, SPC_TIMEOUT, lReplayTimeout_Seconds * 1000)

# start replay
dwError = spcm_dwSetParam_i32(hDevice, SPC_M2CMD, M2CMD_CARD_START | M2CMD_CARD_ENABLETRIGGER | M2CMD_CARD_WAITREADY)

if (dwError == ERR_TIMEOUT)
    print("finished.")
    spcm_dwSetParam_i32(hDevice, SPC_M2CMD, M2CMD_CARD_STOP)
else
    if (dwError != ERR_OK)
        dwError, dwErrorReg, lErrorVal, sErrorText = spcm_dwGetErrorInfo_i32(hDevice)
        println(sErrorText)
    end
end

# close card or digitizerNETBOX
spcm_vClose(hDevice)
