
# ***********************************************************************
#
# SpcErr.h                                        (c) Spectrum GmbH, 2006
#
# ***********************************************************************
#
# error codes of the Spectrum drivers. Until may 2004 this file was 
# errors.h. Name has been changed because errors.h has been already in 
# use by windows.
#
# ***********************************************************************

const SPCM_ERROR_ORIGIN_MASK        = Int32(-2147483648) # 0x80000000 # this bit marks the origin of the error
const SPCM_ERROR_ORIGIN_LOCAL       = Int32(0) # 0x00000000 # error occured on local system
const SPCM_ERROR_ORIGIN_REMOTE      = Int32(-2147483648) # 0x80000000 # error occured on remote system (netbox)

const ERR_OK                = Int32(0) # 0x0000      #   0 No Error
const ERR_INIT              = Int32(1) # 0x0001      #   1 Initialisation error
const ERR_NR                = Int32(2) # 0x0002      #   2 Board number out of range
const ERR_TYP               = Int32(3) # 0x0003      #   3 Unknown board Typ
const ERR_FNCNOTSUPPORTED   = Int32(4) # 0x0004      #   4 This function is not supported by the hardware
const ERR_BRDREMAP          = Int32(5) # 0x0005      #   5 The Board Index Remap table is wrong
const ERR_KERNELVERSION     = Int32(6) # 0x0006      #   6 The kernel version and the dll version are mismatching
const ERR_HWDRVVERSION      = Int32(7) # 0x0007      #   7 The driver version doesn't match the minimum requirements of the board
const ERR_ADRRANGE          = Int32(8) # 0x0008      #   8 The address range is disabled (fatal error)
const ERR_INVALIDHANDLE     = Int32(9) # 0x0009      #   9 Handle not valid
const ERR_BOARDNOTFOUND     = Int32(10) # 0x000A      #  10 Card with given name hasn't been found
const ERR_BOARDINUSE        = Int32(11) # 0x000B      #  11 Card with given name is already in use by another application
const ERR_EXPHW64BITADR     = Int32(12) # 0x000C      #  12 Express hardware version not able to handle 64 bit addressing -> update needed
const ERR_FWVERSION         = Int32(13) # 0x000D      #  13 Firmware versions of synchronized cards or for this driver do not match -> update needed
const ERR_SYNCPROTOCOL      = Int32(14) # 0x000E      #  14 Synchronization protocol of synchronized cards does not match -> either update needed or card types are not synchronizable
const ERR_KERNEL            = Int32(15) # 0x000F      #  15 Some error occurred in the kernel driver
const ERR_LASTERR           = Int32(16) # 0x0010      #  16 Old Error waiting to be read
const ERR_ABORT             = Int32(32) # 0x0020      #  32 Abort of wait function
const ERR_BOARDLOCKED       = Int32(48) # 0x0030      #  48 Board acess already locked by another process. it's not possible to acess one board through multiple processes
const ERR_DEVICE_MAPPING    = Int32(50) # 0x0032      #  50 Device is mapped to an invalid device
const ERR_NETWORKSETUP      = Int32(64) # 0x0040      #  64 Network setup failed
const ERR_NETWORKTRANSFER   = Int32(65) # 0x0041      #  65 Network data transfer failed
const ERR_FWPOWERCYCLE      = Int32(66) # 0x0042      #  66 Power cycle needed to update card's firmware (simple PC reboot not sufficient !)
const ERR_NETWORKTIMEOUT    = Int32(67) # 0x0043      #  67 Network timeout
const ERR_BUFFERSIZE        = Int32(68) # 0x0044      #  68 Buffer too small
const ERR_RESTRICTEDACCESS  = Int32(69) # 0x0045      #  69 access to card has been restricted
const ERR_INVALIDPARAM      = Int32(70) # 0x0046      #  70 invalid parameter for function
const ERR_TEMPERATURE       = Int32(71) # 0x0047      #  71 card temperature too high
const ERR_FAN               = Int32(72) # 0x0048      #  72 fan speed too low

const ERR_REG               = Int32(256) # 0x0100      # 256 unknown Register for this Board
const ERR_VALUE             = Int32(257) # 0x0101      # 257 Not a possible value in this state
const ERR_FEATURE           = Int32(258) # 0x0102      # 258 Feature of the board not installed
const ERR_SEQUENCE          = Int32(259) # 0x0103      # 259 Channel sequence not allowed
const ERR_READABORT         = Int32(260) # 0x0104      # 260 Read not allowed after abort
const ERR_NOACCESS          = Int32(261) # 0x0105      # 261 Access to this register denied
const ERR_POWERDOWN         = Int32(262) # 0x0106      # 262 not allowed in Powerdown mode
const ERR_TIMEOUT           = Int32(263) # 0x0107      # 263 timeout occured while waiting for interrupt
const ERR_CALLTYPE          = Int32(264) # 0x0108      # 264 call type (int32 mux) is not allowed for this register
const ERR_EXCEEDSINT32      = Int32(265) # 0x0109      # 265 return value is int32 but software register exceeds the 32 bit integer range -> use 2x32 or 64
const ERR_NOWRITEALLOWED    = Int32(266) # 0x010A      # 266 register cannot be written, read only
const ERR_SETUP             = Int32(267) # 0x010B      # 267 the setup isn't valid
const ERR_CLOCKNOTLOCKED    = Int32(268) # 0x010C      # 268 clock section not locked: perhaps no external clock signal connected or not stable
const ERR_MEMINIT           = Int32(269) # 0x010D      # 269 on-board memory initialization error
const ERR_POWERSUPPLY       = Int32(270) # 0x010E      # 270 on-board power supply error
const ERR_ADCCOMMUNICATION  = Int32(271) # 0x010F      # 271 communication with ADC failed
const ERR_CHANNEL           = Int32(272) # 0x0110      # 272 Wrong number of Channel to be read out
const ERR_NOTIFYSIZE        = Int32(273) # 0x0111      # 273 Notify block size isn't valid
const ERR_TOOSMALL          = Int32(274) # 0x0112      # 274 resulting value is too small
const ERR_RUNNING           = Int32(288) # 0x0120      # 288 Board is running, changes not allowed
const ERR_ADJUST            = Int32(304) # 0x0130      # 304 Auto Adjust has an error
const ERR_PRETRIGGERLEN     = Int32(320) # 0x0140      # 320 pretrigger length exceeds allowed values
const ERR_DIRMISMATCH       = Int32(321) # 0x0141      # 321 direction of card and memory transfer mismatch
const ERR_POSTEXCDSEGMENT   = Int32(322) # 0x0142      # 322 posttrigger exceeds segment size in multiple recording mode
const ERR_SEGMENTINMEM      = Int32(323) # 0x0143      # 323 memsize is not a multiple of segmentsize, last segment hasn't full length
const ERR_MULTIPLEPW        = Int32(324) # 0x0144      # 324 multiple pulsewidth counters used but card only supports one at the time
const ERR_NOCHANNELPWOR     = Int32(325) # 0x0145      # 325 channel pulsewidth can't be OR'd
const ERR_ANDORMASKOVRLAP   = Int32(326) # 0x0146      # 326 AND mask and OR mask overlap in at least one channel -> not possible
const ERR_ANDMASKEDGE       = Int32(327) # 0x0147      # 327 AND mask together with edge trigger mode is not allowed
const ERR_ORMASKLEVEL       = Int32(328) # 0x0148      # 328 OR mask together with level trigger mode is not allowed
const ERR_EDGEPERMOD        = Int32(329) # 0x0149      # 329 All trigger edges must be simular on one module
const ERR_DOLEVELMINDIFF    = Int32(330) # 0x014A      # 330 minimum difference between low output level and high output level not reached
const ERR_STARHUBENABLE     = Int32(331) # 0x014B      # 331 card holding the star-hub must be active for sync
const ERR_PATPWSMALLEDGE    = Int32(332) # 0x014C      # 332 Combination of pattern with pulsewidht smaller and edge is not allowed
const ERR_XMODESETUP        = Int32(333) # 0x014D      # 333 The chosen setup for (SPCM_X0_MODE .. SPCM_X19_MODE) is not valid. See hardware manual for details.
const ERR_AVRG_TDA          = Int32(334) # 0x014E      # 334 Setup for Average TDA Mode not valid. Check Threshold and Replacement values for chosen AVRGMODE.

const ERR_NOPCI             = Int32(512) # 0x0200      # 512 No PCI bus found
const ERR_PCIVERSION        = Int32(513) # 0x0201      # 513 Wrong PCI bus version
const ERR_PCINOBOARDS       = Int32(514) # 0x0202      # 514 No Spectrum PCI boards found
const ERR_PCICHECKSUM       = Int32(515) # 0x0203      # 515 Checksum error on PCI board
const ERR_DMALOCKED         = Int32(516) # 0x0204      # 516 DMA buffer in use, try later
const ERR_MEMALLOC          = Int32(517) # 0x0205      # 517 Memory Allocation error
const ERR_EEPROMLOAD        = Int32(518) # 0x0206      # 518 EEProm load error
const ERR_CARDNOSUPPORT     = Int32(519) # 0x0207      # 519 no support for that card in the library
const ERR_CONFIGACCESS      = Int32(520) # 0x0208      # 520 error occured during config write or read

const ERR_FIFOBUFOVERRUN    = Int32(768) # 0x0300      # 768 Buffer overrun in FIFO mode
const ERR_FIFOHWOVERRUN     = Int32(769) # 0x0301      # 769 hardware buffer overrun (acquisition) or underrun (replay) in FIFO mode
const ERR_FIFOFINISHED      = Int32(770) # 0x0302      # 770 FIFO transfer hs been finished. Number of buffers has been transferred
const ERR_FIFOSETUP         = Int32(777) # 0x0309      # 777 FIFO setup not possible, transfer rate to high (max 250 MB/s)

const ERR_TIMESTAMP_SYNC    = Int32(784) # 0x0310      # 784 Synchronisation to ref clock failed
const ERR_STARHUB           = Int32(800) # 0x0320      # 800 Autorouting of Starhub failed

const ERR_INTERNAL_ERROR    = Int32(65535) # 0xFFFF      # 65535 Internal hardware error detected, please check for update


