if Sys.islinux()
    const spcm_dll="libspcm_linux.so"
else
    const spcm_dll="c:/Windows/System32/spcm_win64.dll"
end

# *****************************************************************************
# spcm_hOpen
# *****************************************************************************
spcm_hOpen(sDeviceName::String) = ccall((:spcm_hOpen,spcm_dll),Int64,(Cstring,),sDeviceName)

# *****************************************************************************
# spcm_vClose
# *****************************************************************************
spcm_vClose(hDevice::Int64) = ccall((:spcm_vClose,spcm_dll),Cvoid,(Int64,),hDevice)

# *****************************************************************************
# spcm_dwGetParam_i32
# *****************************************************************************
function spcm_dwGetParam_i32(hDevice::Int64,lRegister::Int32)
    plValue=Ref(Int32(0))
    dwError = ccall((:spcm_dwGetParam_i32,spcm_dll),UInt32,(Int64,Int32,Base.Ref{Int32},),hDevice,lRegister,plValue)
    return dwError, plValue[]
end

# *****************************************************************************
# spcm_dwGetParam_i64
# *****************************************************************************
function spcm_dwGetParam_i64(hDevice::Int64,lRegister::Int32)
    pllValue=Ref(Int64(0))
    dwError = ccall((:spcm_dwGetParam_i64,spcm_dll),UInt32,(Int64,Int32,Base.Ref{Int64},),hDevice,lRegister,pllValue)
    return dwError, pllValue[]
end

# *****************************************************************************
# spcm_dwGetParam_ptr 
# *****************************************************************************
function spcm_dwGetParam_ptr(hDevice::Int64,lRegister::Int32,pValue::Array{UInt8,1},qwLen::UInt64)
    ccall((:spcm_dwGetParam_ptr,spcm_dll),UInt32,(Int64,Int32,Ptr{UInt8},UInt64,),hDevice,lRegister,pValue,qwLen)
end

# *****************************************************************************
# spcm_dwGetParam_d64
# *****************************************************************************
function spcm_dwGetParam_d64(hDevice::Int64,lRegister::Int32)
    pdValue=Ref(Float64(0))
    dwError = ccall((:spcm_dwGetParam_d64,spcm_dll),UInt32,(Int64,Int32,Base.Ref{Float64},),hDevice,lRegister,pllValue)
    return dwError, pdValue[]
end

# *****************************************************************************
# spcm_dwSetParam_i32
# *****************************************************************************
function spcm_dwSetParam_i32(hDevice::Int64,lRegister::Int32,lValue::Int32)
    ccall((:spcm_dwSetParam_i32,spcm_dll),UInt32,(Int64,Int32,Int32,),hDevice,lRegister,lValue)
end

# *****************************************************************************
# spcm_dwSetParam_i64
# *****************************************************************************
function spcm_dwSetParam_i64(hDevice::Int64,lRegister::Int32,llValue::Int64)
    ccall((:spcm_dwSetParam_i64,spcm_dll),UInt32,(Int64,Int32,Int64,),hDevice,lRegister,llValue)
end

# *****************************************************************************
# spcm_dwSetParam_ptr
# *****************************************************************************
function spcm_dwSetParam_ptr(hDevice::Int64,lRegister::Int32,pValue::Array{Int8,1},qwLen::UInt64)
    ccall((:spcm_dwSetParam_ptr,spcm_dll),UInt32,(Int64,Int32,Ptr{Int8},UInt64,),hDevice,lRegister,pValue,qwLen)
end

# *****************************************************************************
# spcm_dwSetParam_d64
# *****************************************************************************
function spcm_dwSetParam_d64(hDevice::Int64,lRegister::Int32,dValue::Float64)
    ccall((:spcm_dwSetParam_d64,spcm_dll),UInt32,(Int64,Int32,Float64,),hDevice,lRegister,dValue)
end

# defintions of the transfer direction
const SPCM_DIR_PCTOCARD  = Int32(0)       # transfer from PC memory to card memory
const SPCM_DIR_CARDTOPC  = Int32(1)       # transfer from card memory to PC memory
const SPCM_DIR_CARDTOGPU = Int32(2)       # RDMA transfer from card memory to GPU memory
const SPCM_DIR_GPUTOCARD = Int32(3)       # RDMA transfer from GPU memory to card memory

# defintions of the different data buffers
const SPCM_BUF_DATA      = Int32(1000)    # main data buffer for acquired or generated samples
const SPCM_BUF_ABA       = Int32(2000)    # buffer for ABA data, holds the A-DATA (slow samples)
const SPCM_BUF_TIMESTAMP = Int32(3000)    # buffer for timestamps
const SPCM_BUF_LOG       = Int32(4000)    # write content of buffer to log file

# *****************************************************************************
# spcm_dwDefTransfer_i64 (Int16)
# *****************************************************************************
function spcm_dwDefTransfer_i64(hDevice::Int64, lBufType::Int32, lDirection::Int32, dwNotifySize::UInt32, pDataBuffer::Array{Int16,1}, qwBrdOffs::UInt64, qwTransferLen::UInt64)
    ccall((:spcm_dwDefTransfer_i64,spcm_dll),UInt32,(Int64,Int32,Int32,UInt32,Ptr{Int16},UInt64,UInt64,),hDevice,lBufType,lDirection,dwNotifySize,pDataBuffer,qwBrdOffs,qwTransferLen)
end

# *****************************************************************************
# spcm_dwDefTransfer_i64 (Int8)
# *****************************************************************************
function spcm_dwDefTransfer_i64(hDevice::Int64, lBufType::Int32, lDirection::Int32, dwNotifySize::UInt32, pDataBuffer::Array{Int8,1}, qwBrdOffs::UInt64, qwTransferLen::UInt64)
    ccall((:spcm_dwDefTransfer_i64,spcm_dll),UInt32,(Int64,Int32,Int32,UInt32,Ptr{Int8},UInt64,UInt64,),hDevice,lBufType,lDirection,dwNotifySize,pDataBuffer,qwBrdOffs,qwTransferLen)
end

# *****************************************************************************
# spcm_dwInvalidateBuf
# *****************************************************************************
function spcm_dwInvalidateBuf(hDevice::Int64, lBufType::Int32)
	ccall((:spcm_dwInvalidateBuf,spcm_dll),UInt32,(Int64,Int32,),hDevice,lBufType)
end

# *****************************************************************************
# spcm_dwGetErrorInfo_i32
# *****************************************************************************
function spcm_dwGetErrorInfo_i32(hDevice::Int64)
    pdwErrorReg=Ref(UInt32(0))
    plErrorValue=Ref(Int32(0))
    pszErrorTextBuffer=Vector{UInt8}(undef, 256)
    dwError = ccall((:spcm_dwGetErrorInfo_i32,spcm_dll),UInt32,(Int64,Base.Ref{UInt32},Base.Ref{Int32},Ptr{UInt8},),hDevice,pdwErrorReg,plErrorValue,pszErrorTextBuffer)
	return dwError, pdwErrorReg[], plErrorValue[], unsafe_string(pointer(pszErrorTextBuffer))
end

# *****************************************************************************
# spcm_dwGetErrorInfo_i64
# *****************************************************************************
function spcm_dwGetErrorInfo_i64(hDevice::Int64)
    pdwErrorReg=Ref(UInt32(0))
    pllErrorValue=Ref(Int64(0))
    pszErrorTextBuffer=Vector{UInt8}(undef, 256)
    dwError = ccall((:spcm_dwGetErrorInfo_i64,spcm_dll),UInt32,(Int64,Base.Ref{UInt32},Base.Ref{Int64},Ptr{UInt8},),hDevice,pdwErrorReg,plErrorValue,pszErrorTextBuffer)
	return dwError, pdwErrorReg[], pllErrorValue[], unsafe_string(pointer(pszErrorTextBuffer))
end

# *****************************************************************************
# spcm_dwGetErrorInfo_d64
# *****************************************************************************
function spcm_dwGetErrorInfo_d64(hDevice::Int64)
    pdwErrorReg=Ref(UInt32(0))
    pdErrorValue=Ref(Float64(0))
    pszErrorTextBuffer=Vector{UInt8}(undef, 256)
    dwError = ccall((:spcm_dwGetErrorInfo_d64,spcm_dll),UInt32,(Int64,Base.Ref{UInt32},Base.Ref{Float64},Ptr{UInt8},),hDevice,pdwErrorReg,plErrorValue,pszErrorTextBuffer)
	return dwError, pdwErrorReg[], pdErrorValue[], unsafe_string(pointer(pszErrorTextBuffer))
end
