# ***********************************************************************
#
# regs.h                                          (c) Spectrum GmbH, 2006
#
# ***********************************************************************
#
# software register and constants definition for all Spectrum drivers. 
# Please stick to the card manual to see which of the inhere defined 
# registers are used on your hardware.
#
# ***********************************************************************



# ***********************************************************************
# macros for kilo, Mega or Giga as standard version or binary (_B) (2^x)
# ***********************************************************************





# ***********************************************************************
# card types
# ***********************************************************************

const TYP_PCIDEVICEID              = Int32(0) # 0x00000000

# ***** Board Types ***************
const TYP_EVAL                     = Int32(16) # 0x00000010
const TYP_RSDLGA                   = Int32(20) # 0x00000014
const TYP_GMG                      = Int32(24) # 0x00000018
const TYP_VAN8                     = Int32(32) # 0x00000020
const TYP_VAC                      = Int32(40) # 0x00000028

const TYP_PCIAUTOINSTALL           = Int32(255) # 0x000000FF

const TYP_DAP116                   = Int32(256) # 0x00000100
const TYP_PAD82                    = Int32(512) # 0x00000200
const TYP_PAD82a                   = Int32(528) # 0x00000210
const TYP_PAD82b                   = Int32(544) # 0x00000220
const TYP_PCI212                   = Int32(768) # 0x00000300
const TYP_PAD1232a                 = Int32(1024) # 0x00000400
const TYP_PAD1232b                 = Int32(1040) # 0x00000410
const TYP_PAD1232c                 = Int32(1056) # 0x00000420
const TYP_PAD1616a                 = Int32(1280) # 0x00000500
const TYP_PAD1616b                 = Int32(1296) # 0x00000510
const TYP_PAD1616c                 = Int32(1312) # 0x00000520
const TYP_PAD1616d                 = Int32(1328) # 0x00000530
const TYP_PAD52                    = Int32(1536) # 0x00000600
const TYP_PAD242                   = Int32(1792) # 0x00000700
const TYP_PCK400                   = Int32(2048) # 0x00000800
const TYP_PAD164_2M                = Int32(2304) # 0x00000900
const TYP_PAD164_5M                = Int32(2320) # 0x00000910
const TYP_PCI208                   = Int32(4096) # 0x00001000
const TYP_CPCI208                  = Int32(4097) # 0x00001001
const TYP_PCI412                   = Int32(4352) # 0x00001100
const TYP_PCIDIO32                 = Int32(4608) # 0x00001200
const TYP_PCI248                   = Int32(4864) # 0x00001300
const TYP_PADCO                    = Int32(5120) # 0x00001400
const TYP_TRS582                   = Int32(5376) # 0x00001500
const TYP_PCI258                   = Int32(5632) # 0x00001600


# ------ series and familiy identifiers -----
const TYP_SERIESMASK               = Int32(16711680) # 0x00FF0000     # the series (= type of base card), e.g. MI.xxxx
const TYP_VERSIONMASK              = Int32(65535) # 0x0000FFFF     # the version, e.g. XX.3012
const TYP_FAMILYMASK               = Int32(65280) # 0x0000FF00     # the family, e.g. XX.30xx
const TYP_TYPEMASK                 = Int32(255) # 0x000000FF     # the type, e.g. XX.xx12
const TYP_SPEEDMASK                = Int32(240) # 0x000000F0     # the speed grade, e.g. XX.xx1x
const TYP_CHMASK                   = Int32(15) # 0x0000000F     # the channel/modules, e.g. XX.xxx2

const TYP_MISERIES                 = Int32(0) # 0x00000000
const TYP_MCSERIES                 = Int32(65536) # 0x00010000
const TYP_MXSERIES                 = Int32(131072) # 0x00020000
const TYP_M2ISERIES                = Int32(196608) # 0x00030000
const TYP_M2IEXPSERIES             = Int32(262144) # 0x00040000
const TYP_M3ISERIES                = Int32(327680) # 0x00050000
const TYP_M3IEXPSERIES             = Int32(393216) # 0x00060000
const TYP_M4IEXPSERIES             = Int32(458752) # 0x00070000
const TYP_M4XEXPSERIES             = Int32(524288) # 0x00080000
const TYP_M2PEXPSERIES             = Int32(589824) # 0x00090000
const TYP_M5IEXPSERIES             = Int32(655360) # 0x000A0000



# ----- MI.20xx, MC.20xx, MX.20xx -----
const TYP_MI2020                   = Int32(8224) # 0x00002020
const TYP_MI2021                   = Int32(8225) # 0x00002021
const TYP_MI2025                   = Int32(8229) # 0x00002025
const TYP_MI2030                   = Int32(8240) # 0x00002030
const TYP_MI2031                   = Int32(8241) # 0x00002031

const TYP_M2I2020                  = Int32(204832) # 0x00032020
const TYP_M2I2021                  = Int32(204833) # 0x00032021
const TYP_M2I2025                  = Int32(204837) # 0x00032025
const TYP_M2I2030                  = Int32(204848) # 0x00032030
const TYP_M2I2031                  = Int32(204849) # 0x00032031

const TYP_M2I2020EXP               = Int32(270368) # 0x00042020
const TYP_M2I2021EXP               = Int32(270369) # 0x00042021
const TYP_M2I2025EXP               = Int32(270373) # 0x00042025
const TYP_M2I2030EXP               = Int32(270384) # 0x00042030
const TYP_M2I2031EXP               = Int32(270385) # 0x00042031

const TYP_MC2020                   = Int32(73760) # 0x00012020
const TYP_MC2021                   = Int32(73761) # 0x00012021
const TYP_MC2025                   = Int32(73765) # 0x00012025
const TYP_MC2030                   = Int32(73776) # 0x00012030
const TYP_MC2031                   = Int32(73777) # 0x00012031

const TYP_MX2020                   = Int32(139296) # 0x00022020
const TYP_MX2025                   = Int32(139301) # 0x00022025
const TYP_MX2030                   = Int32(139312) # 0x00022030

# ----- M3i.21xx, M3i.21xx-Exp (8 bit A/D) -----
const TYP_M3I2120                  = Int32(336160) # 0x00052120     # 1x500M
const TYP_M3I2122                  = Int32(336162) # 0x00052122     # 1x500M & 2x250M
const TYP_M3I2130                  = Int32(336176) # 0x00052130     # 1x1G
const TYP_M3I2132                  = Int32(336178) # 0x00052132     # 1x1G & 2x500M

const TYP_M3I2120EXP               = Int32(401696) # 0x00062120     # 1x500M
const TYP_M3I2122EXP               = Int32(401698) # 0x00062122     # 1x500M & 2x250M
const TYP_M3I2130EXP               = Int32(401712) # 0x00062130     # 1x1G
const TYP_M3I2132EXP               = Int32(401714) # 0x00062132     # 1x1G & 2x500M

# ----- M4i.22xx-x8 (8 bit A/D) -----
const TYP_M4I22XX_X8               = Int32(467456) # 0x00072200
const TYP_M4I2210_X8               = Int32(467472) # 0x00072210     # 1x1.25G
const TYP_M4I2211_X8               = Int32(467473) # 0x00072211     # 2x1.25G
const TYP_M4I2212_X8               = Int32(467474) # 0x00072212     # 4x1.25G
const TYP_M4I2220_X8               = Int32(467488) # 0x00072220     # 1x2.5G
const TYP_M4I2221_X8               = Int32(467489) # 0x00072221     # 2x2.5G
const TYP_M4I2223_X8               = Int32(467491) # 0x00072223     # 1x2.5G & 2x1.25G
const TYP_M4I2230_X8               = Int32(467504) # 0x00072230     # 1x5G
const TYP_M4I2233_X8               = Int32(467507) # 0x00072233     # 1x5G & 2x2.5G 
const TYP_M4I2234_X8               = Int32(467508) # 0x00072234     # 1x5G & 2x2.5G & 4x1.25G
const TYP_M4I2280_X8               = Int32(467584) # 0x00072280     # customer specific variant
const TYP_M4I2281_X8               = Int32(467585) # 0x00072281     # customer specific variant
const TYP_M4I2283_X8               = Int32(467587) # 0x00072283     # customer specific variant
const TYP_M4I2290_X8               = Int32(467600) # 0x00072290     # customer specific variant
const TYP_M4I2293_X8               = Int32(467603) # 0x00072293     # customer specific variant
const TYP_M4I2294_X8               = Int32(467604) # 0x00072294     # customer specific variant

# ----- M4x.22xx-x8 (8 bit A/D) -----
const TYP_M4X22XX_X4               = Int32(532992) # 0x00082200
const TYP_M4X2210_X4               = Int32(533008) # 0x00082210     # 1x1.25G
const TYP_M4X2211_X4               = Int32(533009) # 0x00082211     # 2x1.25G
const TYP_M4X2212_X4               = Int32(533010) # 0x00082212     # 4x1.25G
const TYP_M4X2220_X4               = Int32(533024) # 0x00082220     # 1x2.5G
const TYP_M4X2221_X4               = Int32(533025) # 0x00082221     # 2x2.5G
const TYP_M4X2223_X4               = Int32(533027) # 0x00082223     # 1x2.5G & 2x1.25G
const TYP_M4X2230_X4               = Int32(533040) # 0x00082230     # 1x5G
const TYP_M4X2233_X4               = Int32(533043) # 0x00082233     # 1x5G & 2x2.5G 
const TYP_M4X2234_X4               = Int32(533044) # 0x00082234     # 1x5G & 2x2.5G & 4x1.25G

# ----- M4i.23xx-x8 (7 bit A/D) -----
const TYP_M4I23XX_X8               = Int32(467712) # 0x00072300
const TYP_M4I2320_X8               = Int32(467744) # 0x00072320     # 1x2.5G
const TYP_M4I2321_X8               = Int32(467745) # 0x00072321     # 2x2.5G
const TYP_M4I2323_X8               = Int32(467747) # 0x00072323     # 1x2.5G & 2x1.25G
const TYP_M4I2330_X8               = Int32(467760) # 0x00072330     # 1x5G
const TYP_M4I2333_X8               = Int32(467763) # 0x00072333     # 1x5G & 2x2.5G 
const TYP_M4I2334_X8               = Int32(467764) # 0x00072334     # 1x5G & 2x2.5G & 4x1.25G

# ----- MI.30xx, MC.30xx, MX.30xx -----
const TYP_MI3010                   = Int32(12304) # 0x00003010
const TYP_MI3011                   = Int32(12305) # 0x00003011
const TYP_MI3012                   = Int32(12306) # 0x00003012
const TYP_MI3013                   = Int32(12307) # 0x00003013
const TYP_MI3014                   = Int32(12308) # 0x00003014
const TYP_MI3015                   = Int32(12309) # 0x00003015
const TYP_MI3016                   = Int32(12310) # 0x00003016
const TYP_MI3020                   = Int32(12320) # 0x00003020
const TYP_MI3021                   = Int32(12321) # 0x00003021
const TYP_MI3022                   = Int32(12322) # 0x00003022
const TYP_MI3023                   = Int32(12323) # 0x00003023
const TYP_MI3024                   = Int32(12324) # 0x00003024
const TYP_MI3025                   = Int32(12325) # 0x00003025
const TYP_MI3026                   = Int32(12326) # 0x00003026
const TYP_MI3027                   = Int32(12327) # 0x00003027
const TYP_MI3031                   = Int32(12337) # 0x00003031
const TYP_MI3033                   = Int32(12339) # 0x00003033

const TYP_M2I3010                  = Int32(208912) # 0x00033010
const TYP_M2I3011                  = Int32(208913) # 0x00033011
const TYP_M2I3012                  = Int32(208914) # 0x00033012
const TYP_M2I3013                  = Int32(208915) # 0x00033013
const TYP_M2I3014                  = Int32(208916) # 0x00033014
const TYP_M2I3015                  = Int32(208917) # 0x00033015
const TYP_M2I3016                  = Int32(208918) # 0x00033016
const TYP_M2I3020                  = Int32(208928) # 0x00033020
const TYP_M2I3021                  = Int32(208929) # 0x00033021
const TYP_M2I3022                  = Int32(208930) # 0x00033022
const TYP_M2I3023                  = Int32(208931) # 0x00033023
const TYP_M2I3024                  = Int32(208932) # 0x00033024
const TYP_M2I3025                  = Int32(208933) # 0x00033025
const TYP_M2I3026                  = Int32(208934) # 0x00033026
const TYP_M2I3027                  = Int32(208935) # 0x00033027
const TYP_M2I3031                  = Int32(208945) # 0x00033031
const TYP_M2I3033                  = Int32(208947) # 0x00033033

const TYP_M2I3010EXP               = Int32(274448) # 0x00043010
const TYP_M2I3011EXP               = Int32(274449) # 0x00043011
const TYP_M2I3012EXP               = Int32(274450) # 0x00043012
const TYP_M2I3013EXP               = Int32(274451) # 0x00043013
const TYP_M2I3014EXP               = Int32(274452) # 0x00043014
const TYP_M2I3015EXP               = Int32(274453) # 0x00043015
const TYP_M2I3016EXP               = Int32(274454) # 0x00043016
const TYP_M2I3020EXP               = Int32(274464) # 0x00043020
const TYP_M2I3021EXP               = Int32(274465) # 0x00043021
const TYP_M2I3022EXP               = Int32(274466) # 0x00043022
const TYP_M2I3023EXP               = Int32(274467) # 0x00043023
const TYP_M2I3024EXP               = Int32(274468) # 0x00043024
const TYP_M2I3025EXP               = Int32(274469) # 0x00043025
const TYP_M2I3026EXP               = Int32(274470) # 0x00043026
const TYP_M2I3027EXP               = Int32(274471) # 0x00043027
const TYP_M2I3031EXP               = Int32(274481) # 0x00043031
const TYP_M2I3033EXP               = Int32(274483) # 0x00043033

const TYP_MC3010                   = Int32(77840) # 0x00013010
const TYP_MC3011                   = Int32(77841) # 0x00013011
const TYP_MC3012                   = Int32(77842) # 0x00013012
const TYP_MC3013                   = Int32(77843) # 0x00013013
const TYP_MC3014                   = Int32(77844) # 0x00013014
const TYP_MC3015                   = Int32(77845) # 0x00013015
const TYP_MC3016                   = Int32(77846) # 0x00013016
const TYP_MC3020                   = Int32(77856) # 0x00013020
const TYP_MC3021                   = Int32(77857) # 0x00013021
const TYP_MC3022                   = Int32(77858) # 0x00013022
const TYP_MC3023                   = Int32(77859) # 0x00013023
const TYP_MC3024                   = Int32(77860) # 0x00013024
const TYP_MC3025                   = Int32(77861) # 0x00013025
const TYP_MC3026                   = Int32(77862) # 0x00013026
const TYP_MC3027                   = Int32(77863) # 0x00013027
const TYP_MC3031                   = Int32(77873) # 0x00013031
const TYP_MC3033                   = Int32(77875) # 0x00013033

const TYP_MX3010                   = Int32(143376) # 0x00023010
const TYP_MX3011                   = Int32(143377) # 0x00023011
const TYP_MX3012                   = Int32(143378) # 0x00023012
const TYP_MX3020                   = Int32(143392) # 0x00023020
const TYP_MX3021                   = Int32(143393) # 0x00023021
const TYP_MX3022                   = Int32(143394) # 0x00023022
const TYP_MX3031                   = Int32(143409) # 0x00023031



# ----- MI.31xx, MC.31xx, MX.31xx -----
const TYP_MI3110                   = Int32(12560) # 0x00003110
const TYP_MI3111                   = Int32(12561) # 0x00003111
const TYP_MI3112                   = Int32(12562) # 0x00003112
const TYP_MI3120                   = Int32(12576) # 0x00003120
const TYP_MI3121                   = Int32(12577) # 0x00003121
const TYP_MI3122                   = Int32(12578) # 0x00003122
const TYP_MI3130                   = Int32(12592) # 0x00003130
const TYP_MI3131                   = Int32(12593) # 0x00003131
const TYP_MI3132                   = Int32(12594) # 0x00003132
const TYP_MI3140                   = Int32(12608) # 0x00003140

const TYP_M2I3110                  = Int32(209168) # 0x00033110
const TYP_M2I3111                  = Int32(209169) # 0x00033111
const TYP_M2I3112                  = Int32(209170) # 0x00033112
const TYP_M2I3120                  = Int32(209184) # 0x00033120
const TYP_M2I3121                  = Int32(209185) # 0x00033121
const TYP_M2I3122                  = Int32(209186) # 0x00033122
const TYP_M2I3130                  = Int32(209200) # 0x00033130
const TYP_M2I3131                  = Int32(209201) # 0x00033131
const TYP_M2I3132                  = Int32(209202) # 0x00033132

const TYP_M2I3110EXP               = Int32(274704) # 0x00043110
const TYP_M2I3111EXP               = Int32(274705) # 0x00043111
const TYP_M2I3112EXP               = Int32(274706) # 0x00043112
const TYP_M2I3120EXP               = Int32(274720) # 0x00043120
const TYP_M2I3121EXP               = Int32(274721) # 0x00043121
const TYP_M2I3122EXP               = Int32(274722) # 0x00043122
const TYP_M2I3130EXP               = Int32(274736) # 0x00043130
const TYP_M2I3131EXP               = Int32(274737) # 0x00043131
const TYP_M2I3132EXP               = Int32(274738) # 0x00043132

const TYP_MC3110                   = Int32(78096) # 0x00013110
const TYP_MC3111                   = Int32(78097) # 0x00013111
const TYP_MC3112                   = Int32(78098) # 0x00013112
const TYP_MC3120                   = Int32(78112) # 0x00013120
const TYP_MC3121                   = Int32(78113) # 0x00013121
const TYP_MC3122                   = Int32(78114) # 0x00013122
const TYP_MC3130                   = Int32(78128) # 0x00013130
const TYP_MC3131                   = Int32(78129) # 0x00013131
const TYP_MC3132                   = Int32(78130) # 0x00013132

const TYP_MX3110                   = Int32(143632) # 0x00023110
const TYP_MX3111                   = Int32(143633) # 0x00023111
const TYP_MX3120                   = Int32(143648) # 0x00023120
const TYP_MX3121                   = Int32(143649) # 0x00023121
const TYP_MX3130                   = Int32(143664) # 0x00023130
const TYP_MX3131                   = Int32(143665) # 0x00023131



# ----- M3i.32xx, M3i.32xx-Exp (12 bit A/D) -----
const TYP_M3I3220                  = Int32(340512) # 0x00053220     # 1x250M
const TYP_M3I3221                  = Int32(340513) # 0x00053221     # 2x250M
const TYP_M3I3240                  = Int32(340544) # 0x00053240     # 1x500M
const TYP_M3I3242                  = Int32(340546) # 0x00053242     # 1x500M & 2x250M

const TYP_M3I3220EXP               = Int32(406048) # 0x00063220     # 1x250M
const TYP_M3I3221EXP               = Int32(406049) # 0x00063221     # 2x250M
const TYP_M3I3240EXP               = Int32(406080) # 0x00063240     # 1x500M
const TYP_M3I3242EXP               = Int32(406082) # 0x00063242     # 1x500M & 2x250M

# ----- M5i.33xx-x4 (12 bit A/D) -----
const TYP_M5I33XX_X16              = Int32(668416) # 0x000A3300      # generic
const TYP_M5I3321_X16              = Int32(668449) # 0x000A3321      # 2x3.2G 12bit
const TYP_M5I3330_X16              = Int32(668464) # 0x000A3330      # 1x6.4G 12bit
const TYP_M5I3337_X16              = Int32(668471) # 0x000A3337      # 1x6.4G 2x3.2G 12bit
const TYP_M5I3350_X16              = Int32(668496) # 0x000A3350      # 1x10G 12bit
const TYP_M5I3357_X16              = Int32(668503) # 0x000A3357      # 1x10G 2x5G 12bit
const TYP_M5I3360_X16              = Int32(668512) # 0x000A3360      # 1x10G 12bit high bandwidth
const TYP_M5I3367_X16              = Int32(668519) # 0x000A3367      # 1x10G 2x5G 12bit high bandwidth

# ----- MI.40xx, MC.40xx, MX.40xx -----
const TYP_MI4020                   = Int32(16416) # 0x00004020
const TYP_MI4021                   = Int32(16417) # 0x00004021
const TYP_MI4022                   = Int32(16418) # 0x00004022
const TYP_MI4030                   = Int32(16432) # 0x00004030
const TYP_MI4031                   = Int32(16433) # 0x00004031
const TYP_MI4032                   = Int32(16434) # 0x00004032

const TYP_M2I4020                  = Int32(213024) # 0x00034020
const TYP_M2I4021                  = Int32(213025) # 0x00034021
const TYP_M2I4022                  = Int32(213026) # 0x00034022
const TYP_M2I4028                  = Int32(213032) # 0x00034028
const TYP_M2I4030                  = Int32(213040) # 0x00034030
const TYP_M2I4031                  = Int32(213041) # 0x00034031
const TYP_M2I4032                  = Int32(213042) # 0x00034032
const TYP_M2I4038                  = Int32(213048) # 0x00034038

const TYP_M2I4020EXP               = Int32(278560) # 0x00044020
const TYP_M2I4021EXP               = Int32(278561) # 0x00044021
const TYP_M2I4022EXP               = Int32(278562) # 0x00044022
const TYP_M2I4028EXP               = Int32(278568) # 0x00044028
const TYP_M2I4030EXP               = Int32(278576) # 0x00044030
const TYP_M2I4031EXP               = Int32(278577) # 0x00044031
const TYP_M2I4032EXP               = Int32(278578) # 0x00044032
const TYP_M2I4038EXP               = Int32(278584) # 0x00044038

const TYP_MC4020                   = Int32(81952) # 0x00014020
const TYP_MC4021                   = Int32(81953) # 0x00014021
const TYP_MC4022                   = Int32(81954) # 0x00014022
const TYP_MC4030                   = Int32(81968) # 0x00014030
const TYP_MC4031                   = Int32(81969) # 0x00014031
const TYP_MC4032                   = Int32(81970) # 0x00014032

const TYP_MX4020                   = Int32(147488) # 0x00024020
const TYP_MX4021                   = Int32(147489) # 0x00024021
const TYP_MX4030                   = Int32(147504) # 0x00024030
const TYP_MX4031                   = Int32(147505) # 0x00024031



# ----- M3i.41xx, M3i.41xx-Exp (14 bit A/D) -----
const TYP_M3I4110                  = Int32(344336) # 0x00054110     # 1x100M
const TYP_M3I4111                  = Int32(344337) # 0x00054111     # 2x100M
const TYP_M3I4120                  = Int32(344352) # 0x00054120     # 1x250M
const TYP_M3I4121                  = Int32(344353) # 0x00054121     # 2x250M
const TYP_M3I4140                  = Int32(344384) # 0x00054140     # 1x400M
const TYP_M3I4142                  = Int32(344386) # 0x00054142     # 1x400M & 2x250M

const TYP_M3I4110EXP               = Int32(409872) # 0x00064110     # 1x100M
const TYP_M3I4111EXP               = Int32(409873) # 0x00064111     # 2x100M
const TYP_M3I4120EXP               = Int32(409888) # 0x00064120     # 1x250M
const TYP_M3I4121EXP               = Int32(409889) # 0x00064121     # 2x250M
const TYP_M3I4140EXP               = Int32(409920) # 0x00064140     # 1x400M
const TYP_M3I4142EXP               = Int32(409922) # 0x00064142     # 1x400M & 2x250M

# ----- M4i.44xx-x8 (generic) -----
const TYP_M4I44XX_X8               = Int32(476160) # 0x00074400      # 

const TYP_M4I4410_X8               = Int32(476176) # 0x00074410      # 2x130M 16bit
const TYP_M4I4411_X8               = Int32(476177) # 0x00074411      # 4x130M 16bit
const TYP_M4I4420_X8               = Int32(476192) # 0x00074420      # 2x250M 16bit
const TYP_M4I4421_X8               = Int32(476193) # 0x00074421      # 4x250M 16bit
const TYP_M4I4450_X8               = Int32(476240) # 0x00074450      # 2x500M 14bit
const TYP_M4I4451_X8               = Int32(476241) # 0x00074451      # 4x500M 14bit
const TYP_M4I4470_X8               = Int32(476272) # 0x00074470      # 2x180M 16bit
const TYP_M4I4471_X8               = Int32(476273) # 0x00074471      # 4x180M 16bit
const TYP_M4I4480_X8               = Int32(476288) # 0x00074480      # 2x400M 14bit
const TYP_M4I4481_X8               = Int32(476289) # 0x00074481      # 4x400M 14bit

# ----- M4x.44xx-x4 (14/16 bit A/D) -----
const TYP_M4X44XX_X4               = Int32(541696) # 0x00084400      # generic
const TYP_M4X4410_X4               = Int32(541712) # 0x00084410      # 2x130M 16bit
const TYP_M4X4411_X4               = Int32(541713) # 0x00084411      # 4x130M 16bit
const TYP_M4X4420_X4               = Int32(541728) # 0x00084420      # 2x250M 16bit
const TYP_M4X4421_X4               = Int32(541729) # 0x00084421      # 4x250M 16bit
const TYP_M4X4450_X4               = Int32(541776) # 0x00084450      # 2x500M 14bit
const TYP_M4X4451_X4               = Int32(541777) # 0x00084451      # 4x500M 14bit
const TYP_M4X4470_X4               = Int32(541808) # 0x00084470      # 2x180M 16bit
const TYP_M4X4471_X4               = Int32(541809) # 0x00084471      # 4x180M 16bit
const TYP_M4X4480_X4               = Int32(541824) # 0x00084480      # 2x400M 14bit
const TYP_M4X4481_X4               = Int32(541825) # 0x00084481      # 4x400M 14bit


# ----- MI.45xx, MC.45xx, MX.45xx -----
const TYP_MI4520                   = Int32(17696) # 0x00004520
const TYP_MI4521                   = Int32(17697) # 0x00004521
const TYP_MI4530                   = Int32(17712) # 0x00004530
const TYP_MI4531                   = Int32(17713) # 0x00004531
const TYP_MI4540                   = Int32(17728) # 0x00004540
const TYP_MI4541                   = Int32(17729) # 0x00004541

const TYP_M2I4520                  = Int32(214304) # 0x00034520
const TYP_M2I4521                  = Int32(214305) # 0x00034521
const TYP_M2I4530                  = Int32(214320) # 0x00034530
const TYP_M2I4531                  = Int32(214321) # 0x00034531
const TYP_M2I4540                  = Int32(214336) # 0x00034540
const TYP_M2I4541                  = Int32(214337) # 0x00034541

const TYP_MC4520                   = Int32(83232) # 0x00014520
const TYP_MC4521                   = Int32(83233) # 0x00014521
const TYP_MC4530                   = Int32(83248) # 0x00014530
const TYP_MC4531                   = Int32(83249) # 0x00014531
const TYP_MC4540                   = Int32(83264) # 0x00014540
const TYP_MC4541                   = Int32(83265) # 0x00014541

const TYP_MX4520                   = Int32(148768) # 0x00024520
const TYP_MX4530                   = Int32(148784) # 0x00024530
const TYP_MX4540                   = Int32(148800) # 0x00024540



# ----- MI.46xx, MC.46xx, MX.46xx -----
const TYP_MI4620                   = Int32(17952) # 0x00004620
const TYP_MI4621                   = Int32(17953) # 0x00004621
const TYP_MI4622                   = Int32(17954) # 0x00004622
const TYP_MI4630                   = Int32(17968) # 0x00004630
const TYP_MI4631                   = Int32(17969) # 0x00004631
const TYP_MI4632                   = Int32(17970) # 0x00004632
const TYP_MI4640                   = Int32(17984) # 0x00004640
const TYP_MI4641                   = Int32(17985) # 0x00004641
const TYP_MI4642                   = Int32(17986) # 0x00004642
const TYP_MI4650                   = Int32(18000) # 0x00004650
const TYP_MI4651                   = Int32(18001) # 0x00004651
const TYP_MI4652                   = Int32(18002) # 0x00004652

const TYP_M2I4620                  = Int32(214560) # 0x00034620
const TYP_M2I4621                  = Int32(214561) # 0x00034621
const TYP_M2I4622                  = Int32(214562) # 0x00034622
const TYP_M2I4630                  = Int32(214576) # 0x00034630
const TYP_M2I4631                  = Int32(214577) # 0x00034631
const TYP_M2I4632                  = Int32(214578) # 0x00034632
const TYP_M2I4640                  = Int32(214592) # 0x00034640
const TYP_M2I4641                  = Int32(214593) # 0x00034641
const TYP_M2I4642                  = Int32(214594) # 0x00034642
const TYP_M2I4650                  = Int32(214608) # 0x00034650
const TYP_M2I4651                  = Int32(214609) # 0x00034651
const TYP_M2I4652                  = Int32(214610) # 0x00034652

const TYP_M2I4620EXP               = Int32(280096) # 0x00044620
const TYP_M2I4621EXP               = Int32(280097) # 0x00044621
const TYP_M2I4622EXP               = Int32(280098) # 0x00044622
const TYP_M2I4630EXP               = Int32(280112) # 0x00044630
const TYP_M2I4631EXP               = Int32(280113) # 0x00044631
const TYP_M2I4632EXP               = Int32(280114) # 0x00044632
const TYP_M2I4640EXP               = Int32(280128) # 0x00044640
const TYP_M2I4641EXP               = Int32(280129) # 0x00044641
const TYP_M2I4642EXP               = Int32(280130) # 0x00044642
const TYP_M2I4650EXP               = Int32(280144) # 0x00044650
const TYP_M2I4651EXP               = Int32(280145) # 0x00044651
const TYP_M2I4652EXP               = Int32(280146) # 0x00044652

const TYP_MC4620                   = Int32(83488) # 0x00014620
const TYP_MC4621                   = Int32(83489) # 0x00014621
const TYP_MC4622                   = Int32(83490) # 0x00014622
const TYP_MC4630                   = Int32(83504) # 0x00014630
const TYP_MC4631                   = Int32(83505) # 0x00014631
const TYP_MC4632                   = Int32(83506) # 0x00014632
const TYP_MC4640                   = Int32(83520) # 0x00014640
const TYP_MC4641                   = Int32(83521) # 0x00014641
const TYP_MC4642                   = Int32(83522) # 0x00014642
const TYP_MC4650                   = Int32(83536) # 0x00014650
const TYP_MC4651                   = Int32(83537) # 0x00014651
const TYP_MC4652                   = Int32(83538) # 0x00014652

const TYP_MX4620                   = Int32(149024) # 0x00024620
const TYP_MX4621                   = Int32(149025) # 0x00024621
const TYP_MX4630                   = Int32(149040) # 0x00024630
const TYP_MX4631                   = Int32(149041) # 0x00024631
const TYP_MX4640                   = Int32(149056) # 0x00024640
const TYP_MX4641                   = Int32(149057) # 0x00024641
const TYP_MX4650                   = Int32(149072) # 0x00024650
const TYP_MX4651                   = Int32(149073) # 0x00024651



# ----- MI.47xx, MC.47xx, MX.47xx -----
const TYP_MI4710                   = Int32(18192) # 0x00004710
const TYP_MI4711                   = Int32(18193) # 0x00004711
const TYP_MI4720                   = Int32(18208) # 0x00004720
const TYP_MI4721                   = Int32(18209) # 0x00004721
const TYP_MI4730                   = Int32(18224) # 0x00004730
const TYP_MI4731                   = Int32(18225) # 0x00004731
const TYP_MI4740                   = Int32(18240) # 0x00004740
const TYP_MI4741                   = Int32(18241) # 0x00004741

const TYP_M2I4710                  = Int32(214800) # 0x00034710
const TYP_M2I4711                  = Int32(214801) # 0x00034711
const TYP_M2I4720                  = Int32(214816) # 0x00034720
const TYP_M2I4721                  = Int32(214817) # 0x00034721
const TYP_M2I4730                  = Int32(214832) # 0x00034730
const TYP_M2I4731                  = Int32(214833) # 0x00034731
const TYP_M2I4740                  = Int32(214848) # 0x00034740
const TYP_M2I4741                  = Int32(214849) # 0x00034741

const TYP_M2I4710EXP               = Int32(280336) # 0x00044710
const TYP_M2I4711EXP               = Int32(280337) # 0x00044711
const TYP_M2I4720EXP               = Int32(280352) # 0x00044720
const TYP_M2I4721EXP               = Int32(280353) # 0x00044721
const TYP_M2I4730EXP               = Int32(280368) # 0x00044730
const TYP_M2I4731EXP               = Int32(280369) # 0x00044731
const TYP_M2I4740EXP               = Int32(280384) # 0x00044740
const TYP_M2I4741EXP               = Int32(280385) # 0x00044741

const TYP_MC4710                   = Int32(83728) # 0x00014710
const TYP_MC4711                   = Int32(83729) # 0x00014711
const TYP_MC4720                   = Int32(83744) # 0x00014720
const TYP_MC4721                   = Int32(83745) # 0x00014721
const TYP_MC4730                   = Int32(83760) # 0x00014730
const TYP_MC4731                   = Int32(83761) # 0x00014731

const TYP_MX4710                   = Int32(149264) # 0x00024710
const TYP_MX4720                   = Int32(149280) # 0x00024720
const TYP_MX4730                   = Int32(149296) # 0x00024730



# ----- M3i.48xx, M3i.48xx-Exp (16 bit A/D) -----
const TYP_M3I4830                  = Int32(346160) # 0x00054830     
const TYP_M3I4831                  = Int32(346161) # 0x00054831    
const TYP_M3I4840                  = Int32(346176) # 0x00054840     
const TYP_M3I4841                  = Int32(346177) # 0x00054841    
const TYP_M3I4860                  = Int32(346208) # 0x00054860     
const TYP_M3I4861                  = Int32(346209) # 0x00054861    

const TYP_M3I4830EXP               = Int32(411696) # 0x00064830     
const TYP_M3I4831EXP               = Int32(411697) # 0x00064831    
const TYP_M3I4840EXP               = Int32(411712) # 0x00064840     
const TYP_M3I4841EXP               = Int32(411713) # 0x00064841    
const TYP_M3I4860EXP               = Int32(411744) # 0x00064860     
const TYP_M3I4861EXP               = Int32(411745) # 0x00064861    



# ----- MI.46xx, MC.46xx, MX.46xx -----
const TYP_MI4911                   = Int32(18705) # 0x00004911
const TYP_MI4912                   = Int32(18706) # 0x00004912
const TYP_MI4931                   = Int32(18737) # 0x00004931
const TYP_MI4932                   = Int32(18738) # 0x00004932
const TYP_MI4960                   = Int32(18784) # 0x00004960
const TYP_MI4961                   = Int32(18785) # 0x00004961
const TYP_MI4963                   = Int32(18787) # 0x00004963
const TYP_MI4964                   = Int32(18788) # 0x00004964

const TYP_MC4911                   = Int32(84241) # 0x00014911
const TYP_MC4912                   = Int32(84242) # 0x00014912
const TYP_MC4931                   = Int32(84273) # 0x00014931
const TYP_MC4932                   = Int32(84274) # 0x00014932
const TYP_MC4960                   = Int32(84320) # 0x00014960
const TYP_MC4961                   = Int32(84321) # 0x00014961
const TYP_MC4963                   = Int32(84323) # 0x00014963
const TYP_MC4964                   = Int32(84324) # 0x00014964

const TYP_MX4911                   = Int32(149777) # 0x00024911
const TYP_MX4931                   = Int32(149809) # 0x00024931
const TYP_MX4960                   = Int32(149856) # 0x00024960
const TYP_MX4963                   = Int32(149859) # 0x00024963

const TYP_M2I4911                  = Int32(215313) # 0x00034911
const TYP_M2I4912                  = Int32(215314) # 0x00034912
const TYP_M2I4931                  = Int32(215345) # 0x00034931
const TYP_M2I4932                  = Int32(215346) # 0x00034932
const TYP_M2I4960                  = Int32(215392) # 0x00034960
const TYP_M2I4961                  = Int32(215393) # 0x00034961
const TYP_M2I4963                  = Int32(215395) # 0x00034963
const TYP_M2I4964                  = Int32(215396) # 0x00034964

const TYP_M2I4911EXP               = Int32(280849) # 0x00044911
const TYP_M2I4912EXP               = Int32(280850) # 0x00044912
const TYP_M2I4931EXP               = Int32(280881) # 0x00044931
const TYP_M2I4932EXP               = Int32(280882) # 0x00044932
const TYP_M2I4960EXP               = Int32(280928) # 0x00044960
const TYP_M2I4961EXP               = Int32(280929) # 0x00044961
const TYP_M2I4963EXP               = Int32(280931) # 0x00044963
const TYP_M2I4964EXP               = Int32(280932) # 0x00044964

# ----- M2p.59xx-x4 -----
const TYP_M2P59XX_X4               = Int32(612608) # 0x00095900      # generic
const TYP_M2P5911_X4               = Int32(612625) # 0x00095911
const TYP_M2P5912_X4               = Int32(612626) # 0x00095912
const TYP_M2P5913_X4               = Int32(612627) # 0x00095913
const TYP_M2P5916_X4               = Int32(612630) # 0x00095916
const TYP_M2P5920_X4               = Int32(612640) # 0x00095920
const TYP_M2P5921_X4               = Int32(612641) # 0x00095921
const TYP_M2P5922_X4               = Int32(612642) # 0x00095922
const TYP_M2P5923_X4               = Int32(612643) # 0x00095923
const TYP_M2P5926_X4               = Int32(612646) # 0x00095926
const TYP_M2P5930_X4               = Int32(612656) # 0x00095930
const TYP_M2P5931_X4               = Int32(612657) # 0x00095931
const TYP_M2P5932_X4               = Int32(612658) # 0x00095932
const TYP_M2P5933_X4               = Int32(612659) # 0x00095933
const TYP_M2P5936_X4               = Int32(612662) # 0x00095936
const TYP_M2P5940_X4               = Int32(612672) # 0x00095940
const TYP_M2P5941_X4               = Int32(612673) # 0x00095941
const TYP_M2P5942_X4               = Int32(612674) # 0x00095942
const TYP_M2P5943_X4               = Int32(612675) # 0x00095943
const TYP_M2P5946_X4               = Int32(612678) # 0x00095946
const TYP_M2P5960_X4               = Int32(612704) # 0x00095960
const TYP_M2P5961_X4               = Int32(612705) # 0x00095961
const TYP_M2P5962_X4               = Int32(612706) # 0x00095962
const TYP_M2P5966_X4               = Int32(612710) # 0x00095966
const TYP_M2P5968_X4               = Int32(612712) # 0x00095968


# ----- MI.60xx, MC.60xx, MX.60xx -----
const TYP_MI6010                   = Int32(24592) # 0x00006010
const TYP_MI6011                   = Int32(24593) # 0x00006011
const TYP_MI6012                   = Int32(24594) # 0x00006012
const TYP_MI6021                   = Int32(24609) # 0x00006021
const TYP_MI6022                   = Int32(24610) # 0x00006022
const TYP_MI6030                   = Int32(24624) # 0x00006030
const TYP_MI6031                   = Int32(24625) # 0x00006031
const TYP_MI6033                   = Int32(24627) # 0x00006033
const TYP_MI6034                   = Int32(24628) # 0x00006034

const TYP_M2I6010                  = Int32(221200) # 0x00036010
const TYP_M2I6011                  = Int32(221201) # 0x00036011
const TYP_M2I6012                  = Int32(221202) # 0x00036012
const TYP_M2I6021                  = Int32(221217) # 0x00036021
const TYP_M2I6022                  = Int32(221218) # 0x00036022
const TYP_M2I6030                  = Int32(221232) # 0x00036030
const TYP_M2I6031                  = Int32(221233) # 0x00036031
const TYP_M2I6033                  = Int32(221235) # 0x00036033
const TYP_M2I6034                  = Int32(221236) # 0x00036034

const TYP_M2I6010EXP               = Int32(286736) # 0x00046010
const TYP_M2I6011EXP               = Int32(286737) # 0x00046011
const TYP_M2I6012EXP               = Int32(286738) # 0x00046012
const TYP_M2I6021EXP               = Int32(286753) # 0x00046021
const TYP_M2I6022EXP               = Int32(286754) # 0x00046022
const TYP_M2I6030EXP               = Int32(286768) # 0x00046030
const TYP_M2I6031EXP               = Int32(286769) # 0x00046031
const TYP_M2I6033EXP               = Int32(286771) # 0x00046033
const TYP_M2I6034EXP               = Int32(286772) # 0x00046034

const TYP_MC6010                   = Int32(90128) # 0x00016010
const TYP_MC6011                   = Int32(90129) # 0x00016011
const TYP_MC6012                   = Int32(90130) # 0x00016012
const TYP_MC6021                   = Int32(90145) # 0x00016021
const TYP_MC6022                   = Int32(90146) # 0x00016022
const TYP_MC6030                   = Int32(90160) # 0x00016030
const TYP_MC6031                   = Int32(90161) # 0x00016031
const TYP_MC6033                   = Int32(90163) # 0x00016033
const TYP_MC6034                   = Int32(90164) # 0x00016034

const TYP_MX6010                   = Int32(155664) # 0x00026010
const TYP_MX6011                   = Int32(155665) # 0x00026011
const TYP_MX6021                   = Int32(155681) # 0x00026021
const TYP_MX6030                   = Int32(155696) # 0x00026030
const TYP_MX6033                   = Int32(155699) # 0x00026033



# ----- MI.61xx, MC.61xx, MX.61xx -----
const TYP_MI6105                   = Int32(24837) # 0x00006105
const TYP_MI6110                   = Int32(24848) # 0x00006110
const TYP_MI6111                   = Int32(24849) # 0x00006111

const TYP_M2I6105                  = Int32(221445) # 0x00036105
const TYP_M2I6110                  = Int32(221456) # 0x00036110
const TYP_M2I6111                  = Int32(221457) # 0x00036111

const TYP_M2I6105EXP               = Int32(286981) # 0x00046105
const TYP_M2I6110EXP               = Int32(286992) # 0x00046110
const TYP_M2I6111EXP               = Int32(286993) # 0x00046111

const TYP_MC6110                   = Int32(90384) # 0x00016110
const TYP_MC6111                   = Int32(90385) # 0x00016111

const TYP_MX6110                   = Int32(155920) # 0x00026110

# ----- M2p.65xx-x4 -----
const TYP_M2P65XX_X4               = Int32(615680) # 0x00096500      # generic
const TYP_M2P6522_X4               = Int32(615714) # 0x00096522      # 4 ch @   40 MS/s (1x4) (low voltage)
const TYP_M2P6523_X4               = Int32(615715) # 0x00096523      # 8 ch @   40 MS/s (low voltage)
const TYP_M2P6530_X4               = Int32(615728) # 0x00096530      # 1 ch @   40 MS/s
const TYP_M2P6531_X4               = Int32(615729) # 0x00096531      # 2 ch @   40 MS/s
const TYP_M2P6532_X4               = Int32(615730) # 0x00096532      # 4 ch @   40 MS/s (1x4)
const TYP_M2P6536_X4               = Int32(615734) # 0x00096536      # 4 ch @   40 MS/s (2x2)
const TYP_M2P6533_X4               = Int32(615731) # 0x00096533      # 8 ch @   40 MS/s
const TYP_M2P6540_X4               = Int32(615744) # 0x00096540      # 1 ch @   40 MS/s (high voltage)
const TYP_M2P6541_X4               = Int32(615745) # 0x00096541      # 2 ch @   40 MS/s (high voltage)
const TYP_M2P6546_X4               = Int32(615750) # 0x00096546      # 4 ch @   40 MS/s (2x2) (high voltage)
const TYP_M2P6560_X4               = Int32(615776) # 0x00096560      # 1 ch @  125 MS/s
const TYP_M2P6561_X4               = Int32(615777) # 0x00096561      # 2 ch @  125 MS/s
const TYP_M2P6562_X4               = Int32(615778) # 0x00096562      # 4 ch @  125 MS/s (1x4)
const TYP_M2P6566_X4               = Int32(615782) # 0x00096566      # 4 ch @  125 MS/s (2x2)
const TYP_M2P6568_X4               = Int32(615784) # 0x00096568      # 8 ch @  125/80 MS/s
const TYP_M2P6570_X4               = Int32(615792) # 0x00096570      # 1 ch @  125 MS/s (high voltage)
const TYP_M2P6571_X4               = Int32(615793) # 0x00096571      # 2 ch @  125 MS/s (high voltage)
const TYP_M2P6576_X4               = Int32(615798) # 0x00096576      # 4 ch @  125 MS/s (2x2) (high voltage)

# ----- M4i.66xx-x8 (16 bit D/A) -----
# ----- M4i.66xx-x8 (generic) -----
const TYP_M4I66XX_X8               = Int32(484864) # 0x00076600

const TYP_M4I6620_X8               = Int32(484896) # 0x00076620      # 1 ch @  625 MS/s
const TYP_M4I6621_X8               = Int32(484897) # 0x00076621      # 2 ch @  625 MS/s
const TYP_M4I6622_X8               = Int32(484898) # 0x00076622      # 4 ch @  625 MS/s
const TYP_M4I6630_X8               = Int32(484912) # 0x00076630      # 1 ch @ 1250 MS/s
const TYP_M4I6631_X8               = Int32(484913) # 0x00076631      # 2 ch @ 1250 MS/s

# ----- M4x.66xx-x8 (16 bit D/A) -----
# ----- M4x.66xx-x8 (generic) -----
const TYP_M4X66XX_X4               = Int32(550400) # 0x00086600

const TYP_M4X6620_X4               = Int32(550432) # 0x00086620      # 1 ch @  625 MS/s
const TYP_M4X6621_X4               = Int32(550433) # 0x00086621      # 2 ch @  625 MS/s
const TYP_M4X6622_X4               = Int32(550434) # 0x00086622      # 4 ch @  625 MS/s
const TYP_M4X6630_X4               = Int32(550448) # 0x00086630      # 1 ch @ 1250 MS/s
const TYP_M4X6631_X4               = Int32(550449) # 0x00086631      # 2 ch @ 1250 MS/s

# ----- MI.70xx, MC.70xx, MX.70xx -----
const TYP_MI7005                   = Int32(28677) # 0x00007005
const TYP_MI7010                   = Int32(28688) # 0x00007010
const TYP_MI7011                   = Int32(28689) # 0x00007011
const TYP_MI7020                   = Int32(28704) # 0x00007020
const TYP_MI7021                   = Int32(28705) # 0x00007021

const TYP_M2I7005                  = Int32(225285) # 0x00037005
const TYP_M2I7010                  = Int32(225296) # 0x00037010
const TYP_M2I7011                  = Int32(225297) # 0x00037011
const TYP_M2I7020                  = Int32(225312) # 0x00037020
const TYP_M2I7021                  = Int32(225313) # 0x00037021

const TYP_M2I7005EXP               = Int32(290821) # 0x00047005
const TYP_M2I7010EXP               = Int32(290832) # 0x00047010
const TYP_M2I7011EXP               = Int32(290833) # 0x00047011
const TYP_M2I7020EXP               = Int32(290848) # 0x00047020
const TYP_M2I7021EXP               = Int32(290849) # 0x00047021

const TYP_MC7005                   = Int32(94213) # 0x00017005
const TYP_MC7010                   = Int32(94224) # 0x00017010
const TYP_MC7011                   = Int32(94225) # 0x00017011
const TYP_MC7020                   = Int32(94240) # 0x00017020
const TYP_MC7021                   = Int32(94241) # 0x00017021

const TYP_MX7005                   = Int32(159749) # 0x00027005
const TYP_MX7010                   = Int32(159760) # 0x00027010
const TYP_MX7011                   = Int32(159761) # 0x00027011



# ----- MI.72xx, MC.72xx, MX.72xx -----
const TYP_MI7210                   = Int32(29200) # 0x00007210
const TYP_MI7211                   = Int32(29201) # 0x00007211
const TYP_MI7220                   = Int32(29216) # 0x00007220
const TYP_MI7221                   = Int32(29217) # 0x00007221

const TYP_M2I7210                  = Int32(225808) # 0x00037210
const TYP_M2I7211                  = Int32(225809) # 0x00037211
const TYP_M2I7220                  = Int32(225824) # 0x00037220
const TYP_M2I7221                  = Int32(225825) # 0x00037221

const TYP_M2I7210EXP               = Int32(291344) # 0x00047210
const TYP_M2I7211EXP               = Int32(291345) # 0x00047211
const TYP_M2I7220EXP               = Int32(291360) # 0x00047220
const TYP_M2I7221EXP               = Int32(291361) # 0x00047221

const TYP_MC7210                   = Int32(94736) # 0x00017210
const TYP_MC7211                   = Int32(94737) # 0x00017211
const TYP_MC7220                   = Int32(94752) # 0x00017220
const TYP_MC7221                   = Int32(94753) # 0x00017221

const TYP_MX7210                   = Int32(160272) # 0x00027210
const TYP_MX7220                   = Int32(160288) # 0x00027220

# ----- M2p.75xx-x4 -----
const TYP_M2P75XX_X4               = Int32(619776) # 0x00097500      # generic
const TYP_M2P7515_X4               = Int32(619797) # 0x00097515

# ----- M4i.77xx-x8  -----
const TYP_M4I77XX_X8               = Int32(489216) # 0x00077700 # generic
const TYP_M4I7710_X8               = Int32(489232) # 0x00077710 # single-ended
const TYP_M4I7720_X8               = Int32(489248) # 0x00077720 # single-ended
const TYP_M4I7730_X8               = Int32(489264) # 0x00077730 # single-ended
const TYP_M4I7725_X8               = Int32(489253) # 0x00077725 # differential
const TYP_M4I7735_X8               = Int32(489269) # 0x00077735 # differential

# ----- M4x.77xx-x8  -----
const TYP_M4X77XX_X4               = Int32(554752) # 0x00087700 # generic
const TYP_M4X7710_X4               = Int32(554768) # 0x00087710 # single-ended
const TYP_M4X7720_X4               = Int32(554784) # 0x00087720 # single-ended
const TYP_M4X7730_X4               = Int32(554800) # 0x00087730 # single-ended
const TYP_M4X7725_X4               = Int32(554789) # 0x00087725 # differential
const TYP_M4X7735_X4               = Int32(554805) # 0x00087735 # differential

# ----- MX.90xx -----
const TYP_MX9010                   = Int32(167952) # 0x00029010



# ***********************************************************************
# software registers
# ***********************************************************************


# ***** PCI Features Bits (MI/MC/MX and prior cards) *********
const PCIBIT_MULTI                 = Int32(1) # 0x00000001
const PCIBIT_DIGITAL               = Int32(2) # 0x00000002
const PCIBIT_CH0DIGI               = Int32(4) # 0x00000004
const PCIBIT_EXTSAM                = Int32(8) # 0x00000008
const PCIBIT_3CHANNEL              = Int32(16) # 0x00000010
const PCIBIT_GATE                  = Int32(32) # 0x00000020
const PCIBIT_SLAVE                 = Int32(64) # 0x00000040
const PCIBIT_MASTER                = Int32(128) # 0x00000080
const PCIBIT_DOUBLEMEM             = Int32(256) # 0x00000100
const PCIBIT_SYNC                  = Int32(512) # 0x00000200
const PCIBIT_TIMESTAMP             = Int32(1024) # 0x00000400
const PCIBIT_STARHUB               = Int32(2048) # 0x00000800
const PCIBIT_CA                    = Int32(4096) # 0x00001000
const PCIBIT_XIO                   = Int32(8192) # 0x00002000
const PCIBIT_AMPLIFIER             = Int32(16384) # 0x00004000
const PCIBIT_DIFFMODE              = Int32(32768) # 0x00008000

const PCIBIT_ELISA                 = Int32(268435456) # 0x10000000


# ***** PCI features starting with M2i card series *****
const SPCM_FEAT_MULTI              = Int32(1) # 0x00000001      # multiple recording
const SPCM_FEAT_GATE               = Int32(2) # 0x00000002      # gated sampling
const SPCM_FEAT_DIGITAL            = Int32(4) # 0x00000004      # additional synchronous digital inputs or outputs
const SPCM_FEAT_TIMESTAMP          = Int32(8) # 0x00000008      # timestamp
const SPCM_FEAT_STARHUB5           = Int32(32) # 0x00000020      # starhub for  5 cards installed (M2i + M2i-Exp)
const SPCM_FEAT_STARHUB4           = Int32(32) # 0x00000020      # starhub for  4 cards installed (M3i + M3i-Exp)
const SPCM_FEAT_STARHUB6_EXTM      = Int32(32) # 0x00000020      # starhub for  6 cards installed as card extension or piggy back (M2p)
const SPCM_FEAT_STARHUB8_EXTM      = Int32(32) # 0x00000020      # starhub for  8 cards installed as card extension or piggy back (M4i-Exp)
const SPCM_FEAT_STARHUB16          = Int32(64) # 0x00000040      # starhub for 16 cards installed (M2i, M2i-exp)
const SPCM_FEAT_STARHUB16_EXTM     = Int32(64) # 0x00000040      # starhub for 16 cards installed as card extension or piggy back (M2p)
const SPCM_FEAT_STARHUB8           = Int32(64) # 0x00000040      # starhub for  8 cards installed (M3i + M3i-Exp, M5i)
const SPCM_FEAT_STARHUBXX_MASK     = Int32(96) # 0x00000060      # mask to detect one of the above installed starhub
const SPCM_FEAT_ABA                = Int32(128) # 0x00000080      # ABA mode installed
const SPCM_FEAT_BASEXIO            = Int32(256) # 0x00000100      # extra I/O on base card installed
const SPCM_FEAT_AMPLIFIER_10V      = Int32(512) # 0x00000200      # external amplifier for 60/61
const SPCM_FEAT_STARHUBSYSMASTER   = Int32(1024) # 0x00000400      # system starhub master installed
const SPCM_FEAT_DIFFMODE           = Int32(2048) # 0x00000800      # Differential mode installed
const SPCM_FEAT_SEQUENCE           = Int32(4096) # 0x00001000      # Sequence programming mode for generator cards
const SPCM_FEAT_AMPMODULE_10V      = Int32(8192) # 0x00002000      # amplifier module for 60/61
const SPCM_FEAT_STARHUBSYSSLAVE    = Int32(16384) # 0x00004000      # system starhub slave installed
const SPCM_FEAT_NETBOX             = Int32(32768) # 0x00008000      # card is part of netbox
const SPCM_FEAT_REMOTESERVER       = Int32(65536) # 0x00010000      # remote server can be used with this card
const SPCM_FEAT_SCAPP              = Int32(131072) # 0x00020000      # SCAPP option (CUDA RDMA)
const SPCM_FEAT_DIG16_SMB          = Int32(262144) # 0x00040000      # M2p: 16 additional digital inputs or outputs (via SMB connectors) 
const SPCM_FEAT_DIG8_SMA           = Int32(262144) # 0x00040000      # M4i:  8 additional digital inputs or 6 additional outputs (via SMA connectors) 
const SPCM_FEAT_DIG16_FX2          = Int32(524288) # 0x00080000      # M2p: 16 additional digital inputs or outputs (via FX2 connector)
const SPCM_FEAT_DIGITALBWFILTER    = Int32(1048576) # 0x00100000      # Digital BW filter is available
const SPCM_FEAT_CUSTOMMOD_MASK     = Int32(-268435456) # 0xF0000000      # mask for custom modification code, meaning of code depends on type and customer


# ***** Extended Features starting with M4i *****
const SPCM_FEAT_EXTFW_SEGSTAT      = Int32(1) # 0x00000001        # segment (Multiple Recording, ABA) statistics like average, min/max
const SPCM_FEAT_EXTFW_SEGAVERAGE   = Int32(2) # 0x00000002        # average of multiple segments (Multiple Recording, ABA) 
const SPCM_FEAT_EXTFW_BOXCAR       = Int32(4) # 0x00000004        # boxcar averaging (high-res mode)
const SPCM_FEAT_EXTFW_PULSEGEN     = Int32(8) # 0x00000008        # pulse generators on XIO lines available
const SPCM_FEAT_EXTFW_DDS          = Int32(16) # 0x00000010        # DDS available
const SPCM_FEAT_EXTFW_EVALUATION   = Int32(-2147483648) # 0x80000000        # used in combination with other flags to check for evaluation versions


# ***** Error Request *************
const ERRORTEXTLEN                 = Int32(200)
const SPC_LASTERRORTEXT            = Int32(999996)
const SPC_LASTERRORVALUE           = Int32(999997)
const SPC_LASTERRORREG             = Int32(999998)
const SPC_LASTERRORCODE            = Int32(999999)     # Reading this reset the internal error-memory.

# ***** constants to use with the various _ACDC registers *****
const COUPLING_DC  = Int32(0)
const COUPLING_AC  = Int32(1)


# ***** Register and Command Structure
const SPC_COMMAND                  = Int32(0)
const     SPC_RESET                = Int32(0)
const     SPC_SOFTRESET            = Int32(1)
const     SPC_WRITESETUP           = Int32(2)
const     SPC_START                = Int32(10)
const     SPC_STARTANDWAIT         = Int32(11)
const     SPC_FIFOSTART            = Int32(12)
const     SPC_FIFOWAIT             = Int32(13)
const     SPC_FIFOSTARTNOWAIT      = Int32(14)
const     SPC_FORCETRIGGER         = Int32(16)
const     SPC_STOP                 = Int32(20)
const     SPC_FLUSHFIFOBUFFER      = Int32(21)
const     SPC_POWERDOWN            = Int32(30)
const     SPC_SYNCMASTER           = Int32(100)
const     SPC_SYNCTRIGGERMASTER    = Int32(101)
const     SPC_SYNCMASTERFIFO       = Int32(102)
const     SPC_SYNCSLAVE            = Int32(110)
const     SPC_SYNCTRIGGERSLAVE     = Int32(111)
const     SPC_SYNCSLAVEFIFO        = Int32(112)
const     SPC_NOSYNC               = Int32(120)
const     SPC_SYNCSTART            = Int32(130)
const     SPC_SYNCCALCMASTER       = Int32(140)
const     SPC_SYNCCALCMASTERFIFO   = Int32(141)
const     SPC_PXIDIVIDERRESET      = Int32(150)
const     SPC_RELAISON             = Int32(200)
const     SPC_RELAISOFF            = Int32(210)
const     SPC_ADJUSTSTART          = Int32(300)
const     SPC_FIFO_BUFREADY0       = Int32(400)
const     SPC_FIFO_BUFREADY1       = Int32(401)
const     SPC_FIFO_BUFREADY2       = Int32(402)
const     SPC_FIFO_BUFREADY3       = Int32(403)
const     SPC_FIFO_BUFREADY4       = Int32(404)
const     SPC_FIFO_BUFREADY5       = Int32(405)
const     SPC_FIFO_BUFREADY6       = Int32(406)
const     SPC_FIFO_BUFREADY7       = Int32(407)
const     SPC_FIFO_BUFREADY8       = Int32(408)
const     SPC_FIFO_BUFREADY9       = Int32(409)
const     SPC_FIFO_BUFREADY10      = Int32(410)
const     SPC_FIFO_BUFREADY11      = Int32(411)
const     SPC_FIFO_BUFREADY12      = Int32(412)
const     SPC_FIFO_BUFREADY13      = Int32(413)
const     SPC_FIFO_BUFREADY14      = Int32(414)
const     SPC_FIFO_BUFREADY15      = Int32(415)
const     SPC_FIFO_AUTOBUFSTART    = Int32(500)
const     SPC_FIFO_AUTOBUFEND      = Int32(510)

const SPC_STATUS                   = Int32(10)
const     SPC_RUN                  = Int32(0)
const     SPC_TRIGGER              = Int32(10)
const     SPC_READY                = Int32(20)



# commands for M2 cards
const SPC_M2CMD                    = Int32(100)                # write a command
const     M2CMD_CARD_RESET             = Int32(1) # 0x00000001     # hardware reset   
const     M2CMD_CARD_WRITESETUP        = Int32(2) # 0x00000002     # write setup only
const     M2CMD_CARD_START             = Int32(4) # 0x00000004     # start of card (including writesetup)
const     M2CMD_CARD_ENABLETRIGGER     = Int32(8) # 0x00000008     # enable trigger engine
const     M2CMD_CARD_FORCETRIGGER      = Int32(16) # 0x00000010     # force trigger
const     M2CMD_CARD_DISABLETRIGGER    = Int32(32) # 0x00000020     # disable trigger engine again (multi or gate)
const     M2CMD_CARD_STOP              = Int32(64) # 0x00000040     # stop run
const     M2CMD_CARD_FLUSHFIFO         = Int32(128) # 0x00000080     # flush fifos to memory
const     M2CMD_CARD_INVALIDATEDATA    = Int32(256) # 0x00000100     # current data in memory is invalidated, next data transfer start will wait until new data is available
const     M2CMD_CARD_INTERNALRESET     = Int32(512) # 0x00000200     # INTERNAL reset command

const     M2CMD_ALL_STOP               = Int32(4456544) # 0x00440060     # stops card and all running transfers

const     M2CMD_CARD_WAITPREFULL       = Int32(4096) # 0x00001000     # wait until pretrigger is full
const     M2CMD_CARD_WAITTRIGGER       = Int32(8192) # 0x00002000     # wait for trigger recognition
const     M2CMD_CARD_WAITREADY         = Int32(16384) # 0x00004000     # wait for card ready

const     M2CMD_DATA_STARTDMA          = Int32(65536) # 0x00010000     # start of DMA transfer for data
const     M2CMD_DATA_WAITDMA           = Int32(131072) # 0x00020000     # wait for end of data transfer / next block ready
const     M2CMD_DATA_STOPDMA           = Int32(262144) # 0x00040000     # abort the data transfer
const     M2CMD_DATA_POLL              = Int32(524288) # 0x00080000     # transfer data using single access and polling

const     M2CMD_EXTRA_STARTDMA         = Int32(1048576) # 0x00100000     # start of DMA transfer for extra (ABA + timestamp) data
const     M2CMD_EXTRA_WAITDMA          = Int32(2097152) # 0x00200000     # wait for end of extra (ABA + timestamp) data transfer / next block ready
const     M2CMD_EXTRA_STOPDMA          = Int32(4194304) # 0x00400000     # abort the extra (ABA + timestamp) data transfer
const     M2CMD_EXTRA_POLL             = Int32(8388608) # 0x00800000     # transfer data using single access and polling

const     M2CMD_DATA_SGFLUSH           = Int32(16777216) # 0x01000000     # flush incomplete pages from sg list


# status for M2 cards (bitmask)
const SPC_M2STATUS                 = Int32(110)                # read the current status
const     M2STAT_NONE                  = Int32(0) # 0x00000000     # status empty
const     M2STAT_CARD_PRETRIGGER       = Int32(1) # 0x00000001     # pretrigger area is full
const     M2STAT_CARD_TRIGGER          = Int32(2) # 0x00000002     # trigger recognized
const     M2STAT_CARD_READY            = Int32(4) # 0x00000004     # card is ready, run finished
const     M2STAT_CARD_SEGMENT_PRETRG   = Int32(8) # 0x00000008     # since M4i: at muliple-recording: pretrigger area of a segment is full

const     M2STAT_DATA_BLOCKREADY       = Int32(256) # 0x00000100     # next data block is available
const     M2STAT_DATA_END              = Int32(512) # 0x00000200     # data transfer has ended
const     M2STAT_DATA_OVERRUN          = Int32(1024) # 0x00000400     # FIFO overrun (record) or underrun (replay)
const     M2STAT_DATA_ERROR            = Int32(2048) # 0x00000800     # internal error

const     M2STAT_EXTRA_BLOCKREADY      = Int32(4096) # 0x00001000     # next extra data (ABA and timestamp) block is available
const     M2STAT_EXTRA_END             = Int32(8192) # 0x00002000     # extra data (ABA and timestamp) transfer has ended
const     M2STAT_EXTRA_OVERRUN         = Int32(16384) # 0x00004000     # FIFO overrun
const     M2STAT_EXTRA_ERROR           = Int32(32768) # 0x00008000     # internal error

const     M2STAT_TSCNT_OVERRUN         = Int32(65536) # 0x00010000     # timestamp counter overrun

const     M2STAT_INTERNALMASK          = Int32(-16777216) # 0xff000000     # mask for internal status signals
const     M2STAT_INTERNAL_SYSLOCK      = Int32(33554432) # 0x02000000

const SPC_REGISTER_LIST            = Int32(120)


# buffer control registers for samples data
const SPC_DATA_AVAIL_USER_LEN      = Int32(200)                # number of bytes available for user (valid data if READ, free buffer if WRITE)
const SPC_DATA_AVAIL_USER_POS      = Int32(201)                # the current byte position where the available user data starts
const SPC_DATA_AVAIL_CARD_LEN      = Int32(202)                # number of bytes available for card (free buffer if READ, filled data if WRITE)
const SPC_DATA_OUTBUFSIZE          = Int32(209)                # output buffer size in bytes

# buffer control registers for extra data (ABA slow data, timestamps)
const SPC_ABA_AVAIL_USER_LEN       = Int32(210)                # number of bytes available for user (valid data if READ, free buffer if WRITE)
const SPC_ABA_AVAIL_USER_POS       = Int32(211)                # the current byte position where the available user data starts
const SPC_ABA_AVAIL_CARD_LEN       = Int32(212)                # number of bytes available for card (free buffer if READ, filled data if WRITE)

const SPC_TS_AVAIL_USER_LEN        = Int32(220)                # number of bytes available for user (valid data if READ, free buffer if WRITE)
const SPC_TS_AVAIL_USER_POS        = Int32(221)                # the current byte position where the available user data starts
const SPC_TS_AVAIL_CARD_LEN        = Int32(222)                # number of bytes available for card (free buffer if READ, filled data if WRITE)



# Installation
const SPC_VERSION                  = Int32(1000)
const SPC_ISAADR                   = Int32(1010)
const SPC_INSTMEM                  = Int32(1020)
const SPC_INSTSAMPLERATE           = Int32(1030)
const SPC_BRDTYP                   = Int32(1040)

# MI/MC/MX type information (internal use)
const SPC_MIINST_MODULES           = Int32(1100)
const SPC_MIINST_CHPERMODULE       = Int32(1110)
const SPC_MIINST_BYTESPERSAMPLE    = Int32(1120)
const SPC_MIINST_BITSPERSAMPLE     = Int32(1125)
const SPC_MIINST_MAXADCVALUE       = Int32(1126)
const SPC_MIINST_MINADCLOCK        = Int32(1130)
const SPC_MIINST_MAXADCLOCK        = Int32(1140)
const SPC_MIINST_MINEXTCLOCK       = Int32(1145)
const SPC_MIINST_MAXEXTCLOCK       = Int32(1146)
const SPC_MIINST_MINSYNCCLOCK      = Int32(1147)
const SPC_MIINST_MINEXTREFCLOCK    = Int32(1148)
const SPC_MIINST_MAXEXTREFCLOCK    = Int32(1149)
const SPC_MIINST_QUARZ             = Int32(1150)
const SPC_MIINST_QUARZ2            = Int32(1151)
const SPC_MIINST_MINEXTCLOCK1      = Int32(1152)
const SPC_MIINST_FLAGS             = Int32(1160)
const SPC_MIINST_FIFOSUPPORT       = Int32(1170)
const SPC_MIINST_ISDEMOCARD        = Int32(1175)

# Driver information
const SPC_GETDRVVERSION            = Int32(1200)
const SPC_GETKERNELVERSION         = Int32(1210)
const SPC_GETDRVTYPE               = Int32(1220)
const     DRVTYP_DOS               = Int32(0)
const     DRVTYP_LINUX32           = Int32(1)
const     DRVTYP_VXD               = Int32(2)
const     DRVTYP_NTLEGACY          = Int32(3)
const     DRVTYP_WDM32             = Int32(4)
const     DRVTYP_WDM64             = Int32(5)
const     DRVTYP_WOW64             = Int32(6)
const     DRVTYP_LINUX64           = Int32(7)
const     DRVTYP_QNX32             = Int32(8)
const     DRVTYP_QNX64             = Int32(9)
const SPC_GETCOMPATIBILITYVERSION  = Int32(1230)
const SPC_GETMINDRVVERSION         = Int32(1240)

# PCI, CompactPCI and PXI Installation Information
const SPC_PCITYP                   = Int32(2000)

# ***** available card function types *****
const SPC_FNCTYPE                  = Int32(2001)
const     SPCM_TYPE_AI             = Int32(1) # 0x01
const     SPCM_TYPE_AO             = Int32(2) # 0x02
const     SPCM_TYPE_DI             = Int32(4) # 0x04
const     SPCM_TYPE_DO             = Int32(8) # 0x08
const     SPCM_TYPE_DIO            = Int32(16) # 0x10

const SPC_PCIVERSION               = Int32(2010)
const SPC_PCIEXTVERSION            = Int32(2011)
const SPC_PCIMODULEVERSION         = Int32(2012)
const SPC_PCIMODULEBVERSION        = Int32(2013)
const SPC_BASEPCBVERSION           = Int32(2014)
const SPC_MODULEPCBVERSION         = Int32(2015)
const SPC_MODULEAPCBVERSION        = Int32(2015)
const SPC_MODULEBPCBVERSION        = Int32(2016)
const SPC_EXTPCBVERSION            = Int32(2017)
const SPC_PCIDIGVERSION            = Int32(2018)
const SPC_DIGPCBVERSION            = Int32(2019)
const SPC_PCIDATE                  = Int32(2020)
const SPC_CALIBDATE                = Int32(2025)
const SPC_CALIBDATEONBOARD         = Int32(2026)
const SPC_PCISERIALNR              = Int32(2030)
const SPC_PCISERIALNO              = Int32(2030)
const SPC_PCIHWBUSNO               = Int32(2040)
const SPC_PCIHWDEVNO               = Int32(2041)
const SPC_PCIHWFNCNO               = Int32(2042)
const SPC_PCIHWSLOTNO              = Int32(2043)
const SPC_PCIEXPGENERATION         = Int32(2050)
const SPC_PCIEXPLANES              = Int32(2051)
const SPC_PCIEXPPAYLOAD            = Int32(2052)
const SPC_PCIEXPREADREQUESTSIZE    = Int32(2053)
const SPC_PCIEXPREADCOMPLBOUNDARY  = Int32(2054)
const SPC_PXIHWSLOTNO              = Int32(2055)
const SPC_PCISAMPLERATE            = Int32(2100)
const SPC_PCIMEMSIZE               = Int32(2110)
const SPC_PCIFEATURES              = Int32(2120)
const SPC_PCIEXTFEATURES           = Int32(2121)
const SPC_PCIINFOADR               = Int32(2200)
const SPC_PCIINTERRUPT             = Int32(2300)
const SPC_PCIBASEADR0              = Int32(2400)
const SPC_PCIBASEADR1              = Int32(2401)
const SPC_PCIREGION0               = Int32(2410)
const SPC_PCIREGION1               = Int32(2411)
const SPC_READTRGLVLCOUNT          = Int32(2500)
const SPC_READIRCOUNT              = Int32(3000)
const SPC_READUNIPOLAR0            = Int32(3010)
const SPC_READUNIPOLAR1            = Int32(3020)
const SPC_READUNIPOLAR2            = Int32(3030)
const SPC_READUNIPOLAR3            = Int32(3040)
const SPC_READMAXOFFSET            = Int32(3100)

const SPC_READAIFEATURES           = Int32(3101)
const     SPCM_AI_TERM             = Int32(1) # 0x00000001  # input termination available
const     SPCM_AI_SE               = Int32(2) # 0x00000002  # single-ended mode available
const     SPCM_AI_DIFF             = Int32(4) # 0x00000004  # differential mode available
const     SPCM_AI_OFFSPERCENT      = Int32(8) # 0x00000008  # offset programming is done in percent of input range
const     SPCM_AI_OFFSMV           = Int32(16) # 0x00000010  # offset programming is done in mV absolut
const     SPCM_AI_OVERRANGEDETECT  = Int32(32) # 0x00000020  # overrange detection is programmable
const     SPCM_AI_DCCOUPLING       = Int32(64) # 0x00000040  # DC coupling available
const     SPCM_AI_ACCOUPLING       = Int32(128) # 0x00000080  # AC coupling available
const     SPCM_AI_LOWPASS          = Int32(256) # 0x00000100  # selectable low pass
const     SPCM_AI_ACDC_OFFS_COMP   = Int32(512) # 0x00000200  # AC/DC offset compensation
const     SPCM_AI_DIFFMUX          = Int32(1024) # 0x00000400  # differential mode (two channels combined to one) available
const     SPCM_AI_GLOBALLOWPASS    = Int32(2048) # 0x00000800  # globally selectable low pass (all channels same setting)
const     SPCM_AI_AUTOCALOFFS      = Int32(4096) # 0x00001000  # automatic offset calibration in hardware
const     SPCM_AI_AUTOCALGAIN      = Int32(8192) # 0x00002000  # automatic gain calibration in hardware
const     SPCM_AI_AUTOCALOFFSNOIN  = Int32(16384) # 0x00004000  # automatic offset calibration with open inputs
const     SPCM_AI_HIGHIMP          = Int32(32768) # 0x00008000  # high impedance available
const     SPCM_AI_LOWIMP           = Int32(65536) # 0x00010000  # low impedance available (50 ohm)
const     SPCM_AI_DIGITALLOWPASS   = Int32(131072) # 0x00020000  # selectable digital low pass filter
const     SPCM_AI_INDIVPULSEWIDTH  = Int32(1048576) # 0x00100000  # individual pulsewidth per channel available

const SPC_READAOFEATURES           = Int32(3102)
const     SPCM_AO_SE               = Int32(2) # 0x00000002  # single-ended mode available
const     SPCM_AO_DIFF             = Int32(4) # 0x00000004  # differential mode available
const     SPCM_AO_PROGFILTER       = Int32(8) # 0x00000008  # programmable filters available
const     SPCM_AO_PROGOFFSET       = Int32(16) # 0x00000010  # programmable offset available
const     SPCM_AO_PROGGAIN         = Int32(32) # 0x00000020  # programmable gain available
const     SPCM_AO_PROGSTOPLEVEL    = Int32(64) # 0x00000040  # programmable stop level available
const     SPCM_AO_DOUBLEOUT        = Int32(128) # 0x00000080  # double out mode available
const     SPCM_AO_ENABLEOUT        = Int32(256) # 0x00000100  # outputs can be disabled/enabled

const SPC_READDIFEATURES           = Int32(3103)
const     SPCM_DI_TERM             = Int32(1) # 0x00000001  # input termination available
const     SPCM_DI_SE               = Int32(2) # 0x00000002  # single-ended mode available
const     SPCM_DI_DIFF             = Int32(4) # 0x00000004  # differential mode available
const     SPCM_DI_PROGTHRESHOLD    = Int32(8) # 0x00000008  # programmable threshold available
const     SPCM_DI_HIGHIMP          = Int32(16) # 0x00000010  # high impedance available
const     SPCM_DI_LOWIMP           = Int32(32) # 0x00000020  # low impedance available
const     SPCM_DI_INDIVPULSEWIDTH  = Int32(1048576) # 0x00100000  # individual pulsewidth per channel available
const     SPCM_DI_IOCHANNEL        = Int32(2097152) # 0x00200000  # connected with DO channel

const SPC_READDOFEATURES           = Int32(3104)
const     SPCM_DO_SE               = Int32(2) # 0x00000002  # single-ended mode available
const     SPCM_DO_DIFF             = Int32(4) # 0x00000004  # differential mode available
const     SPCM_DO_PROGSTOPLEVEL    = Int32(8) # 0x00000008  # programmable stop level available
const     SPCM_DO_PROGOUTLEVELS    = Int32(16) # 0x00000010  # programmable output levels (low + high) available
const     SPCM_DO_ENABLEMASK       = Int32(32) # 0x00000020  # individual enable mask for each output channel
const     SPCM_DO_IOCHANNEL        = Int32(2097152) # 0x00200000  # connected with DI channel

const SPC_READCHGROUPING           = Int32(3110)
const SPC_READAIPATHCOUNT          = Int32(3120)       # number of available analog input paths
const SPC_READAIPATH               = Int32(3121)       # the current path for which all the settings are read

const SPCM_CUSTOMMOD               = Int32(3130)
const     SPCM_CUSTOMMOD_BASE_MASK     = Int32(255) # 0x000000FF
const     SPCM_CUSTOMMOD_MODULE_MASK   = Int32(65280) # 0x0000FF00
const     SPCM_CUSTOMMOD_STARHUB_MASK  = Int32(16711680) # 0x00FF0000

const SPC_READRANGECH0_0           = Int32(3200)
const SPC_READRANGECH0_1           = Int32(3201)
const SPC_READRANGECH0_2           = Int32(3202)
const SPC_READRANGECH0_3           = Int32(3203)
const SPC_READRANGECH0_4           = Int32(3204)
const SPC_READRANGECH0_5           = Int32(3205)
const SPC_READRANGECH0_6           = Int32(3206)
const SPC_READRANGECH0_7           = Int32(3207)
const SPC_READRANGECH0_8           = Int32(3208)
const SPC_READRANGECH0_9           = Int32(3209)
const SPC_READRANGECH1_0           = Int32(3300)
const SPC_READRANGECH1_1           = Int32(3301)
const SPC_READRANGECH1_2           = Int32(3302)
const SPC_READRANGECH1_3           = Int32(3303)
const SPC_READRANGECH1_4           = Int32(3304)
const SPC_READRANGECH1_5           = Int32(3305)
const SPC_READRANGECH1_6           = Int32(3306)
const SPC_READRANGECH1_7           = Int32(3307)
const SPC_READRANGECH1_8           = Int32(3308)
const SPC_READRANGECH1_9           = Int32(3309)
const SPC_READRANGECH2_0           = Int32(3400)
const SPC_READRANGECH2_1           = Int32(3401)
const SPC_READRANGECH2_2           = Int32(3402)
const SPC_READRANGECH2_3           = Int32(3403)
const SPC_READRANGECH3_0           = Int32(3500)
const SPC_READRANGECH3_1           = Int32(3501)
const SPC_READRANGECH3_2           = Int32(3502)
const SPC_READRANGECH3_3           = Int32(3503)

const SPC_READRANGEMIN0            = Int32(4000)
const SPC_READRANGEMIN99           = Int32(4099)
const SPC_READRANGEMAX0            = Int32(4100)
const SPC_READRANGEMAX99           = Int32(4199)
const SPC_READOFFSMIN0             = Int32(4200)
const SPC_READOFFSMIN99            = Int32(4299)
const SPC_READOFFSMAX0             = Int32(4300)
const SPC_READOFFSMAX99            = Int32(4399)
const SPC_PCICOUNTER               = Int32(9000)
const SPC_BUFFERPOS                = Int32(9010)

const SPC_READAOGAINMIN            = Int32(9100)
const SPC_READAOGAINMAX            = Int32(9110)
const SPC_READAOOFFSETMIN          = Int32(9120)
const SPC_READAOOFFSETMAX          = Int32(9130)

const SPC_CARDMODE                 = Int32(9500)       # card modes as listed below
const SPC_AVAILCARDMODES           = Int32(9501)       # list with available card modes

# card modes
const     SPC_REC_STD_SINGLE           = Int32(1) # 0x00000001  # singleshot recording to memory
const     SPC_REC_STD_MULTI            = Int32(2) # 0x00000002  # multiple records to memory on each trigger event
const     SPC_REC_STD_GATE             = Int32(4) # 0x00000004  # gated recording to memory on gate signal
const     SPC_REC_STD_ABA              = Int32(8) # 0x00000008  # ABA: A slowly to extra FIFO, B to memory on each trigger event 
const     SPC_REC_STD_SEGSTATS         = Int32(65536) # 0x00010000  # segment information stored on each trigger segment -> stored in on-board memory
const     SPC_REC_STD_AVERAGE          = Int32(131072) # 0x00020000  # multiple records summed to average memory on each trigger event -> stored in on-board memory
const     SPC_REC_STD_AVERAGE_16BIT    = Int32(524288) # 0x00080000  # multiple records summed to average memory on each trigger event -> stored in on-board memory
const     SPC_REC_STD_BOXCAR           = Int32(8388608) # 0x00800000  # boxcar averaging

const     SPC_REC_FIFO_SINGLE          = Int32(16) # 0x00000010  # singleshot to FIFO on trigger event
const     SPC_REC_FIFO_MULTI           = Int32(32) # 0x00000020  # multiple records to FIFO on each trigger event
const     SPC_REC_FIFO_GATE            = Int32(64) # 0x00000040  # gated sampling to FIFO on gate signal
const     SPC_REC_FIFO_ABA             = Int32(128) # 0x00000080  # ABA: A slowly to extra FIFO, B to FIFO on each trigger event
const     SPC_REC_FIFO_SEGSTATS        = Int32(1048576) # 0x00100000  # segment information stored on each trigger segment -> streamed to host
const     SPC_REC_FIFO_AVERAGE         = Int32(2097152) # 0x00200000  # multiple records summed to average memory on each trigger event -> streamed to host
const     SPC_REC_FIFO_AVERAGE_16BIT   = Int32(4194304) # 0x00400000  # multiple records summed to average memory on each trigger event -> streamed to host
const     SPC_REC_FIFO_BOXCAR          = Int32(16777216) # 0x01000000  # boxcar averaging FIFO mode
const     SPC_REC_FIFO_SINGLE_MONITOR  = Int32(33554432) # 0x02000000  # like SPC_REC_FIFO_SINGLE but with additional slow A data stream for monitoring

const     SPC_REP_STD_SINGLE           = Int32(256) # 0x00000100  # single replay from memory on trigger event 
const     SPC_REP_STD_MULTI            = Int32(512) # 0x00000200  # multiple replay from memory on each trigger event
const     SPC_REP_STD_GATE             = Int32(1024) # 0x00000400  # gated replay from memory on gate signal

const     SPC_REP_FIFO_SINGLE          = Int32(2048) # 0x00000800  # single replay from FIFO on trigger event
const     SPC_REP_FIFO_MULTI           = Int32(4096) # 0x00001000  # multiple replay from FIFO on each trigger event
const     SPC_REP_FIFO_GATE            = Int32(8192) # 0x00002000  # gated replay from FIFO on gate signal

const     SPC_REP_STD_CONTINUOUS       = Int32(16384) # 0x00004000  # continuous replay started by one trigger event
const     SPC_REP_STD_SINGLERESTART    = Int32(32768) # 0x00008000  # single replays on every detected trigger event
const     SPC_REP_STD_SEQUENCE         = Int32(262144) # 0x00040000  # sequence mode replay
const     SPC_REP_STD_DDS              = Int32(67108864) # 0x04000000  # DDS replay

# Waveforms for demo cards
const SPC_DEMOWAVEFORM             = Int32(9600)
const SPC_AVAILDEMOWAVEFORMS       = Int32(9601)
const     SPCM_DEMOWAVEFORM_SINE       = Int32(1) # 0x00000001
const     SPCM_DEMOWAVEFORM_RECT       = Int32(2) # 0x00000002
const     SPCM_DEMOWAVEFORM_TRIANGLE   = Int32(4) # 0x00000004


# Memory
const SPC_MEMSIZE                  = Int32(10000)
const SPC_SEGMENTSIZE              = Int32(10010)
const SPC_LOOPS                    = Int32(10020)
const SPC_PRETRIGGER               = Int32(10030)
const SPC_ABADIVIDER               = Int32(10040)
const SPC_AVERAGES                 = Int32(10050)
const SPC_BOX_AVERAGES             = Int32(10060)
const SPC_SEGSPLIT_START           = Int32(10070)
const SPC_SEGSPLIT_PAUSE           = Int32(10071)
const SPC_POSTTRIGGER              = Int32(10100)
const SPC_STARTOFFSET              = Int32(10200)

# Memory info (depends on mode and channelenable)
const SPC_AVAILMEMSIZE_MIN         = Int32(10201)
const SPC_AVAILMEMSIZE_MAX         = Int32(10202)
const SPC_AVAILMEMSIZE_STEP        = Int32(10203)
const SPC_AVAILPOSTTRIGGER_MIN     = Int32(10204)
const SPC_AVAILPOSTTRIGGER_MAX     = Int32(10205)
const SPC_AVAILPOSTTRIGGER_STEP    = Int32(10206)

const SPC_AVAILABADIVIDER_MIN      = Int32(10207)
const SPC_AVAILABADIVIDER_MAX      = Int32(10208)
const SPC_AVAILABADIVIDER_STEP     = Int32(10209)

const SPC_AVAILLOOPS_MIN           = Int32(10210)
const SPC_AVAILLOOPS_MAX           = Int32(10211)
const SPC_AVAILLOOPS_STEP          = Int32(10212)

const SPC_AVAILAVERAGES_MIN        = Int32(10220)
const SPC_AVAILAVERAGES_MAX        = Int32(10221)
const SPC_AVAILAVERAGES_STEP       = Int32(10222)

const SPC_AVAILAVRGSEGSIZE_MIN     = Int32(10223)
const SPC_AVAILAVRGSEGSIZE_MAX     = Int32(10224)
const SPC_AVAILAVRGSEGSIZE_STEP    = Int32(10225)

const SPC_AVAILAVERAGES16BIT_MIN      = Int32(10226)
const SPC_AVAILAVERAGES16BIT_MAX      = Int32(10227)
const SPC_AVAILAVERAGES16BIT_STEP     = Int32(10228)

const SPC_AVAILAVRG16BITSEGSIZE_MIN   = Int32(10229)
const SPC_AVAILAVRG16BITSEGSIZE_MAX   = Int32(10230)
const SPC_AVAILAVRG16BITSEGSIZE_STEP  = Int32(10231)

const SPC_AVAILBOXCARAVERAGES_MIN          = Int32(10232)
const SPC_AVAILBOXCARAVERAGES_MAX          = Int32(10233)
const SPC_AVAILBOXCARAVERAGES_STEPFACTOR   = Int32(10234)


# Channels
const SPC_CHENABLE                 = Int32(11000)
const SPC_CHCOUNT                  = Int32(11001)
const SPC_CHMODACOUNT              = Int32(11100)
const SPC_CHMODBCOUNT              = Int32(11101)


# ----- channel enable flags for A/D and D/A boards (MI/MC/MX series) -----
#       and all cards on M2i series
const     CHANNEL0                 = Int32(1) # 0x00000001
const     CHANNEL1                 = Int32(2) # 0x00000002
const     CHANNEL2                 = Int32(4) # 0x00000004
const     CHANNEL3                 = Int32(8) # 0x00000008
const     CHANNEL4                 = Int32(16) # 0x00000010
const     CHANNEL5                 = Int32(32) # 0x00000020
const     CHANNEL6                 = Int32(64) # 0x00000040
const     CHANNEL7                 = Int32(128) # 0x00000080
const     CHANNEL8                 = Int32(256) # 0x00000100
const     CHANNEL9                 = Int32(512) # 0x00000200
const     CHANNEL10                = Int32(1024) # 0x00000400
const     CHANNEL11                = Int32(2048) # 0x00000800
const     CHANNEL12                = Int32(4096) # 0x00001000
const     CHANNEL13                = Int32(8192) # 0x00002000
const     CHANNEL14                = Int32(16384) # 0x00004000
const     CHANNEL15                = Int32(32768) # 0x00008000
const     CHANNEL16                = Int32(65536) # 0x00010000
const     CHANNEL17                = Int32(131072) # 0x00020000
const     CHANNEL18                = Int32(262144) # 0x00040000
const     CHANNEL19                = Int32(524288) # 0x00080000
const     CHANNEL20                = Int32(1048576) # 0x00100000
const     CHANNEL21                = Int32(2097152) # 0x00200000
const     CHANNEL22                = Int32(4194304) # 0x00400000
const     CHANNEL23                = Int32(8388608) # 0x00800000
const     CHANNEL24                = Int32(16777216) # 0x01000000
const     CHANNEL25                = Int32(33554432) # 0x02000000
const     CHANNEL26                = Int32(67108864) # 0x04000000
const     CHANNEL27                = Int32(134217728) # 0x08000000
const     CHANNEL28                = Int32(268435456) # 0x10000000
const     CHANNEL29                = Int32(536870912) # 0x20000000
const     CHANNEL30                = Int32(1073741824) # 0x40000000
const     CHANNEL31                = Int32(-2147483648) # 0x80000000
# CHANNEL32 up to CHANNEL63 are placed in the upper 32 bit of a 64 bit word (M2i only)


# ----- old digital i/o settings for 16 bit implementation (MI/MC/MX series)  -----
const     CH0_8BITMODE             = Int32(65536)  # for MI.70xx only
const     CH0_16BIT                = Int32(1)
const     CH0_32BIT                = Int32(3)
const     CH1_16BIT                = Int32(4)
const     CH1_32BIT                = Int32(12)

# ----- new digital i/o settings for 8 bit implementation (MI/MC/MX series) -----
const     MOD0_8BIT                = Int32(1)
const     MOD0_16BIT               = Int32(3)
const     MOD0_32BIT               = Int32(15)
const     MOD1_8BIT                = Int32(16)
const     MOD1_16BIT               = Int32(48)
const     MOD1_32BIT               = Int32(240)

const SPC_CHROUTE0                 = Int32(11010)
const SPC_CHROUTE1                 = Int32(11020)

const SPC_BITENABLE                = Int32(11030)



# ----- Clock Settings -----
const SPC_SAMPLERATE               = Int32(20000)
const SPC_SYNCCLOCK                = Int32(20005)
const SPC_SAMPLERATE2              = Int32(20010)
const SPC_SR2                      = Int32(20020)
const SPC_PLL_ENABLE               = Int32(20030)
const SPC_PLL_ISLOCKED             = Int32(20031)
const SPC_CLOCKDIV                 = Int32(20040)
const SPC_INTCLOCKDIV              = Int32(20041)
const SPC_PXICLOCKDIV              = Int32(20042)
const SPC_PLL_R                    = Int32(20060)
const SPC_PLL_F                    = Int32(20061)
const SPC_PLL_S                    = Int32(20062)
const SPC_PLL_DIV                  = Int32(20063)
const SPC_PXI_CLK_OUT              = Int32(20090)
const SPC_EXTERNALCLOCK            = Int32(20100)
const SPC_EXTERNOUT                = Int32(20110)
const SPC_CLOCKOUT                 = Int32(20110)
const SPC_CLOCKOUTFREQUENCY        = Int32(20111)
const SPC_CLOCK50OHM               = Int32(20120)
const SPC_CLOCK110OHM              = Int32(20120)
const SPC_CLOCK75OHM               = Int32(20120)
const SPC_STROBE75OHM              = Int32(20121)
const SPC_EXTERNRANGE              = Int32(20130)
const SPC_EXTRANGESHDIRECT         = Int32(20131)
const     EXRANGE_NONE             = Int32(0)
const     EXRANGE_NOPLL            = Int32(1)
const     EXRANGE_SINGLE           = Int32(2)
const     EXRANGE_BURST_S          = Int32(4)
const     EXRANGE_BURST_M          = Int32(8)
const     EXRANGE_BURST_L          = Int32(16)
const     EXRANGE_BURST_XL         = Int32(32)
const     EXRANGE_LOW              = Int32(64)
const     EXRANGE_HIGH             = Int32(128)
const     EXRANGE_LOW_DPS          = Int32(256)            # digital phase synchronization
const SPC_REFERENCECLOCK           = Int32(20140)
const     REFCLOCK_PXI             = Int32(-1)

# ----- new clock registers starting with M2i cards -----
const SPC_CLOCKMODE                = Int32(20200)      # clock mode as listed below
const SPC_AVAILCLOCKMODES          = Int32(20201)      # returns all available clock modes
const     SPC_CM_INTPLL            = Int32(1) # 0x00000001      # use internal PLL
const     SPC_CM_QUARTZ1           = Int32(2) # 0x00000002      # use plain quartz1 (with divider)
const     SPC_CM_QUARTZ2           = Int32(4) # 0x00000004      # use plain quartz2 (with divider)
const     SPC_CM_EXTERNAL          = Int32(8) # 0x00000008      # use external clock directly
const     SPC_CM_EXTERNAL0         = Int32(8) # 0x00000008      # use external clock0 directly (identical value to SPC_CM_EXTERNAL)
const     SPC_CM_EXTDIVIDER        = Int32(16) # 0x00000010      # use external clock with programmed divider
const     SPC_CM_EXTREFCLOCK       = Int32(32) # 0x00000020      # external reference clock fed in (defined with SPC_REFERENCECLOCK)
const     SPC_CM_PXIREFCLOCK       = Int32(64) # 0x00000040      # PXI reference clock
const     SPC_CM_SHDIRECT          = Int32(128) # 0x00000080      # Star-hub direct clock (not synchronised)
const     SPC_CM_QUARTZ2_DIRSYNC   = Int32(256) # 0x00000100      # use plain quartz2 (with divider) and put the Q2 clock on the star-hub module
const     SPC_CM_QUARTZ1_DIRSYNC   = Int32(512) # 0x00000200      # use plain quartz1 (with divider) and put the Q1 clock on the star-hub module
const     SPC_CM_EXTERNAL1         = Int32(1024) # 0x00000400      # use external clock1 directly
# ----- internal use only! -----
const     SPC_CM_SYNCINT           = Int32(16777216) # 0x01000000
const     SPC_CM_SYNCEXT           = Int32(33554432) # 0x02000000

const SPC_CLOCK_READFEATURES       = Int32(20205)
const SPC_CLOCK_READFEATURES0      = Int32(20205)
const SPC_CLOCK_READFEATURES1      = Int32(20206)
const     SPCM_CKFEAT_TERM             = Int32(1) # 0x00000001
const     SPCM_CKFEAT_HIGHIMP          = Int32(2) # 0x00000002
const     SPCM_CKFEAT_DCCOUPLING       = Int32(4) # 0x00000004
const     SPCM_CKFEAT_ACCOUPLING       = Int32(8) # 0x00000008
const     SPCM_CKFEAT_SE               = Int32(16) # 0x00000010
const     SPCM_CKFEAT_DIFF             = Int32(32) # 0x00000020
const     SPCM_CKFEAT_PROGEDGE         = Int32(64) # 0x00000040
const     SPCM_CKFEAT_LEVELPROG        = Int32(256) # 0x00000100
const     SPCM_CKFEAT_PROGTHRESHOLD    = Int32(512) # 0x00000200
const     SPCM_CKFEAT_PROGDELAY        = Int32(1024) # 0x00000400

const SPC_BURSTSYSCLOCKMODE        = Int32(20210)
const SPC_SYNCMASTERSYSCLOCKMODE   = Int32(20211)
const SPC_CLOCK_SETUP_CHANGED      = Int32(20212)

# clock delay if available
const SPC_CLOCK_AVAILDELAY_MIN     = Int32(20220)
const SPC_CLOCK_AVAILDELAY_MAX     = Int32(20221)
const SPC_CLOCK_AVAILDELAY_STEP    = Int32(20222)
const SPC_CLOCK_DELAY              = Int32(20223)

# clock edges
const SPC_AVAILCLOCKEDGES          = Int32(20224)
const     SPCM_EDGE_FALLING        = Int32(1) # 0x00000001 # Originally SPCM_RISING_EDGE  : name and value of constant intentionally changed with driver versions greater than V5.24. See hardware manual for details.
const     SPCM_EDGE_RISING         = Int32(2) # 0x00000002 # Originally SPCM_FALLING_EDGE : name and value of constant intentionally changed with driver versions greater than V5.24. See hardware manual for details.
const     SPCM_BOTH_EDGES          = Int32(4) # 0x00000004
const     SPCM_EDGES_BOTH          = Int32(4) # 0x00000004 #Just added for good measure to match naming scheme of above SPCM_EDGE_FALLING and SPCM_EDGE_RISING constants.
const SPC_CLOCK_EDGE               = Int32(20225)

# mux definitions for channel routing
const SPC_CHANNELMUXINFO           = Int32(20300)
const     SPCM_MUX_NONE             = Int32(0) # 0x00000000  # nothing is interlaced
const     SPCM_MUX_MUXONMOD         = Int32(1) # 0x00000001  # data on module is multiplexed, only one channel can have full speed
const     SPCM_MUX_INVERTCLKONMOD   = Int32(2) # 0x00000002  # two channels on one module run with inverted clock
const     SPCM_MUX_DLY              = Int32(3) # 0x00000003  # delay cable between modules, one channel can have full interlace speed
const     SPCM_MUX_DLYANDMUXONMOD   = Int32(4) # 0x00000004  # delay cable between modules and multplexing on module
const     SPCM_MUX_MUXBETWEENMODS   = Int32(5) # 0x00000005  # multiplexed between modules (fastest sampling rate only with one module)
const     SPCM_MUX_MUXONMOD2CH      = Int32(6) # 0x00000006  # data on module is multiplexed, only two channel can have full speed
const     SPCM_MUX_MAX4CH           = Int32(7) # 0x00000007  # only four channels can have full speed, independent of distribution on modules


# ----- In/Out Range -----
const SPC_OFFS0                    = Int32(30000)
const SPC_AMP0                     = Int32(30010)
const SPC_ACDC0                    = Int32(30020)
const SPC_ACDC_OFFS_COMPENSATION0  = Int32(30021)
const SPC_50OHM0                   = Int32(30030)
const SPC_DIFF0                    = Int32(30040)
const SPC_DOUBLEOUT0               = Int32(30041)
const SPC_DIGITAL0                 = Int32(30050)
const SPC_110OHM0                  = Int32(30060)
const SPC_110OHM0L                 = Int32(30060)
const SPC_75OHM0                   = Int32(30060)
const SPC_INOUT0                   = Int32(30070)
const SPC_FILTER0                  = Int32(30080)
const SPC_BANKSWITCH0              = Int32(30081)
const SPC_PATH0                    = Int32(30090)
const SPC_ENABLEOUT0               = Int32(30091)

const SPC_OFFS1                    = Int32(30100)
const SPC_AMP1                     = Int32(30110)
const SPC_ACDC1                    = Int32(30120)
const SPC_ACDC_OFFS_COMPENSATION1  = Int32(30121)
const SPC_50OHM1                   = Int32(30130)
const SPC_DIFF1                    = Int32(30140)
const SPC_DOUBLEOUT1               = Int32(30141)
const SPC_DIGITAL1                 = Int32(30150)
const SPC_110OHM1                  = Int32(30160)
const SPC_110OHM0H                 = Int32(30160)
const SPC_75OHM1                   = Int32(30160)
const SPC_INOUT1                   = Int32(30170)
const SPC_FILTER1                  = Int32(30180)
const SPC_BANKSWITCH1              = Int32(30181)
const SPC_PATH1                    = Int32(30190)
const SPC_ENABLEOUT1               = Int32(30191)

const SPC_OFFS2                    = Int32(30200)
const SPC_AMP2                     = Int32(30210)
const SPC_ACDC2                    = Int32(30220)
const SPC_ACDC_OFFS_COMPENSATION2  = Int32(30221)
const SPC_50OHM2                   = Int32(30230)
const SPC_DIFF2                    = Int32(30240)
const SPC_DOUBLEOUT2               = Int32(30241)
const SPC_110OHM2                  = Int32(30260)
const SPC_110OHM1L                 = Int32(30260)
const SPC_75OHM2                   = Int32(30260)
const SPC_INOUT2                   = Int32(30270)
const SPC_FILTER2                  = Int32(30280)
const SPC_BANKSWITCH2              = Int32(30281)
const SPC_PATH2                    = Int32(30290)
const SPC_ENABLEOUT2               = Int32(30291)

const SPC_OFFS3                    = Int32(30300)
const SPC_AMP3                     = Int32(30310)
const SPC_ACDC3                    = Int32(30320)
const SPC_ACDC_OFFS_COMPENSATION3  = Int32(30321)
const SPC_50OHM3                   = Int32(30330)
const SPC_DIFF3                    = Int32(30340)
const SPC_DOUBLEOUT3               = Int32(30341)
const SPC_110OHM3                  = Int32(30360)
const SPC_110OHM1H                 = Int32(30360)
const SPC_75OHM3                   = Int32(30360)
const SPC_INOUT3                   = Int32(30370)
const SPC_FILTER3                  = Int32(30380)
const SPC_BANKSWITCH3              = Int32(30381)
const SPC_PATH3                    = Int32(30390)
const SPC_ENABLEOUT3               = Int32(30391)

const SPC_OFFS4                    = Int32(30400)
const SPC_AMP4                     = Int32(30410)
const SPC_ACDC4                    = Int32(30420)
const SPC_50OHM4                   = Int32(30430)
const SPC_DIFF4                    = Int32(30440)
const SPC_DOUBLEOUT4               = Int32(30441)
const SPC_FILTER4                  = Int32(30480)
const SPC_ENABLEOUT4               = Int32(30491)
const SPC_PATH4                    = Int32(30490)

const SPC_OFFS5                    = Int32(30500)
const SPC_AMP5                     = Int32(30510)
const SPC_ACDC5                    = Int32(30520)
const SPC_50OHM5                   = Int32(30530)
const SPC_DIFF5                    = Int32(30540)
const SPC_DOUBLEOUT5               = Int32(30541)
const SPC_FILTER5                  = Int32(30580)
const SPC_ENABLEOUT5               = Int32(30591)
const SPC_PATH5                    = Int32(30590)

const SPC_OFFS6                    = Int32(30600)
const SPC_AMP6                     = Int32(30610)
const SPC_ACDC6                    = Int32(30620)
const SPC_50OHM6                   = Int32(30630)
const SPC_DIFF6                    = Int32(30640)
const SPC_DOUBLEOUT6               = Int32(30641)
const SPC_FILTER6                  = Int32(30680)
const SPC_ENABLEOUT6               = Int32(30691)
const SPC_PATH6                    = Int32(30690)

const SPC_OFFS7                    = Int32(30700)
const SPC_AMP7                     = Int32(30710)
const SPC_ACDC7                    = Int32(30720)
const SPC_50OHM7                   = Int32(30730)
const SPC_DIFF7                    = Int32(30740)
const SPC_DOUBLEOUT7               = Int32(30741)
const SPC_FILTER7                  = Int32(30780)
const SPC_ENABLEOUT7               = Int32(30791)
const SPC_PATH7                    = Int32(30790)

const SPC_OFFS8                    = Int32(30800)
const SPC_AMP8                     = Int32(30810)
const SPC_ACDC8                    = Int32(30820)
const SPC_50OHM8                   = Int32(30830)
const SPC_DIFF8                    = Int32(30840)
const SPC_PATH8                    = Int32(30890)

const SPC_OFFS9                    = Int32(30900)
const SPC_AMP9                     = Int32(30910)
const SPC_ACDC9                    = Int32(30920)
const SPC_50OHM9                   = Int32(30930)
const SPC_DIFF9                    = Int32(30940)
const SPC_PATH9                    = Int32(30990)

const SPC_OFFS10                   = Int32(31000)
const SPC_AMP10                    = Int32(31010)
const SPC_ACDC10                   = Int32(31020)
const SPC_50OHM10                  = Int32(31030)
const SPC_DIFF10                   = Int32(31040)
const SPC_PATH10                   = Int32(31090)

const SPC_OFFS11                   = Int32(31100)
const SPC_AMP11                    = Int32(31110)
const SPC_ACDC11                   = Int32(31120)
const SPC_50OHM11                  = Int32(31130)
const SPC_DIFF11                   = Int32(31140)
const SPC_PATH11                   = Int32(31190)

const SPC_OFFS12                   = Int32(31200)
const SPC_AMP12                    = Int32(31210)
const SPC_ACDC12                   = Int32(31220)
const SPC_50OHM12                  = Int32(31230)
const SPC_DIFF12                   = Int32(31240)
const SPC_PATH12                   = Int32(31290)

const SPC_OFFS13                   = Int32(31300)
const SPC_AMP13                    = Int32(31310)
const SPC_ACDC13                   = Int32(31320)
const SPC_50OHM13                  = Int32(31330)
const SPC_DIFF13                   = Int32(31340)
const SPC_PATH13                   = Int32(31390)

const SPC_OFFS14                   = Int32(31400)
const SPC_AMP14                    = Int32(31410)
const SPC_ACDC14                   = Int32(31420)
const SPC_50OHM14                  = Int32(31430)
const SPC_DIFF14                   = Int32(31440)
const SPC_PATH14                   = Int32(31490)

const SPC_OFFS15                   = Int32(31500)
const SPC_AMP15                    = Int32(31510)
const SPC_ACDC15                   = Int32(31520)
const SPC_50OHM15                  = Int32(31530)
const SPC_DIFF15                   = Int32(31540)
const SPC_PATH15                   = Int32(31590)

const SPC_110OHMTRIGGER            = Int32(30400)
const SPC_110OHMCLOCK              = Int32(30410)


const   AMP_BI200                  = Int32(200)
const   AMP_BI500                  = Int32(500)
const   AMP_BI1000                 = Int32(1000)
const   AMP_BI2000                 = Int32(2000)
const   AMP_BI2500                 = Int32(2500)
const   AMP_BI4000                 = Int32(4000)
const   AMP_BI5000                 = Int32(5000)
const   AMP_BI10000                = Int32(10000)
const   AMP_UNI400                 = Int32(100400)
const   AMP_UNI1000                = Int32(101000)
const   AMP_UNI2000                = Int32(102000)


# ----- Trigger Settings -----
const SPC_TRIGGERMODE              = Int32(40000)
const SPC_TRIG_OUTPUT              = Int32(40100)
const SPC_TRIGGEROUT               = Int32(40100)
const SPC_TRIG_TERM                = Int32(40110)
const SPC_TRIG_TERM0               = Int32(40110)
const SPC_TRIGGER50OHM             = Int32(40110)
const SPC_TRIGGER110OHM0           = Int32(40110)
const SPC_TRIGGER75OHM0            = Int32(40110)
const SPC_TRIG_TERM1               = Int32(40111)
const SPC_TRIGGER110OHM1           = Int32(40111)
const SPC_TRIG_EXT0_ACDC           = Int32(40120)
const SPC_TRIG_EXT1_ACDC           = Int32(40121)
const SPC_TRIG_EXT2_ACDC           = Int32(40122)

const SPC_TRIGGERMODE0             = Int32(40200)
const SPC_TRIGGERMODE1             = Int32(40201)
const SPC_TRIGGERMODE2             = Int32(40202)
const SPC_TRIGGERMODE3             = Int32(40203)
const SPC_TRIGGERMODE4             = Int32(40204)
const SPC_TRIGGERMODE5             = Int32(40205)
const SPC_TRIGGERMODE6             = Int32(40206)
const SPC_TRIGGERMODE7             = Int32(40207)
const SPC_TRIGGERMODE8             = Int32(40208)
const SPC_TRIGGERMODE9             = Int32(40209)
const SPC_TRIGGERMODE10            = Int32(40210)
const SPC_TRIGGERMODE11            = Int32(40211)
const SPC_TRIGGERMODE12            = Int32(40212)
const SPC_TRIGGERMODE13            = Int32(40213)
const SPC_TRIGGERMODE14            = Int32(40214)
const SPC_TRIGGERMODE15            = Int32(40215)

const     TM_SOFTWARE              = Int32(0)
const     TM_NOTRIGGER             = Int32(10)
const     TM_CHXPOS                = Int32(10000)
const     TM_CHXPOS_LP             = Int32(10001)
const     TM_CHXPOS_SP             = Int32(10002)
const     TM_CHXPOS_GS             = Int32(10003)
const     TM_CHXPOS_SS             = Int32(10004)
const     TM_CHXNEG                = Int32(10010)
const     TM_CHXNEG_LP             = Int32(10011)
const     TM_CHXNEG_SP             = Int32(10012)
const     TM_CHXNEG_GS             = Int32(10013)
const     TM_CHXNEG_SS             = Int32(10014)
const     TM_CHXOFF                = Int32(10020)
const     TM_CHXBOTH               = Int32(10030)
const     TM_CHXWINENTER           = Int32(10040)
const     TM_CHXWINENTER_LP        = Int32(10041)
const     TM_CHXWINENTER_SP        = Int32(10042)
const     TM_CHXWINLEAVE           = Int32(10050)
const     TM_CHXWINLEAVE_LP        = Int32(10051)
const     TM_CHXWINLEAVE_SP        = Int32(10052)
const     TM_CHXLOW                = Int32(10060)
const     TM_CHXHIGH               = Int32(10061)
const     TM_CHXINWIN              = Int32(10062)
const     TM_CHXOUTWIN             = Int32(10063)
const     TM_CHXSPIKE              = Int32(10064)


const     TM_CH0POS                = Int32(10000)
const     TM_CH0NEG                = Int32(10010)
const     TM_CH0OFF                = Int32(10020)
const     TM_CH0BOTH               = Int32(10030)
const     TM_CH1POS                = Int32(10100)
const     TM_CH1NEG                = Int32(10110)
const     TM_CH1OFF                = Int32(10120)
const     TM_CH1BOTH               = Int32(10130)
const     TM_CH2POS                = Int32(10200)
const     TM_CH2NEG                = Int32(10210)
const     TM_CH2OFF                = Int32(10220)
const     TM_CH2BOTH               = Int32(10230)
const     TM_CH3POS                = Int32(10300)
const     TM_CH3NEG                = Int32(10310)
const     TM_CH3OFF                = Int32(10320)
const     TM_CH3BOTH               = Int32(10330)

const     TM_TTLPOS                = Int32(20000)
const     TM_TTLHIGH_LP            = Int32(20001)
const     TM_TTLHIGH_SP            = Int32(20002)
const     TM_TTLNEG                = Int32(20010)
const     TM_TTLLOW_LP             = Int32(20011)
const     TM_TTLLOW_SP             = Int32(20012)
const     TM_TTL                   = Int32(20020)
const     TM_TTLBOTH               = Int32(20030)
const     TM_TTLBOTH_LP            = Int32(20031)
const     TM_TTLBOTH_SP            = Int32(20032)
const     TM_CHANNEL               = Int32(20040)
const     TM_TTLHIGH               = Int32(20050)
const     TM_TTLLOW                = Int32(20051)
const     TM_PATTERN               = Int32(21000)
const     TM_PATTERN_LP            = Int32(21001)
const     TM_PATTERN_SP            = Int32(21002)
const     TM_PATTERNANDEDGE        = Int32(22000)
const     TM_PATTERNANDEDGE_LP     = Int32(22001)
const     TM_PATTERNANDEDGE_SP     = Int32(22002)
const     TM_GATELOW               = Int32(30000)
const     TM_GATEHIGH              = Int32(30010)
const     TM_GATEPATTERN           = Int32(30020)
const     TM_CHOR                  = Int32(35000)
const     TM_CHAND                 = Int32(35010)
const     TM_CHORTTLPOS            = Int32(35020)
const     TM_CHORTTLNEG            = Int32(35021)

const SPC_PXITRGOUT                = Int32(40300)
const     PTO_OFF                   = Int32(0)
const     PTO_LINE0                 = Int32(1)
const     PTO_LINE1                 = Int32(2)
const     PTO_LINE2                 = Int32(3)
const     PTO_LINE3                 = Int32(4)
const     PTO_LINE4                 = Int32(5)
const     PTO_LINE5                 = Int32(6)
const     PTO_LINE6                 = Int32(7)
const     PTO_LINE7                 = Int32(8)
const     PTO_LINESTAR              = Int32(9)
const SPC_PXITRGOUT_AVAILABLE      = Int32(40301)  # bitmap register

const SPC_PXISTARTRG_DIVRST_OUT    = Int32(40302)  # bitmap register
const SPC_PXISTARTRG_DIVRST_OUT_AVAILABLE    = Int32(40303)
const SPC_PXISTARTRG_OUT           = Int32(40304)  # bitmap register
const     PSTO_LINESTAR0            = Int32(1) # 0x00000001
const     PSTO_LINESTAR1            = Int32(2) # 0x00000002
const     PSTO_LINESTAR2            = Int32(4) # 0x00000004
const     PSTO_LINESTAR3            = Int32(8) # 0x00000008
const     PSTO_LINESTAR4            = Int32(16) # 0x00000010
const     PSTO_LINESTAR5            = Int32(32) # 0x00000020
const     PSTO_LINESTAR6            = Int32(64) # 0x00000040
const     PSTO_LINESTAR7            = Int32(128) # 0x00000080
const     PSTO_LINESTAR8            = Int32(256) # 0x00000100
const     PSTO_LINESTAR9            = Int32(512) # 0x00000200
const     PSTO_LINESTAR10           = Int32(1024) # 0x00000400
const     PSTO_LINESTAR11           = Int32(2048) # 0x00000800
const     PSTO_LINESTAR12           = Int32(4096) # 0x00001000
const     PSTO_LINE0                = Int32(65536) # 0x00010000
const     PSTO_LINE1                = Int32(131072) # 0x00020000
const     PSTO_LINE2                = Int32(262144) # 0x00040000
const     PSTO_LINE3                = Int32(524288) # 0x00080000
const     PSTO_LINE4                = Int32(1048576) # 0x00100000
const     PSTO_LINE5                = Int32(2097152) # 0x00200000
const     PSTO_LINE6                = Int32(4194304) # 0x00400000
const     PSTO_LINE7                = Int32(8388608) # 0x00800000
const SPC_PXISTARTRG_OUT_AVAILABLE           = Int32(40305)

const SPC_PXITRGIN                 = Int32(40310)  # bitmap register
const     PTI_OFF                   = Int32(0)
const     PTI_LINE0                 = Int32(1)
const     PTI_LINE1                 = Int32(2)
const     PTI_LINE2                 = Int32(4)
const     PTI_LINE3                 = Int32(8)
const     PTI_LINE4                 = Int32(16)
const     PTI_LINE5                 = Int32(32)
const     PTI_LINE6                 = Int32(64)
const     PTI_LINE7                 = Int32(128)
const     PTI_LINESTAR              = Int32(256)
const SPC_PXITRGIN_AVAILABLE       = Int32(40311)  # bitmap register
const SPC_PXI_DIVIDER_RESET_IN             = Int32(40320)
const SPC_PXI_DIVIDER_RESET_IN_AVAILABLE   = Int32(40321)


# new registers of M2i driver
const SPC_TRIG_AVAILORMASK         = Int32(40400)
const SPC_TRIG_ORMASK              = Int32(40410)
const SPC_TRIG_AVAILANDMASK        = Int32(40420)
const SPC_TRIG_ANDMASK             = Int32(40430)
const     SPC_TMASK_NONE           = Int32(0) # 0x00000000
const     SPC_TMASK_SOFTWARE       = Int32(1) # 0x00000001
const     SPC_TMASK_EXT0           = Int32(2) # 0x00000002
const     SPC_TMASK_EXT1           = Int32(4) # 0x00000004
const     SPC_TMASK_EXT2           = Int32(8) # 0x00000008
const     SPC_TMASK_EXT3           = Int32(16) # 0x00000010
const     SPC_TMASK_EXT4           = Int32(32) # 0x00000020
const     SPC_TMASK_XIO0           = Int32(256) # 0x00000100
const     SPC_TMASK_XIO1           = Int32(512) # 0x00000200
const     SPC_TMASK_XIO2           = Int32(1024) # 0x00000400
const     SPC_TMASK_XIO3           = Int32(2048) # 0x00000800
const     SPC_TMASK_XIO4           = Int32(4096) # 0x00001000
const     SPC_TMASK_XIO5           = Int32(8192) # 0x00002000
const     SPC_TMASK_XIO6           = Int32(16384) # 0x00004000
const     SPC_TMASK_XIO7           = Int32(32768) # 0x00008000
const     SPC_TMASK_PXI0           = Int32(1048576) # 0x00100000
const     SPC_TMASK_PXI1           = Int32(2097152) # 0x00200000
const     SPC_TMASK_PXI2           = Int32(4194304) # 0x00400000
const     SPC_TMASK_PXI3           = Int32(8388608) # 0x00800000
const     SPC_TMASK_PXI4           = Int32(16777216) # 0x01000000
const     SPC_TMASK_PXI5           = Int32(33554432) # 0x02000000
const     SPC_TMASK_PXI6           = Int32(67108864) # 0x04000000
const     SPC_TMASK_PXI7           = Int32(134217728) # 0x08000000
const     SPC_TMASK_PXISTAR        = Int32(268435456) # 0x10000000
const     SPC_TMASK_PXIDSTARB      = Int32(536870912) # 0x20000000

const SPC_TRIG_CH_AVAILORMASK0     = Int32(40450)
const SPC_TRIG_CH_AVAILORMASK1     = Int32(40451)
const SPC_TRIG_CH_ORMASK0          = Int32(40460)
const SPC_TRIG_CH_ORMASK1          = Int32(40461)
const SPC_TRIG_CH_AVAILANDMASK0    = Int32(40470)
const SPC_TRIG_CH_AVAILANDMASK1    = Int32(40471)
const SPC_TRIG_CH_ANDMASK0         = Int32(40480) 
const SPC_TRIG_CH_ANDMASK1         = Int32(40481) 
const     SPC_TMASK0_NONE          = Int32(0) # 0x00000000
const     SPC_TMASK0_CH0           = Int32(1) # 0x00000001
const     SPC_TMASK0_CH1           = Int32(2) # 0x00000002
const     SPC_TMASK0_CH2           = Int32(4) # 0x00000004
const     SPC_TMASK0_CH3           = Int32(8) # 0x00000008
const     SPC_TMASK0_CH4           = Int32(16) # 0x00000010
const     SPC_TMASK0_CH5           = Int32(32) # 0x00000020
const     SPC_TMASK0_CH6           = Int32(64) # 0x00000040
const     SPC_TMASK0_CH7           = Int32(128) # 0x00000080
const     SPC_TMASK0_CH8           = Int32(256) # 0x00000100
const     SPC_TMASK0_CH9           = Int32(512) # 0x00000200
const     SPC_TMASK0_CH10          = Int32(1024) # 0x00000400
const     SPC_TMASK0_CH11          = Int32(2048) # 0x00000800
const     SPC_TMASK0_CH12          = Int32(4096) # 0x00001000
const     SPC_TMASK0_CH13          = Int32(8192) # 0x00002000
const     SPC_TMASK0_CH14          = Int32(16384) # 0x00004000
const     SPC_TMASK0_CH15          = Int32(32768) # 0x00008000
const     SPC_TMASK0_CH16          = Int32(65536) # 0x00010000
const     SPC_TMASK0_CH17          = Int32(131072) # 0x00020000
const     SPC_TMASK0_CH18          = Int32(262144) # 0x00040000
const     SPC_TMASK0_CH19          = Int32(524288) # 0x00080000
const     SPC_TMASK0_CH20          = Int32(1048576) # 0x00100000
const     SPC_TMASK0_CH21          = Int32(2097152) # 0x00200000
const     SPC_TMASK0_CH22          = Int32(4194304) # 0x00400000
const     SPC_TMASK0_CH23          = Int32(8388608) # 0x00800000
const     SPC_TMASK0_CH24          = Int32(16777216) # 0x01000000
const     SPC_TMASK0_CH25          = Int32(33554432) # 0x02000000
const     SPC_TMASK0_CH26          = Int32(67108864) # 0x04000000
const     SPC_TMASK0_CH27          = Int32(134217728) # 0x08000000
const     SPC_TMASK0_CH28          = Int32(268435456) # 0x10000000
const     SPC_TMASK0_CH29          = Int32(536870912) # 0x20000000
const     SPC_TMASK0_CH30          = Int32(1073741824) # 0x40000000
const     SPC_TMASK0_CH31          = Int32(-2147483648) # 0x80000000

const     SPC_TMASK1_NONE          = Int32(0) # 0x00000000
const     SPC_TMASK1_CH32          = Int32(1) # 0x00000001
const     SPC_TMASK1_CH33          = Int32(2) # 0x00000002
const     SPC_TMASK1_CH34          = Int32(4) # 0x00000004
const     SPC_TMASK1_CH35          = Int32(8) # 0x00000008
const     SPC_TMASK1_CH36          = Int32(16) # 0x00000010
const     SPC_TMASK1_CH37          = Int32(32) # 0x00000020
const     SPC_TMASK1_CH38          = Int32(64) # 0x00000040
const     SPC_TMASK1_CH39          = Int32(128) # 0x00000080
const     SPC_TMASK1_CH40          = Int32(256) # 0x00000100
const     SPC_TMASK1_CH41          = Int32(512) # 0x00000200
const     SPC_TMASK1_CH42          = Int32(1024) # 0x00000400
const     SPC_TMASK1_CH43          = Int32(2048) # 0x00000800
const     SPC_TMASK1_CH44          = Int32(4096) # 0x00001000
const     SPC_TMASK1_CH45          = Int32(8192) # 0x00002000
const     SPC_TMASK1_CH46          = Int32(16384) # 0x00004000
const     SPC_TMASK1_CH47          = Int32(32768) # 0x00008000
const     SPC_TMASK1_CH48          = Int32(65536) # 0x00010000
const     SPC_TMASK1_CH49          = Int32(131072) # 0x00020000
const     SPC_TMASK1_CH50          = Int32(262144) # 0x00040000
const     SPC_TMASK1_CH51          = Int32(524288) # 0x00080000
const     SPC_TMASK1_CH52          = Int32(1048576) # 0x00100000
const     SPC_TMASK1_CH53          = Int32(2097152) # 0x00200000
const     SPC_TMASK1_CH54          = Int32(4194304) # 0x00400000
const     SPC_TMASK1_CH55          = Int32(8388608) # 0x00800000
const     SPC_TMASK1_CH56          = Int32(16777216) # 0x01000000
const     SPC_TMASK1_CH57          = Int32(33554432) # 0x02000000
const     SPC_TMASK1_CH58          = Int32(67108864) # 0x04000000
const     SPC_TMASK1_CH59          = Int32(134217728) # 0x08000000
const     SPC_TMASK1_CH60          = Int32(268435456) # 0x10000000
const     SPC_TMASK1_CH61          = Int32(536870912) # 0x20000000
const     SPC_TMASK1_CH62          = Int32(1073741824) # 0x40000000
const     SPC_TMASK1_CH63          = Int32(-2147483648) # 0x80000000

const SPC_TRIG_EXT_AVAILMODES      = Int32(40500)
const SPC_TRIG_EXT0_AVAILMODES     = Int32(40500)
const SPC_TRIG_EXT1_AVAILMODES     = Int32(40501)
const SPC_TRIG_EXT2_AVAILMODES     = Int32(40502)
const SPC_TRIG_EXT0_AVAILMODESOR   = Int32(40503)
const SPC_TRIG_EXT1_AVAILMODESOR   = Int32(40504)
const SPC_TRIG_EXT2_AVAILMODESOR   = Int32(40505)
const SPC_TRIG_EXT0_AVAILMODESAND  = Int32(40506)
const SPC_TRIG_EXT1_AVAILMODESAND  = Int32(40507)
const SPC_TRIG_EXT2_AVAILMODESAND  = Int32(40508)
const SPC_TRIG_EXT3_AVAILMODESAND  = Int32(40509)
const SPC_TRIG_EXT0_MODE           = Int32(40510)
const SPC_TRIG_EXT1_MODE           = Int32(40511)
const SPC_TRIG_EXT2_MODE           = Int32(40512)
const SPC_TRIG_EXT3_MODE           = Int32(40513)
const SPC_TRIG_EXT3_AVAILMODES     = Int32(40514)
const SPC_TRIG_EXT3_AVAILMODESOR   = Int32(40515)
const SPC_TRIG_EXT4_AVAILMODES     = Int32(40516)
const SPC_TRIG_EXT4_AVAILMODESOR   = Int32(40517)
const SPC_TRIG_EXT4_AVAILMODESAND  = Int32(40518)
const SPC_TRIG_EXT4_MODE           = Int32(40519)

const SPC_TRIG_EXT0_READFEATURES   = Int32(40520)
const SPC_TRIG_EXT1_READFEATURES   = Int32(40521)
const SPC_TRIG_EXT2_READFEATURES   = Int32(40522)
const SPC_TRIG_EXT3_READFEATURES   = Int32(40523)
const SPC_TRIG_EXT4_READFEATURES   = Int32(40524)
const     SPCM_TRFEAT_TERM             = Int32(1) # 0x00000001
const     SPCM_TRFEAT_HIGHIMP          = Int32(2) # 0x00000002
const     SPCM_TRFEAT_DCCOUPLING       = Int32(4) # 0x00000004
const     SPCM_TRFEAT_ACCOUPLING       = Int32(8) # 0x00000008
const     SPCM_TRFEAT_SE               = Int32(16) # 0x00000010
const     SPCM_TRFEAT_DIFF             = Int32(32) # 0x00000020
const     SPCM_TRFEAT_LEVELPROG        = Int32(256) # 0x00000100
const     SPCM_TRFEAT_PROGTHRESHOLD    = Int32(512) # 0x00000200

# legacy constants: not enough contiguous constants possible for X4..X19
const SPC_LEGACY_X0_READFEATURES   = Int32(40530)
const SPC_LEGACY_X1_READFEATURES   = Int32(40531)
const SPC_LEGACY_X2_READFEATURES   = Int32(40532)
const SPC_LEGACY_X3_READFEATURES   = Int32(40533)

# legacy constants: not enough contiguous constants possible for X4..X19
const SPC_LEGACY_X0_TERM           = Int32(40535)
const SPC_LEGACY_X1_TERM           = Int32(40536)
const SPC_LEGACY_X2_TERM           = Int32(40537)
const SPC_LEGACY_X3_TERM           = Int32(40538)

const SPC_TRIG_XIO_AVAILMODES      = Int32(40550)
const SPC_TRIG_XIO_AVAILMODESOR    = Int32(40551)
const SPC_TRIG_XIO_AVAILMODESAND   = Int32(40552)
const SPC_TRIG_XIO0_MODE           = Int32(40560)
const SPC_TRIG_XIO1_MODE           = Int32(40561)
const     SPC_TM_MODEMASK          = Int32(16777215) # 0x00FFFFFF
const     SPC_TM_NONE              = Int32(0) # 0x00000000
const     SPC_TM_POS               = Int32(1) # 0x00000001
const     SPC_TM_NEG               = Int32(2) # 0x00000002
const     SPC_TM_BOTH              = Int32(4) # 0x00000004
const     SPC_TM_HIGH              = Int32(8) # 0x00000008
const     SPC_TM_LOW               = Int32(16) # 0x00000010
const     SPC_TM_WINENTER          = Int32(32) # 0x00000020
const     SPC_TM_WINLEAVE          = Int32(64) # 0x00000040
const     SPC_TM_INWIN             = Int32(128) # 0x00000080
const     SPC_TM_OUTSIDEWIN        = Int32(256) # 0x00000100
const     SPC_TM_SPIKE             = Int32(512) # 0x00000200
const     SPC_TM_PATTERN           = Int32(1024) # 0x00000400
const     SPC_TM_STEEPPOS          = Int32(2048) # 0x00000800
const     SPC_TM_STEEPNEG          = Int32(4096) # 0x00001000
const     SPC_TM_EXTRAMASK         = Int32(-16777216) # 0xFF000000
const     SPC_TM_REARM             = Int32(16777216) # 0x01000000
const     SPC_TM_PW_SMALLER        = Int32(33554432) # 0x02000000
const     SPC_TM_PW_GREATER        = Int32(67108864) # 0x04000000
const     SPC_TM_DOUBLEEDGE        = Int32(134217728) # 0x08000000
const     SPC_TM_PULSESTRETCH      = Int32(268435456) # 0x10000000
const     SPC_TM_HYSTERESIS        = Int32(536870912) # 0x20000000

const SPC_TRIG_PATTERN_AVAILMODES  = Int32(40580)
const SPC_TRIG_PATTERN_MODE        = Int32(40590)

const SPC_TRIG_CH_AVAILMODES       = Int32(40600)
const SPC_TRIG_CH_AVAILMODESOR     = Int32(40601)
const SPC_TRIG_CH_AVAILMODESAND    = Int32(40602)
const SPC_TRIG_CH0_MODE            = Int32(40610)
const SPC_TRIG_CH1_MODE            = Int32(40611)
const SPC_TRIG_CH2_MODE            = Int32(40612)
const SPC_TRIG_CH3_MODE            = Int32(40613)
const SPC_TRIG_CH4_MODE            = Int32(40614)
const SPC_TRIG_CH5_MODE            = Int32(40615)
const SPC_TRIG_CH6_MODE            = Int32(40616)
const SPC_TRIG_CH7_MODE            = Int32(40617)
const SPC_TRIG_CH8_MODE            = Int32(40618)
const SPC_TRIG_CH9_MODE            = Int32(40619)
const SPC_TRIG_CH10_MODE           = Int32(40620)
const SPC_TRIG_CH11_MODE           = Int32(40621)
const SPC_TRIG_CH12_MODE           = Int32(40622)
const SPC_TRIG_CH13_MODE           = Int32(40623)
const SPC_TRIG_CH14_MODE           = Int32(40624)
const SPC_TRIG_CH15_MODE           = Int32(40625)
const SPC_TRIG_CH16_MODE           = Int32(40626)
const SPC_TRIG_CH17_MODE           = Int32(40627)
const SPC_TRIG_CH18_MODE           = Int32(40628)
const SPC_TRIG_CH19_MODE           = Int32(40629)
const SPC_TRIG_CH20_MODE           = Int32(40630)
const SPC_TRIG_CH21_MODE           = Int32(40631)
const SPC_TRIG_CH22_MODE           = Int32(40632)
const SPC_TRIG_CH23_MODE           = Int32(40633)
const SPC_TRIG_CH24_MODE           = Int32(40634)
const SPC_TRIG_CH25_MODE           = Int32(40635)
const SPC_TRIG_CH26_MODE           = Int32(40636)
const SPC_TRIG_CH27_MODE           = Int32(40637)
const SPC_TRIG_CH28_MODE           = Int32(40638)
const SPC_TRIG_CH29_MODE           = Int32(40639)
const SPC_TRIG_CH30_MODE           = Int32(40640)
const SPC_TRIG_CH31_MODE           = Int32(40641)

const SPC_TRIG_CH32_MODE           = Int32(40642)
const SPC_TRIG_CH33_MODE           = Int32(40643)
const SPC_TRIG_CH34_MODE           = Int32(40644)
const SPC_TRIG_CH35_MODE           = Int32(40645)
const SPC_TRIG_CH36_MODE           = Int32(40646)
const SPC_TRIG_CH37_MODE           = Int32(40647)
const SPC_TRIG_CH38_MODE           = Int32(40648)
const SPC_TRIG_CH39_MODE           = Int32(40649)
const SPC_TRIG_CH40_MODE           = Int32(40650)
const SPC_TRIG_CH41_MODE           = Int32(40651)
const SPC_TRIG_CH42_MODE           = Int32(40652)
const SPC_TRIG_CH43_MODE           = Int32(40653)
const SPC_TRIG_CH44_MODE           = Int32(40654)
const SPC_TRIG_CH45_MODE           = Int32(40655)
const SPC_TRIG_CH46_MODE           = Int32(40656)
const SPC_TRIG_CH47_MODE           = Int32(40657)
const SPC_TRIG_CH48_MODE           = Int32(40658)
const SPC_TRIG_CH49_MODE           = Int32(40659)
const SPC_TRIG_CH50_MODE           = Int32(40660)
const SPC_TRIG_CH51_MODE           = Int32(40661)
const SPC_TRIG_CH52_MODE           = Int32(40662)
const SPC_TRIG_CH53_MODE           = Int32(40663)
const SPC_TRIG_CH54_MODE           = Int32(40664)
const SPC_TRIG_CH55_MODE           = Int32(40665)
const SPC_TRIG_CH56_MODE           = Int32(40666)
const SPC_TRIG_CH57_MODE           = Int32(40667)
const SPC_TRIG_CH58_MODE           = Int32(40668)
const SPC_TRIG_CH59_MODE           = Int32(40669)
const SPC_TRIG_CH60_MODE           = Int32(40670)
const SPC_TRIG_CH61_MODE           = Int32(40671)
const SPC_TRIG_CH62_MODE           = Int32(40672)
const SPC_TRIG_CH63_MODE           = Int32(40673)


const SPC_TRIG_AVAILDELAY          = Int32(40800)
const SPC_TRIG_AVAILDELAY_STEP     = Int32(40801)
const SPC_TRIG_DELAY               = Int32(40810)

const SPC_TRIG_AVAILHOLDOFF        = Int32(40802)
const SPC_TRIG_AVAILHOLDOFF_STEP   = Int32(40803)
const SPC_TRIG_HOLDOFF             = Int32(40811)

const SPC_SINGLESHOT               = Int32(41000)
const SPC_OUTONTRIGGER             = Int32(41100)
const SPC_RESTARTCONT              = Int32(41200)
const SPC_SINGLERESTART            = Int32(41300)

const SPC_TRIGGERLEVEL             = Int32(42000)
const SPC_TRIGGERLEVEL0            = Int32(42000)
const SPC_TRIGGERLEVEL1            = Int32(42001)
const SPC_TRIGGERLEVEL2            = Int32(42002)
const SPC_TRIGGERLEVEL3            = Int32(42003)
const SPC_TRIGGERLEVEL4            = Int32(42004)
const SPC_TRIGGERLEVEL5            = Int32(42005)
const SPC_TRIGGERLEVEL6            = Int32(42006)
const SPC_TRIGGERLEVEL7            = Int32(42007)
const SPC_TRIGGERLEVEL8            = Int32(42008)
const SPC_TRIGGERLEVEL9            = Int32(42009)
const SPC_TRIGGERLEVEL10           = Int32(42010)
const SPC_TRIGGERLEVEL11           = Int32(42011)
const SPC_TRIGGERLEVEL12           = Int32(42012)
const SPC_TRIGGERLEVEL13           = Int32(42013)
const SPC_TRIGGERLEVEL14           = Int32(42014)
const SPC_TRIGGERLEVEL15           = Int32(42015)

const SPC_AVAILHIGHLEVEL_MIN       = Int32(41997)
const SPC_AVAILHIGHLEVEL_MAX       = Int32(41998)
const SPC_AVAILHIGHLEVEL_STEP      = Int32(41999)

const SPC_HIGHLEVEL0               = Int32(42000)
const SPC_HIGHLEVEL1               = Int32(42001)
const SPC_HIGHLEVEL2               = Int32(42002)
const SPC_HIGHLEVEL3               = Int32(42003)
const SPC_HIGHLEVEL4               = Int32(42004)
const SPC_HIGHLEVEL5               = Int32(42005)
const SPC_HIGHLEVEL6               = Int32(42006)
const SPC_HIGHLEVEL7               = Int32(42007)
const SPC_HIGHLEVEL8               = Int32(42008)
const SPC_HIGHLEVEL9               = Int32(42009)
const SPC_HIGHLEVEL10              = Int32(42010)
const SPC_HIGHLEVEL11              = Int32(42011)
const SPC_HIGHLEVEL12              = Int32(42012)
const SPC_HIGHLEVEL13              = Int32(42013)
const SPC_HIGHLEVEL14              = Int32(42014)
const SPC_HIGHLEVEL15              = Int32(42015)

const SPC_AVAILLOWLEVEL_MIN        = Int32(42097)
const SPC_AVAILLOWLEVEL_MAX        = Int32(42098)
const SPC_AVAILLOWLEVEL_STEP       = Int32(42099)

const SPC_LOWLEVEL0                = Int32(42100)
const SPC_LOWLEVEL1                = Int32(42101)
const SPC_LOWLEVEL2                = Int32(42102)
const SPC_LOWLEVEL3                = Int32(42103)
const SPC_LOWLEVEL4                = Int32(42104)
const SPC_LOWLEVEL5                = Int32(42105)
const SPC_LOWLEVEL6                = Int32(42106)
const SPC_LOWLEVEL7                = Int32(42107)
const SPC_LOWLEVEL8                = Int32(42108)
const SPC_LOWLEVEL9                = Int32(42109)
const SPC_LOWLEVEL10               = Int32(42110)
const SPC_LOWLEVEL11               = Int32(42111)
const SPC_LOWLEVEL12               = Int32(42112)
const SPC_LOWLEVEL13               = Int32(42113)
const SPC_LOWLEVEL14               = Int32(42114)
const SPC_LOWLEVEL15               = Int32(42115)

const SPC_TRIG_CH0_LEVEL0          = Int32(42200)
const SPC_TRIG_CH1_LEVEL0          = Int32(42201)
const SPC_TRIG_CH2_LEVEL0          = Int32(42202)
const SPC_TRIG_CH3_LEVEL0          = Int32(42203)
const SPC_TRIG_CH4_LEVEL0          = Int32(42204)
const SPC_TRIG_CH5_LEVEL0          = Int32(42205)
const SPC_TRIG_CH6_LEVEL0          = Int32(42206)
const SPC_TRIG_CH7_LEVEL0          = Int32(42207)
const SPC_TRIG_CH8_LEVEL0          = Int32(42208)
const SPC_TRIG_CH9_LEVEL0          = Int32(42209)
const SPC_TRIG_CH10_LEVEL0         = Int32(42210)
const SPC_TRIG_CH11_LEVEL0         = Int32(42211)
const SPC_TRIG_CH12_LEVEL0         = Int32(42212)
const SPC_TRIG_CH13_LEVEL0         = Int32(42213)
const SPC_TRIG_CH14_LEVEL0         = Int32(42214)
const SPC_TRIG_CH15_LEVEL0         = Int32(42215)

const SPC_TRIG_CH0_LEVEL1          = Int32(42300)
const SPC_TRIG_CH1_LEVEL1          = Int32(42301)
const SPC_TRIG_CH2_LEVEL1          = Int32(42302)
const SPC_TRIG_CH3_LEVEL1          = Int32(42303)
const SPC_TRIG_CH4_LEVEL1          = Int32(42304)
const SPC_TRIG_CH5_LEVEL1          = Int32(42305)
const SPC_TRIG_CH6_LEVEL1          = Int32(42306)
const SPC_TRIG_CH7_LEVEL1          = Int32(42307)
const SPC_TRIG_CH8_LEVEL1          = Int32(42308)
const SPC_TRIG_CH9_LEVEL1          = Int32(42309)
const SPC_TRIG_CH10_LEVEL1         = Int32(42310)
const SPC_TRIG_CH11_LEVEL1         = Int32(42311)
const SPC_TRIG_CH12_LEVEL1         = Int32(42312)
const SPC_TRIG_CH13_LEVEL1         = Int32(42313)
const SPC_TRIG_CH14_LEVEL1         = Int32(42314)
const SPC_TRIG_CH15_LEVEL1         = Int32(42315)

const SPC_TRIG_EXT0_LEVEL0         = Int32(42320)
const SPC_TRIG_EXT1_LEVEL0         = Int32(42321)
const SPC_TRIG_EXT2_LEVEL0         = Int32(42322)

const SPC_TRIG_EXT0_LEVEL1         = Int32(42330)
const SPC_TRIG_EXT1_LEVEL1         = Int32(42331)
const SPC_TRIG_EXT2_LEVEL1         = Int32(42332)

const SPC_TRIG_EXT_AVAIL0_MIN      = Int32(42340)
const SPC_TRIG_EXT_AVAIL0_MAX      = Int32(42341)
const SPC_TRIG_EXT_AVAIL0_STEP     = Int32(42342)

const SPC_TRIG_EXT_AVAIL1_MIN      = Int32(42345)
const SPC_TRIG_EXT_AVAIL1_MAX      = Int32(42346)
const SPC_TRIG_EXT_AVAIL1_STEP     = Int32(42347)

# threshold levels (for 77xx)
const SPC_THRESHOLD0               = Int32(42400)  # threshold level for channel group 0
const SPC_THRESHOLD1               = Int32(42401)  # threshold level for channel group 1
const SPC_THRESHOLD2               = Int32(42402)  # threshold level for channel group 2
const SPC_THRESHOLD3               = Int32(42403)  # threshold level for channel group 3
const SPC_CLOCK_THRESHOLD          = Int32(42410)  # threshold level for clock input
const SPC_TRIG_THRESHOLD           = Int32(42411)  # threshold level for trigger input
const SPC_X0X1_THRESHOLD           = Int32(42412)  # threshold level for X0/X1 input
const SPC_STROBE_THRESHOLD         = Int32(42413)  # threshold level for strobe input

const SPC_AVAILTHRESHOLD_MIN       = Int32(42420)
const SPC_AVAILTHRESHOLD_MAX       = Int32(42421)
const SPC_AVAILTHRESHOLD_STEP      = Int32(42422)

const SPC_AVAILAVRGMODES           = Int32(42429)
const SPC_AVRGMODE_CH0             = Int32(42430) 
const SPC_AVRGMODE_CH1             = Int32(42431)
const SPC_AVRGMODE_CH2             = Int32(42432)
const SPC_AVRGMODE_CH3             = Int32(42433)
const     AVRGMODE_NORMAL            = Int32(0) # 0x0  # normal block averaging
const     AVRGMODE_TDA_HIGH          = Int32(1) # 0x1  # threshold defined averaging: (samples >  threshhold) will be used, others replaced
const     AVRGMODE_TDA_LOW           = Int32(2) # 0x2  # threshold defined averaging: (samples <= threshhold) will be used, others replaced

const SPC_TDA_THRESHOLD_CH0        = Int32(42440)  # threshhold for threshold defined averaging in LSB of ADC
const SPC_TDA_THRESHOLD_CH1        = Int32(42441)
const SPC_TDA_THRESHOLD_CH2        = Int32(42442)
const SPC_TDA_THRESHOLD_CH3        = Int32(42443)

const SPC_TDA_REPLACEMENT_CH0      = Int32(42450)  # replacement sample value for threshold defined averaging in LSB of ADC
const SPC_TDA_REPLACEMENT_CH1      = Int32(42451)
const SPC_TDA_REPLACEMENT_CH2      = Int32(42452)
const SPC_TDA_REPLACEMENT_CH3      = Int32(42453)

const SPC_CLOCK_AVAILTHRESHOLD_MIN   = Int32(42423)
const SPC_CLOCK_AVAILTHRESHOLD_MAX   = Int32(42424)
const SPC_CLOCK_AVAILTHRESHOLD_STEP  = Int32(42425)

const SPC_TRIG_AVAILTHRESHOLD_MIN   = Int32(42426)
const SPC_TRIG_AVAILTHRESHOLD_MAX   = Int32(42427)
const SPC_TRIG_AVAILTHRESHOLD_STEP  = Int32(42428)

const SPC_TRIGGERPATTERN           = Int32(43000)
const SPC_TRIGGERPATTERN0          = Int32(43000)
const SPC_TRIGGERPATTERN1          = Int32(43001)
const SPC_TRIGGERMASK              = Int32(43100)
const SPC_TRIGGERMASK0             = Int32(43100)
const SPC_TRIGGERMASK1             = Int32(43101)

const SPC_PULSEWIDTH               = Int32(44000)
const SPC_PULSEWIDTH0              = Int32(44000)
const SPC_PULSEWIDTH1              = Int32(44001)

const SPC_TRIG_CH_AVAILPULSEWIDTH  = Int32(44100)
const SPC_TRIG_CH_PULSEWIDTH       = Int32(44101)
const SPC_TRIG_CH0_PULSEWIDTH      = Int32(44101)
const SPC_TRIG_CH1_PULSEWIDTH      = Int32(44102)
const SPC_TRIG_CH2_PULSEWIDTH      = Int32(44103)
const SPC_TRIG_CH3_PULSEWIDTH      = Int32(44104)
const SPC_TRIG_CH4_PULSEWIDTH      = Int32(44105)
const SPC_TRIG_CH5_PULSEWIDTH      = Int32(44106)
const SPC_TRIG_CH6_PULSEWIDTH      = Int32(44107)
const SPC_TRIG_CH7_PULSEWIDTH      = Int32(44108)
const SPC_TRIG_CH8_PULSEWIDTH      = Int32(44109)
const SPC_TRIG_CH9_PULSEWIDTH      = Int32(44110)
const SPC_TRIG_CH10_PULSEWIDTH     = Int32(44111)
const SPC_TRIG_CH11_PULSEWIDTH     = Int32(44112)
const SPC_TRIG_CH12_PULSEWIDTH     = Int32(44113)
const SPC_TRIG_CH13_PULSEWIDTH     = Int32(44114)
const SPC_TRIG_CH14_PULSEWIDTH     = Int32(44115)
const SPC_TRIG_CH15_PULSEWIDTH     = Int32(44116)

const SPC_TRIG_EXT_AVAILPULSEWIDTH  = Int32(44200)
const SPC_TRIG_EXT0_PULSEWIDTH     = Int32(44210)
const SPC_TRIG_EXT1_PULSEWIDTH     = Int32(44211)
const SPC_TRIG_EXT2_PULSEWIDTH     = Int32(44212)
const SPC_TRIG_EXT3_PULSEWIDTH     = Int32(44213)

# available dividers for MICX
const SPC_READCLOCKDIVCOUNT        = Int32(44300)
const SPC_CLOCKDIV0                = Int32(44301)
const SPC_CLOCKDIV1                = Int32(44302)
const SPC_CLOCKDIV2                = Int32(44303)
const SPC_CLOCKDIV3                = Int32(44304)
const SPC_CLOCKDIV4                = Int32(44305)
const SPC_CLOCKDIV5                = Int32(44306)
const SPC_CLOCKDIV6                = Int32(44307)
const SPC_CLOCKDIV7                = Int32(44308)
const SPC_CLOCKDIV8                = Int32(44309)
const SPC_CLOCKDIV9                = Int32(44310)
const SPC_CLOCKDIV10               = Int32(44311)
const SPC_CLOCKDIV11               = Int32(44312)
const SPC_CLOCKDIV12               = Int32(44313)
const SPC_CLOCKDIV13               = Int32(44314)
const SPC_CLOCKDIV14               = Int32(44315)
const SPC_CLOCKDIV15               = Int32(44316)
const SPC_CLOCKDIV16               = Int32(44317)

const SPC_READTROFFSET             = Int32(45000)
const SPC_TRIGGEREDGE              = Int32(46000)
const SPC_TRIGGEREDGE0             = Int32(46000)
const SPC_TRIGGEREDGE1             = Int32(46001)
const     TE_POS                   = Int32(10000)
const     TE_NEG                   = Int32(10010)
const     TE_BOTH                  = Int32(10020)
const     TE_NONE                  = Int32(10030)


# ----- Timestamp -----
const CH_TIMESTAMP                 = Int32(9999)

const SPC_TIMESTAMP_CMD            = Int32(47000)
const     TS_RESET                     = Int32(0)
const     TS_MODE_DISABLE              = Int32(10)
const     TS_MODE_STARTRESET           = Int32(11)
const     TS_MODE_STANDARD             = Int32(12)
const     TS_MODE_REFCLOCK             = Int32(13)
const     TS_MODE_TEST5555             = Int32(90)
const     TS_MODE_TESTAAAA             = Int32(91)
const     TS_MODE_ZHTEST               = Int32(92)

# ----- modes for M2i, M3i, M4i, M4x, M2p hardware (bitmap) -----
const SPC_TIMESTAMP_AVAILMODES     = Int32(47001)
const     SPC_TSMODE_DISABLE       = Int32(0) # 0x00000000
const     SPC_TS_RESET             = Int32(1) # 0x00000001
const     SPC_TSMODE_STANDARD      = Int32(2) # 0x00000002
const     SPC_TSMODE_STARTRESET    = Int32(4) # 0x00000004
const     SPC_TS_RESET_WAITREFCLK  = Int32(8) # 0x00000008
const     SPC_TSCNT_INTERNAL       = Int32(256) # 0x00000100
const     SPC_TSCNT_REFCLOCKPOS    = Int32(512) # 0x00000200
const     SPC_TSCNT_REFCLOCKNEG    = Int32(1024) # 0x00000400
const     SPC_TSFEAT_NONE          = Int32(0) # 0x00000000
const     SPC_TSFEAT_STORE1STABA   = Int32(65536) # 0x00010000
const     SPC_TSFEAT_INCRMODE      = Int32(131072) # 0x00020000
const     SPC_TSFEAT_INCRMODE12    = Int32(262144) # 0x00040000
const     SPC_TSFEAT_TRGSRC        = Int32(524288) # 0x00080000

const     SPC_TSXIOACQ_DISABLE     = Int32(0) # 0x00000000
const     SPC_TSXIOACQ_ENABLE      = Int32(4096) # 0x00001000
const     SPC_TSXIOINC_ENABLE      = Int32(8192) # 0x00002000
const     SPC_TSXIOINC12_ENABLE    = Int32(16384) # 0x00004000

const     SPC_TSMODE_MASK          = Int32(255) # 0x000000FF
const     SPC_TSCNT_MASK           = Int32(3840) # 0x00000F00
const     SPC_TSFEAT_MASK          = Int32(983040) # 0x000F0000

const     SPC_TRGSRC_MASK_CH0        = Int32(1) # 0x00000001
const     SPC_TRGSRC_MASK_CH1        = Int32(2) # 0x00000002
const     SPC_TRGSRC_MASK_CH2        = Int32(4) # 0x00000004
const     SPC_TRGSRC_MASK_CH3        = Int32(8) # 0x00000008
const     SPC_TRGSRC_MASK_CH4        = Int32(16) # 0x00000010
const     SPC_TRGSRC_MASK_CH5        = Int32(32) # 0x00000020
const     SPC_TRGSRC_MASK_CH6        = Int32(64) # 0x00000040
const     SPC_TRGSRC_MASK_CH7        = Int32(128) # 0x00000080
const     SPC_TRGSRC_MASK_EXT0       = Int32(256) # 0x00000100
const     SPC_TRGSRC_MASK_EXT1       = Int32(512) # 0x00000200
const     SPC_TRGSRC_MASK_FORCE      = Int32(1024) # 0x00000400
# space for digital channels using TSXIOACQ_ENABLE of standard multi-purpose lines
const     SPC_TRGSRC_MASK_PXI0       = Int32(65536) # 0x00010000
const     SPC_TRGSRC_MASK_PXI1       = Int32(131072) # 0x00020000
const     SPC_TRGSRC_MASK_PXI2       = Int32(262144) # 0x00040000
const     SPC_TRGSRC_MASK_PXI3       = Int32(524288) # 0x00080000
const     SPC_TRGSRC_MASK_PXI4       = Int32(1048576) # 0x00100000
const     SPC_TRGSRC_MASK_PXI5       = Int32(2097152) # 0x00200000
const     SPC_TRGSRC_MASK_PXI6       = Int32(4194304) # 0x00400000
const     SPC_TRGSRC_MASK_PXI7       = Int32(8388608) # 0x00800000
const     SPC_TRGSRC_MASK_PXISTAR    = Int32(16777216) # 0x01000000
const     SPC_TRGSRC_MASK_PXIDSTARB  = Int32(33554432) # 0x02000000
const     SPC_TRGSRC_MASK_X0         = Int32(268435456) # 0x10000000
const     SPC_TRGSRC_MASK_X1         = Int32(536870912) # 0x20000000
const     SPC_TRGSRC_MASK_X2         = Int32(1073741824) # 0x40000000
const     SPC_TRGSRC_MASK_X3         = Int32(-2147483648) # 0x80000000
# space for more digital channels using TSXIOACQ_ENABLE of additional multi-purpose lines (optional)


const SPC_TIMESTAMP_STATUS         = Int32(47010)
const     TS_FIFO_EMPTY                = Int32(0)
const     TS_FIFO_LESSHALF             = Int32(1)
const     TS_FIFO_MOREHALF             = Int32(2)
const     TS_FIFO_OVERFLOW             = Int32(3)

const SPC_TIMESTAMP_SIZE           = Int32(47011)

const SPC_TIMESTAMP_COUNT          = Int32(47020)
const SPC_TIMESTAMP_STARTTIME      = Int32(47030)
const SPC_TIMESTAMP_STARTDATE      = Int32(47031)
const SPC_TIMESTAMP_FIFO           = Int32(47040)
const SPC_TIMESTAMP_TIMEOUT        = Int32(47045)

const SPC_TIMESTAMP_RESETMODE      = Int32(47050)
const     TS_RESET_POS                = Int32(10)
const     TS_RESET_NEG                = Int32(20)



# ----- Extra I/O module -----
const SPC_XIO_DIRECTION            = Int32(47100)
const     XD_CH0_INPUT                 = Int32(0)
const     XD_CH0_OUTPUT                = Int32(1)
const     XD_CH1_INPUT                 = Int32(0)
const     XD_CH1_OUTPUT                = Int32(2)
const     XD_CH2_INPUT                 = Int32(0)
const     XD_CH2_OUTPUT                = Int32(4)
const SPC_XIO_DIGITALIO            = Int32(47110)
const SPC_XIO_ANALOGOUT0           = Int32(47120)
const SPC_XIO_ANALOGOUT1           = Int32(47121)
const SPC_XIO_ANALOGOUT2           = Int32(47122)
const SPC_XIO_ANALOGOUT3           = Int32(47123)
const SPC_XIO_WRITEDACS            = Int32(47130)



# ----- M3i        multi purpose lines (X0, X1        ) 
# ----- M4i + M4x  multi purpose lines (X0, X1, X2    ) 
# ----- M2p        multi purpose lines (X0, X1, X2, X3) and with installed option also (X4 .. X19)

# legacy constants: not enough contiguous constants possible for X4..X19,
# hence new constants for X-modes (SPCM_X0_MODE.. SPCM_X19_MODE) exist further below
const SPCM_LEGACY_X0_MODE          = Int32(47200)
const SPCM_LEGACY_X1_MODE          = Int32(47201)
const SPCM_LEGACY_X2_MODE          = Int32(47202)
const SPCM_LEGACY_X3_MODE          = Int32(47203)
const SPCM_LEGACY_X0_AVAILMODES    = Int32(47210)
const SPCM_LEGACY_X1_AVAILMODES    = Int32(47211)
const SPCM_LEGACY_X2_AVAILMODES    = Int32(47212)
const SPCM_LEGACY_X3_AVAILMODES    = Int32(47213)
const     SPCM_XMODE_DISABLE            = Int32(0) # 0x00000000
const     SPCM_XMODE_ASYNCIN            = Int32(1) # 0x00000001  # used as asynchronous input
const     SPCM_XMODE_ASYNCOUT           = Int32(2) # 0x00000002  # used as asynchronous output
const     SPCM_XMODE_DIGIN              = Int32(4) # 0x00000004  # used as synchronous digital input
const     SPCM_XMODE_DDS                = Int32(4) # 0x00000004  # same as SPCM_XMODE_DIGIN because that one is used for inputs only, and the other is output only
const     SPCM_XMODE_DIGOUT             = Int32(8) # 0x00000008  # used as synchronous digital output
const     SPCM_XMODE_TRIGIN             = Int32(16) # 0x00000010  # used as trigger input
const     SPCM_XMODE_TRIGOUT            = Int32(32) # 0x00000020  # used as trigger output
const     SPCM_XMODE_OVROUT             = Int32(64) # 0x00000040  # used as ADC overrange output
const     SPCM_XMODE_DIGIN2BIT          = Int32(128) # 0x00000080  # used as synchronous digital input, 2bits per channel
const     SPCM_XMODE_RUNSTATE           = Int32(256) # 0x00000100  # shows the run state of the card (high = run)
const     SPCM_XMODE_ARMSTATE           = Int32(512) # 0x00000200  # shows the arm state (high = armed for trigger of one single card)
const     SPCM_XMODE_DIRECTTRIGOUT      = Int32(1024) # 0x00000400  # used as direct trigger output (safe mode) 
const     SPCM_XMODE_DIRECTTRIGOUT_LR   = Int32(2048) # 0x00000800  # used as direct trigger output (low re-arm)
const     SPCM_XMODE_REFCLKOUT          = Int32(4096) # 0x00001000  # outputs internal or fed in external refclock
const     SPCM_XMODE_CONTOUTMARK        = Int32(8192) # 0x00002000  # outputs a half posttrigger long HIGH pulse on replay
const     SPCM_XMODE_SYSCLKOUT          = Int32(16384) # 0x00004000  # outputs internal system clock
const     SPCM_XMODE_CLKOUT             = Int32(32768) # 0x00008000  # clock output
const     SPCM_XMODE_SYNCARMSTATE       = Int32(65536) # 0x00010000  # shows the arm state (high = armed for trigger when all cards connected to a Star-Hub are armed)
const     SPCM_XMODE_OPTDIGIN2BIT       = Int32(131072) # 0x00020000  # used as synchronous digital input from digitaloption, 2bits per channel
const     SPCM_XMODE_OPTDIGIN4BIT       = Int32(262144) # 0x00040000  # used as synchronous digital input from digitaloption, 4bits per channel
const     SPCM_XMODE_PULSEGEN           = Int32(524288) # 0x00080000  # output of pulse generator
const     SPCM_XMODE_MODEMASK           = Int32(1048575) # 0x000FFFFF

# additional constants to be combined together with SPCM_XMODE_DIGOUT to select analog channel containing digital data
const     SPCM_XMODE_DIGOUTSRC_CH0      = Int32(16777216) # 0x01000000  # Select Ch0 as source 
const     SPCM_XMODE_DIGOUTSRC_CH1      = Int32(33554432) # 0x02000000  # Select Ch1 as source
const     SPCM_XMODE_DIGOUTSRC_CH2      = Int32(67108864) # 0x04000000  # Select Ch2 as source
const     SPCM_XMODE_DIGOUTSRC_CH3      = Int32(134217728) # 0x08000000  # Select Ch3 as source
const     SPCM_XMODE_DIGOUTSRC_CH4      = Int32(268435456) # 0x10000000  # Select Ch4 as source
const     SPCM_XMODE_DIGOUTSRC_CH5      = Int32(536870912) # 0x20000000  # Select Ch5 as source
const     SPCM_XMODE_DIGOUTSRC_CH6      = Int32(1073741824) # 0x40000000  # Select Ch6 as source
const     SPCM_XMODE_DIGOUTSRC_CH7      = Int32(-2147483648) # 0x80000000  # Select Ch7 as source
const     SPCM_XMODE_DIGOUTSRC_CHMASK   = Int32(-16777216) # 0xFF000000

# additional constants to be combined together with SPCM_XMODE_DIGOUT to select digital signal source
const     SPCM_XMODE_DIGOUTSRC_BIT15               = Int32(1048576) # 0x00100000  # Use Bit15 (MSB    ) of selected channel: channel resolution will be reduced to 15 bit
const     SPCM_XMODE_DIGOUTSRC_BIT14               = Int32(2097152) # 0x00200000  # Use Bit14 (MSB - 1) of selected channel: channel resolution will be reduced to 14 bit
const     SPCM_XMODE_DIGOUTSRC_BIT13               = Int32(4194304) # 0x00400000  # Use Bit13 (MSB - 2) of selected channel: channel resolution will be reduced to 13 bit
const     SPCM_XMODE_DIGOUTSRC_BIT12               = Int32(8388608) # 0x00800000  # Use Bit12 (MSB - 3) of selected channel: channel resolution will be reduced to 12 bit
const     SPCM_XMODE_DIGOUTSRC_BITMASK             = Int32(15728640) # 0x00F00000
# special combinations for M2p.65xx cards with options SPCM_FEAT_DIG16_SMB or SPCM_FEAT_DIG16_FX2
const     SPCM_XMODE_DIGOUTSRC_BIT15_downto_0      = Int32(15728640) # 0x00F00000  # use all   16 bits of selected channel on  (X19..X4)              : channel will only contain digital data
const     SPCM_XMODE_DIGOUTSRC_BIT15_downto_8      = Int32(7340032) # 0x00700000  # use upper  8 bits of selected channel for (X19..X12) or (X11..X4): channel resolution will be reduced to 8 bit

const SPCM_XX_ASYNCIO              = Int32(47220)           # asynchronous in/out register

const SPC_DIGMODE0  = Int32(47250)
const SPC_DIGMODE1  = Int32(47251)
const SPC_DIGMODE2  = Int32(47252)
const SPC_DIGMODE3  = Int32(47253)
const SPC_DIGMODE4  = Int32(47254)
const SPC_DIGMODE5  = Int32(47255)
const SPC_DIGMODE6  = Int32(47256)
const SPC_DIGMODE7  = Int32(47257)
const     SPCM_DIGMODE_OFF  = Int32(0) # 0x00000000
const     SPCM_DIGMODE_X0   = Int32(554188800) # 0x21084000 # (M2P_DIGMODE_X0 << (32 - 5)) | (M2P_DIGMODE_X0 << (32 - 10)) | (M2P_DIGMODE_X0 << (32 - 15)) | (M2P_DIGMODE_X0 << (32 - 20))
const     SPCM_DIGMODE_X1   = Int32(692736000) # 0x294A5000 # (M2P_DIGMODE_X1 << (32 - 5)) | (M2P_DIGMODE_X1 << (32 - 10)) | (M2P_DIGMODE_X1 << (32 - 15)) | (M2P_DIGMODE_X1 << (32 - 20))
const     SPCM_DIGMODE_X2   = Int32(831283200) # 0x318C6000 # (M2P_DIGMODE_X2 << (32 - 5)) | (M2P_DIGMODE_X2 << (32 - 10)) | (M2P_DIGMODE_X2 << (32 - 15)) | (M2P_DIGMODE_X2 << (32 - 20))
const     SPCM_DIGMODE_X3   = Int32(969830400) # 0x39CE7000 # (M2P_DIGMODE_X3 << (32 - 5)) | (M2P_DIGMODE_X3 << (32 - 10)) | (M2P_DIGMODE_X3 << (32 - 15)) | (M2P_DIGMODE_X3 << (32 - 20))
const     SPCM_DIGMODE_X4   = Int32(-2078212095) # 0x84210001
const     SPCM_DIGMODE_X5   = Int32(-1939664894) # 0x8c631002
const     SPCM_DIGMODE_X6   = Int32(-1801117692) # 0x94a52004
const     SPCM_DIGMODE_X7   = Int32(-1662570488) # 0x9ce73008
const     SPCM_DIGMODE_X8   = Int32(-1524023280) # 0xa5294010
const     SPCM_DIGMODE_X9   = Int32(-1385476064) # 0xad6b5020
const     SPCM_DIGMODE_X10  = Int32(-1246928832) # 0xb5ad6040
const     SPCM_DIGMODE_X11  = Int32(-1108381568) # 0xbdef7080
const     SPCM_DIGMODE_X12  = Int32(-969834240) # 0xc6318100
const     SPCM_DIGMODE_X13  = Int32(-831286784) # 0xce739200
const     SPCM_DIGMODE_X14  = Int32(-692739072) # 0xd6b5a400
const     SPCM_DIGMODE_X15  = Int32(-554190848) # 0xdef7b800
const     SPCM_DIGMODE_X16  = Int32(-415645696) # 0xe739c000
const     SPCM_DIGMODE_X17  = Int32(-277098496) # 0xef7bd000
const     SPCM_DIGMODE_X18  = Int32(-138551296) # 0xf7bde000
const     SPCM_DIGMODE_X19  = Int32(-4096) # 0xfffff000

const     DIGMODEMASK_BIT15  = Int32(-134217728) # 0xF8000000
const     DIGMODEMASK_BIT14  = Int32(130023424) # 0x07C00000
const     DIGMODEMASK_BIT13  = Int32(4063232) # 0x003E0000
const     DIGMODEMASK_BIT12  = Int32(126976) # 0x0001F000
const     DIGMODEMASK_BIT11  = Int32(2048) # 0x00000800 # one bit only for bit 11 downto 0
const     DIGMODEMASK_BIT10  = Int32(1024) # 0x00000400
const     DIGMODEMASK_BIT9   = Int32(512) # 0x00000200
const     DIGMODEMASK_BIT8   = Int32(256) # 0x00000100
const     DIGMODEMASK_BIT7   = Int32(128) # 0x00000080
const     DIGMODEMASK_BIT6   = Int32(64) # 0x00000040
const     DIGMODEMASK_BIT5   = Int32(32) # 0x00000020
const     DIGMODEMASK_BIT4   = Int32(16) # 0x00000010
const     DIGMODEMASK_BIT3   = Int32(8) # 0x00000008
const     DIGMODEMASK_BIT2   = Int32(4) # 0x00000004
const     DIGMODEMASK_BIT1   = Int32(2) # 0x00000002
const     DIGMODEMASK_BIT0   = Int32(1) # 0x00000001

# provided for convenience
const SPCM_DIGMODE_CHREPLACE      = Int32(-4468737) # 0xFFBBCFFF
##define SPCM_DIGMODE_CHREPLACE    (  (DIGMODEMASK_BIT15 & SPCM_DIGMODE_X19)
#                                   | (DIGMODEMASK_BIT14 & SPCM_DIGMODE_X18)
#                                   | (DIGMODEMASK_BIT13 & SPCM_DIGMODE_X17)
#                                   | (DIGMODEMASK_BIT12 & SPCM_DIGMODE_X16)
#                                   | (DIGMODEMASK_BIT11 & SPCM_DIGMODE_X15)
#                                   | (DIGMODEMASK_BIT10 & SPCM_DIGMODE_X14)
#                                   | (DIGMODEMASK_BIT9  & SPCM_DIGMODE_X13)
#                                   | (DIGMODEMASK_BIT8  & SPCM_DIGMODE_X12)
#                                   | (DIGMODEMASK_BIT7  & SPCM_DIGMODE_X11)
#                                   | (DIGMODEMASK_BIT6  & SPCM_DIGMODE_X10)
#                                   | (DIGMODEMASK_BIT5  & SPCM_DIGMODE_X9 )
#                                   | (DIGMODEMASK_BIT4  & SPCM_DIGMODE_X8 )
#                                   | (DIGMODEMASK_BIT3  & SPCM_DIGMODE_X7 )
#                                   | (DIGMODEMASK_BIT2  & SPCM_DIGMODE_X6 )
#                                   | (DIGMODEMASK_BIT1  & SPCM_DIGMODE_X5 )
#                                   | (DIGMODEMASK_BIT0  & SPCM_DIGMODE_X4 ) )
#


# ----- M4x PXI Trigger lines -----
const SPC_PXITRG0_MODE            = Int32(47300)
const SPC_PXITRG1_MODE            = Int32(47301)
const SPC_PXITRG2_MODE            = Int32(47302)
const SPC_PXITRG3_MODE            = Int32(47303)
const SPC_PXITRG4_MODE            = Int32(47304)
const SPC_PXITRG5_MODE            = Int32(47305)
const SPC_PXITRG6_MODE            = Int32(47306)
const SPC_PXITRG7_MODE            = Int32(47307)
const SPC_PXISTAR_MODE            = Int32(47308)
const SPC_PXIDSTARC_MODE          = Int32(47309)
const SPC_PXITRG0_AVAILMODES      = Int32(47310)
const SPC_PXITRG1_AVAILMODES      = Int32(47311)
const SPC_PXITRG2_AVAILMODES      = Int32(47312)
const SPC_PXITRG3_AVAILMODES      = Int32(47313)
const SPC_PXITRG4_AVAILMODES      = Int32(47314)
const SPC_PXITRG5_AVAILMODES      = Int32(47315)
const SPC_PXITRG6_AVAILMODES      = Int32(47316)
const SPC_PXITRG7_AVAILMODES      = Int32(47317)
const SPC_PXISTAR_AVAILMODES      = Int32(47318)
const SPC_PXIDSTARC_AVAILMODES    = Int32(47319)
const SPC_PXITRG_ASYNCIO          = Int32(47320)          # asynchronous in/out register
const     SPCM_PXITRGMODE_DISABLE      = Int32(0) # 0x00000000
const     SPCM_PXITRGMODE_IN           = Int32(1) # 0x00000001  # used as input
const     SPCM_PXITRGMODE_ASYNCOUT     = Int32(2) # 0x00000002  # used as asynchronous output
const     SPCM_PXITRGMODE_RUNSTATE     = Int32(4) # 0x00000004  # shows the run state of the card (high = run)
const     SPCM_PXITRGMODE_ARMSTATE     = Int32(8) # 0x00000008  # shows the arm state (high = armed for trigger)
const     SPCM_PXITRGMODE_TRIGOUT      = Int32(16) # 0x00000010  # used as trigger output
const     SPCM_PXITRGMODE_REFCLKOUT    = Int32(32) # 0x00000020  # outputs PXI refclock (10 MHz)
const     SPCM_PXITRGMODE_CONTOUTMARK  = Int32(64) # 0x00000040  # outputs a half posttrigger long HIGH pulse on replay


# ----- Star-Hub -----
# 48000 not usable

const SPC_STARHUB_STATUS           = Int32(48010)

const SPC_STARHUB_ROUTE0           = Int32(48100)  # Routing Information for Test
const SPC_STARHUB_ROUTE99          = Int32(48199)  # ...


# Spcm driver (M2i, M3i, M4i, M4x, M2p) sync setup registers
const SPC_SYNC_READ_SYNCCOUNT      = Int32(48990)  # number of sync'd cards
const SPC_SYNC_READ_NUMCONNECTORS  = Int32(48991)  # number of connectors on starhub

const SPC_SYNC_READ_CARDIDX0       = Int32(49000)  # read index of card at location 0 of sync
const SPC_SYNC_READ_CARDIDX1       = Int32(49001)  # ...
const SPC_SYNC_READ_CARDIDX2       = Int32(49002)  # ...
const SPC_SYNC_READ_CARDIDX3       = Int32(49003)  # ...
const SPC_SYNC_READ_CARDIDX4       = Int32(49004)  # ...
const SPC_SYNC_READ_CARDIDX5       = Int32(49005)  # ...
const SPC_SYNC_READ_CARDIDX6       = Int32(49006)  # ...
const SPC_SYNC_READ_CARDIDX7       = Int32(49007)  # ...
const SPC_SYNC_READ_CARDIDX8       = Int32(49008)  # ...
const SPC_SYNC_READ_CARDIDX9       = Int32(49009)  # ...
const SPC_SYNC_READ_CARDIDX10      = Int32(49010)  # ...
const SPC_SYNC_READ_CARDIDX11      = Int32(49011)  # ...
const SPC_SYNC_READ_CARDIDX12      = Int32(49012)  # ...
const SPC_SYNC_READ_CARDIDX13      = Int32(49013)  # ...
const SPC_SYNC_READ_CARDIDX14      = Int32(49014)  # ...
const SPC_SYNC_READ_CARDIDX15      = Int32(49015)  # ...

const SPC_SYNC_READ_CABLECON0      = Int32(49100)  # read cable connection of card at location 0 of sync
const SPC_SYNC_READ_CABLECON1      = Int32(49101)  # ...
const SPC_SYNC_READ_CABLECON2      = Int32(49102)  # ...
const SPC_SYNC_READ_CABLECON3      = Int32(49103)  # ...
const SPC_SYNC_READ_CABLECON4      = Int32(49104)  # ...
const SPC_SYNC_READ_CABLECON5      = Int32(49105)  # ...
const SPC_SYNC_READ_CABLECON6      = Int32(49106)  # ...
const SPC_SYNC_READ_CABLECON7      = Int32(49107)  # ...
const SPC_SYNC_READ_CABLECON8      = Int32(49108)  # ...
const SPC_SYNC_READ_CABLECON9      = Int32(49109)  # ...
const SPC_SYNC_READ_CABLECON10     = Int32(49110)  # ...
const SPC_SYNC_READ_CABLECON11     = Int32(49111)  # ...
const SPC_SYNC_READ_CABLECON12     = Int32(49112)  # ...
const SPC_SYNC_READ_CABLECON13     = Int32(49113)  # ...
const SPC_SYNC_READ_CABLECON14     = Int32(49114)  # ...
const SPC_SYNC_READ_CABLECON15     = Int32(49115)  # ...

const SPC_SYNC_ENABLEMASK          = Int32(49200)  # synchronisation enable (mask)
const SPC_SYNC_NOTRIGSYNCMASK      = Int32(49210)  # trigger disabled for sync (mask)
const SPC_SYNC_CLKMASK             = Int32(49220)  # clock master (mask)
const SPC_SYNC_MODE                = Int32(49230)  # synchronization mode
const SPC_AVAILSYNC_MODES          = Int32(49231)  # available synchronization modes
const     SPC_SYNC_STANDARD          = Int32(1) # 0x00000001  # starhub uses its own clock and trigger sources
const     SPC_SYNC_SYSTEMCLOCK       = Int32(2) # 0x00000002  # starhub uses own trigger sources and takes clock from system starhub
const     SPC_SYNC_SYSTEMCLOCKTRIG   = Int32(4) # 0x00000004  # starhub takes clock and trigger from system starhub (trigger sampled on rising  clock edge)
const     SPC_SYNC_SYSTEMCLOCKTRIGN  = Int32(8) # 0x00000008  # starhub takes clock and trigger from system starhub (trigger sampled on falling clock edge)
const SPC_SYNC_SYSTEM_TRIGADJUST   = Int32(49240)  # Delay value for adjusting trigger position using system starhub


# ----- Gain and Offset Adjust DAC's -----
const SPC_ADJ_START                = Int32(50000)

const SPC_ADJ_LOAD                 = Int32(50000)
const SPC_ADJ_SAVE                 = Int32(50010)
const     ADJ_DEFAULT                  = Int32(0)
const     ADJ_USER0                    = Int32(1)
const     ADJ_USER1                    = Int32(2)
const     ADJ_USER2                    = Int32(3)
const     ADJ_USER3                    = Int32(4)
const     ADJ_USER4                    = Int32(5)
const     ADJ_USER5                    = Int32(6)
const     ADJ_USER6                    = Int32(7)
const     ADJ_USER7                    = Int32(8)

const SPC_ADJ_AUTOADJ              = Int32(50020)
const     ADJ_ALL                      = Int32(0)
const     ADJ_CURRENT                  = Int32(1)
const     ADJ_EXTERNAL                 = Int32(2)
const     ADJ_1MOHM                    = Int32(3)

const     ADJ_CURRENT_CLOCK            = Int32(4)
const     ADJ_CURRENT_IR               = Int32(8)
const     ADJ_OFFSET_ONLY             = Int32(16)
const     ADJ_SPECIAL_CLOCK           = Int32(32)

const SPC_ADJ_SOURCE_CALLBACK      = Int32(50021)
const SPC_ADJ_PROGRESS_CALLBACK    = Int32(50022)

const SPC_ADJ_SET                  = Int32(50030)
const SPC_ADJ_FAILMASK             = Int32(50040)

const SPC_ADJ_CALIBSOURCE             = Int32(50050)
const        ADJ_CALSRC_GAIN              = Int32(1)
const        ADJ_CALSRC_OFF               = Int32(0)
const        ADJ_CALSRC_GND              = Int32(-1)
const        ADJ_CALSRC_GNDOFFS          = Int32(-2)
const        ADJ_CALSRC_AC               = Int32(10)
const        ADJ_CALSRC_ADC              = Int32(11)

const SPC_ADJ_CALIBVALUE0             = Int32(50060)
const SPC_ADJ_CALIBVALUE1             = Int32(50061)
const SPC_ADJ_CALIBVALUE2             = Int32(50062)
const SPC_ADJ_CALIBVALUE3             = Int32(50063)
const SPC_ADJ_CALIBVALUE4             = Int32(50064)
const SPC_ADJ_CALIBVALUE5             = Int32(50065)
const SPC_ADJ_CALIBVALUE6             = Int32(50066)
const SPC_ADJ_CALIBVALUE7             = Int32(50067)

const SPC_ADJ_OFFSET_CH0           = Int32(50900)
const SPC_ADJ_OFFSET_CH1           = Int32(50901)
const SPC_ADJ_OFFSET_CH2           = Int32(50902)
const SPC_ADJ_OFFSET_CH3           = Int32(50903)
const SPC_ADJ_OFFSET_CH4           = Int32(50904)
const SPC_ADJ_OFFSET_CH5           = Int32(50905)
const SPC_ADJ_OFFSET_CH6           = Int32(50906)
const SPC_ADJ_OFFSET_CH7           = Int32(50907)
const SPC_ADJ_OFFSET_CH8           = Int32(50908)
const SPC_ADJ_OFFSET_CH9           = Int32(50909)
const SPC_ADJ_OFFSET_CH10          = Int32(50910)
const SPC_ADJ_OFFSET_CH11          = Int32(50911)
const SPC_ADJ_OFFSET_CH12          = Int32(50912)
const SPC_ADJ_OFFSET_CH13          = Int32(50913)
const SPC_ADJ_OFFSET_CH14          = Int32(50914)
const SPC_ADJ_OFFSET_CH15          = Int32(50915)

const SPC_ADJ_GAIN_CH0             = Int32(50916)
const SPC_ADJ_GAIN_CH1             = Int32(50917)
const SPC_ADJ_GAIN_CH2             = Int32(50918)
const SPC_ADJ_GAIN_CH3             = Int32(50919)
const SPC_ADJ_GAIN_CH4             = Int32(50920)
const SPC_ADJ_GAIN_CH5             = Int32(50921)
const SPC_ADJ_GAIN_CH6             = Int32(50922)
const SPC_ADJ_GAIN_CH7             = Int32(50923)
const SPC_ADJ_GAIN_CH8             = Int32(50924)
const SPC_ADJ_GAIN_CH9             = Int32(50925)
const SPC_ADJ_GAIN_CH10            = Int32(50926)
const SPC_ADJ_GAIN_CH11            = Int32(50927)
const SPC_ADJ_GAIN_CH12            = Int32(50928)
const SPC_ADJ_GAIN_CH13            = Int32(50929)
const SPC_ADJ_GAIN_CH14            = Int32(50930)
const SPC_ADJ_GAIN_CH15            = Int32(50931)

const SPC_ADJ_OFFSET0              = Int32(51000)
const SPC_ADJ_OFFSET999            = Int32(51999)

const SPC_ADJ_GAIN0                = Int32(52000)
const SPC_ADJ_GAIN999              = Int32(52999)

const SPC_ADJ_CORRECT0             = Int32(53000)
const SPC_ADJ_OFFS_CORRECT0        = Int32(53000)
const SPC_ADJ_CORRECT999           = Int32(53999)
const SPC_ADJ_OFFS_CORRECT999      = Int32(53999)

const SPC_ADJ_XIOOFFS0             = Int32(54000)
const SPC_ADJ_XIOOFFS1             = Int32(54001)
const SPC_ADJ_XIOOFFS2             = Int32(54002)
const SPC_ADJ_XIOOFFS3             = Int32(54003)

const SPC_ADJ_XIOGAIN0             = Int32(54010)
const SPC_ADJ_XIOGAIN1             = Int32(54011)
const SPC_ADJ_XIOGAIN2             = Int32(54012)
const SPC_ADJ_XIOGAIN3             = Int32(54013)

const SPC_ADJ_GAIN_CORRECT0        = Int32(55000)
const SPC_ADJ_GAIN_CORRECT999      = Int32(55999)

const SPC_ADJ_OFFSCALIBCORRECT0    = Int32(56000)
const SPC_ADJ_OFFSCALIBCORRECT999  = Int32(56999)

const SPC_ADJ_GAINCALIBCORRECT0    = Int32(57000)
const SPC_ADJ_GAINCALIBCORRECT999  = Int32(57999)

const SPC_ADJ_ANALOGTRIGGER0       = Int32(58000)
const SPC_ADJ_ANALOGTRIGGER99      = Int32(58099)

const SPC_ADJ_CALIBSAMPLERATE0     = Int32(58100)
const SPC_ADJ_CALIBSAMPLERATE99    = Int32(58199)

const SPC_ADJ_CALIBSAMPLERATE_GAIN0     = Int32(58200)
const SPC_ADJ_CALIBSAMPLERATE_GAIN99    = Int32(58299)

const SPC_ADJ_REFCLOCK             = Int32(58300)
const SPC_ADJ_STARHUB_REFCLOCK     = Int32(58301)

const SPC_ADJ_TIMING_CH0           = Int32(58400)
const SPC_ADJ_TIMING_CH1           = Int32(58401)

const SPC_ADJ_END                  = Int32(59999)



# ----- FIFO Control -----
const SPC_FIFO_BUFFERS             = Int32(60000)          # number of FIFO buffers
const SPC_FIFO_BUFLEN              = Int32(60010)          # len of each FIFO buffer
const SPC_FIFO_BUFCOUNT            = Int32(60020)          # number of FIFO buffers tranfered until now
const SPC_FIFO_BUFMAXCNT           = Int32(60030)          # number of FIFO buffers to be transfered (0=continuous)
const SPC_FIFO_BUFADRCNT           = Int32(60040)          # number of FIFO buffers allowed
const SPC_FIFO_BUFREADY            = Int32(60050)          # fifo buffer ready register (same as SPC_COMMAND + SPC_FIFO_BUFREADY0...)
const SPC_FIFO_BUFFILLCNT          = Int32(60060)          # number of currently filled buffers
const SPC_FIFO_BUFADR0             = Int32(60100)          # adress of FIFO buffer no. 0
const SPC_FIFO_BUFADR1             = Int32(60101)          # ...
const SPC_FIFO_BUFADR2             = Int32(60102)          # ...
const SPC_FIFO_BUFADR3             = Int32(60103)          # ...
const SPC_FIFO_BUFADR4             = Int32(60104)          # ...
const SPC_FIFO_BUFADR5             = Int32(60105)          # ...
const SPC_FIFO_BUFADR6             = Int32(60106)          # ...
const SPC_FIFO_BUFADR7             = Int32(60107)          # ...
const SPC_FIFO_BUFADR8             = Int32(60108)          # ...
const SPC_FIFO_BUFADR9             = Int32(60109)          # ...
const SPC_FIFO_BUFADR10            = Int32(60110)          # ...
const SPC_FIFO_BUFADR11            = Int32(60111)          # ...
const SPC_FIFO_BUFADR12            = Int32(60112)          # ...
const SPC_FIFO_BUFADR13            = Int32(60113)          # ...
const SPC_FIFO_BUFADR14            = Int32(60114)          # ...
const SPC_FIFO_BUFADR15            = Int32(60115)          # ...
const SPC_FIFO_BUFADR255           = Int32(60355)          # last



# ----- Filter -----
const SPC_FILTER                   = Int32(100000)
const SPC_READNUMFILTERS           = Int32(100001)         # number of programable filters
const SPC_FILTERFREQUENCY0         = Int32(100002)         # frequency of filter 0 (bypass)
const SPC_FILTERFREQUENCY1         = Int32(100003)         # frequency of filter 1
const SPC_FILTERFREQUENCY2         = Int32(100004)         # frequency of filter 2
const SPC_FILTERFREQUENCY3         = Int32(100005)         # frequency of filter 3
const SPC_DIGITALBWFILTER          = Int32(100100)         # enable/disable digital bandwith filter


# ----- Pattern -----
const SPC_PATTERNENABLE            = Int32(110000)
const SPC_READDIGITAL              = Int32(110100)

const SPC_DIGITALMODE0             = Int32(110200)
const SPC_DIGITALMODE1             = Int32(110201)
const SPC_DIGITALMODE2             = Int32(110202)
const SPC_DIGITALMODE3             = Int32(110203)
const SPC_DIGITALMODE4             = Int32(110204)
const SPC_DIGITALMODE5             = Int32(110205)
const SPC_DIGITALMODE6             = Int32(110206)
const SPC_DIGITALMODE7             = Int32(110207)
const     SPC_DIGITALMODE_OFF          = Int32(0)
const     SPC_DIGITALMODE_2BIT         = Int32(1)
const     SPC_DIGITALMODE_4BIT         = Int32(2)
const     SPC_DIGITALMODE_CHREPLACE    = Int32(3)


# ----- Miscellanous -----
const SPC_MISCDAC0                 = Int32(200000)
const SPC_MISCDAC1                 = Int32(200010)
const SPC_FACTORYMODE              = Int32(200020)
const SPC_DIRECTDAC                = Int32(200030)
const SPC_NOTRIGSYNC               = Int32(200040)
const SPC_DSPDIRECT                = Int32(200100)
const SPC_DMAPHYSICALADR           = Int32(200110)
const SPC_MICXCOMP_CLOSEBOARD      = Int32(200119)
const SPC_MICXCOMPATIBILITYMODE    = Int32(200120)
const SPC_TEST_FIFOSPEED           = Int32(200121)
const SPC_RELOADDEMO               = Int32(200122)
const SPC_OVERSAMPLINGFACTOR       = Int32(200123)
const SPC_ISMAPPEDCARD             = Int32(200124)
const     SPCM_NOT_MAPPED              = Int32(0)
const     SPCM_LOCAL_MAPPED            = Int32(1)
const     SPCM_REMOTE_MAPPED           = Int32(2)
const SPC_GETTHREADHANDLE          = Int32(200130)
const SPC_GETKERNELHANDLE          = Int32(200131)
const SPC_XYZMODE                  = Int32(200200)
const SPC_INVERTDATA               = Int32(200300)
const SPC_GATEMARKENABLE           = Int32(200400)
const SPC_GATE_LEN_ALIGNMENT       = Int32(200401)
const SPC_CONTOUTMARK              = Int32(200450)
const SPC_EXPANDINT32              = Int32(200500)
const SPC_NOPRETRIGGER             = Int32(200600)
const SPC_RELAISWAITTIME           = Int32(200700)
const SPC_DACWAITTIME              = Int32(200710)
const SPC_DELAY_US                 = Int32(200720)
const SPC_ILAMODE                  = Int32(200800)
const SPC_NMDGMODE                 = Int32(200810)
const SPC_CKADHALF_OUTPUT          = Int32(200820)
const SPC_LONGTRIG_OUTPUT          = Int32(200830)
const SPC_STOREMODAENDOFSEGMENT    = Int32(200840)
const SPC_COUNTERMODE              = Int32(200850)
const     SPC_CNTMOD_MASK              = Int32(15) # 0x0000000F
const     SPC_CNTMOD_PARALLELDATA      = Int32(0) # 0x00000000
const     SPC_CNTMOD_8BITCNT           = Int32(1) # 0x00000001
const     SPC_CNTMOD_2x8BITCNT         = Int32(2) # 0x00000002
const     SPC_CNTMOD_16BITCNT          = Int32(3) # 0x00000003
const     SPC_CNT0_MASK                = Int32(240) # 0x000000F0
const     SPC_CNT0_CNTONPOSEDGE        = Int32(0) # 0x00000000
const     SPC_CNT0_CNTONNEGEDGE        = Int32(16) # 0x00000010
const     SPC_CNT0_RESETHIGHLVL        = Int32(0) # 0x00000000
const     SPC_CNT0_RESETLOWLVL         = Int32(32) # 0x00000020
const     SPC_CNT0_STOPATMAX           = Int32(0) # 0x00000000
const     SPC_CNT0_ROLLOVER            = Int32(64) # 0x00000040
const     SPC_CNT1_MASK                = Int32(3840) # 0x00000F00
const     SPC_CNT1_CNTONPOSEDGE        = Int32(0) # 0x00000000
const     SPC_CNT1_CNTONNEGEDGE        = Int32(256) # 0x00000100
const     SPC_CNT1_RESETHIGHLVL        = Int32(0) # 0x00000000
const     SPC_CNT1_RESETLOWLVL         = Int32(512) # 0x00000200
const     SPC_CNT1_STOPATMAX           = Int32(0) # 0x00000000
const     SPC_CNT1_ROLLOVER            = Int32(1024) # 0x00000400
const     SPC_CNTCMD_MASK              = Int32(61440) # 0x0000F000
const     SPC_CNTCMD_RESETCNT0         = Int32(4096) # 0x00001000
const     SPC_CNTCMD_RESETCNT1         = Int32(8192) # 0x00002000
const SPC_ENHANCEDSTATUS           = Int32(200900)
const     SPC_ENHSTAT_OVERRANGE0       = Int32(1) # 0x00000001
const     SPC_ENHSTAT_OVERRANGE1       = Int32(2) # 0x00000002
const     SPC_ENHSTAT_OVERRANGE2       = Int32(4) # 0x00000004
const     SPC_ENHSTAT_OVERRANGE3       = Int32(8) # 0x00000008
const     SPC_ENHSTAT_OVERRANGE4       = Int32(16) # 0x00000010
const     SPC_ENHSTAT_OVERRANGE5       = Int32(32) # 0x00000020
const     SPC_ENHSTAT_OVERRANGE6       = Int32(64) # 0x00000040
const     SPC_ENHSTAT_OVERRANGE7       = Int32(128) # 0x00000080
const     SPC_ENHSTAT_COMPARATOR0      = Int32(1073741824) # 0x40000000
const     SPC_ENHSTAT_COMPARATOR1      = Int32(-2147483648) # 0x80000000
const     SPC_ENHSTAT_COMPARATOR2      = Int32(536870912) # 0x20000000
const     SPC_ENHSTAT_TRGCOMPARATOR    = Int32(1073741824) # 0x40000000
const     SPC_ENHSTAT_CLKCOMPARATOR    = Int32(-2147483648) # 0x80000000
const SPC_TRIGGERCOUNTER           = Int32(200905)
const SPC_FILLSIZEPROMILLE         = Int32(200910)
const SPC_OVERRANGEBIT             = Int32(201000)
const SPC_2CH8BITMODE              = Int32(201100)
const SPC_12BITMODE                = Int32(201200)
const SPC_HOLDLASTSAMPLE           = Int32(201300)

const SPC_DATACONVERSION           = Int32(201400)
const SPC_AVAILDATACONVERSION      = Int32(201401)
const     SPCM_DC_NONE             = Int32(0) # 0x00000000
const     SPCM_DC_12BIT_TO_14BIT   = Int32(1) # 0x00000001
const     SPCM_DC_16BIT_TO_14BIT   = Int32(2) # 0x00000002
const     SPCM_DC_12BIT_TO_16BIT   = Int32(4) # 0x00000004
const     SPCM_DC_14BIT_TO_16BIT   = Int32(8) # 0x00000008
const     SPCM_DC_15BIT_TO_16BIT   = Int32(16) # 0x00000010
const     SPCM_DC_13BIT_TO_16BIT   = Int32(32) # 0x00000020
const     SPCM_DC_12BIT_TO_8BIT    = Int32(128) # 0x00000080
const     SPCM_DC_14BIT_TO_8BIT    = Int32(256) # 0x00000100
const     SPCM_DC_16BIT_TO_8BIT    = Int32(512) # 0x00000200
const     SPCM_DC_16BIT_TO_12BIT   = Int32(1024) # 0x00000400
const     SPCM_DC_TO_OFFSETBINARY  = Int32(2048) # 0x00000800

const SPC_CARDIDENTIFICATION       = Int32(201500)

const SPC_HANDSHAKE                = Int32(201600)

const SPC_CKSYNC0                  = Int32(202000)
const SPC_CKSYNC1                  = Int32(202001)
const SPC_DISABLEMOD0              = Int32(203000)
const SPC_DISABLEMOD1              = Int32(203010)
const SPC_ENABLEOVERRANGECHECK     = Int32(204000)
const SPC_OVERRANGESTATUS          = Int32(204010)
const SPC_BITMODE                  = Int32(205000)

const SPC_READBACK                 = Int32(206000)
const SPC_AVAILSTOPLEVEL           = Int32(206009)
const SPC_STOPLEVEL1               = Int32(206010)
const SPC_STOPLEVEL0               = Int32(206020)
const SPC_CH0_STOPLEVEL            = Int32(206020)
const SPC_CH1_STOPLEVEL            = Int32(206021)
const SPC_CH2_STOPLEVEL            = Int32(206022)
const SPC_CH3_STOPLEVEL            = Int32(206023)
const SPC_CH4_STOPLEVEL            = Int32(206024)
const SPC_CH5_STOPLEVEL            = Int32(206025)
const SPC_CH6_STOPLEVEL            = Int32(206026)
const SPC_CH7_STOPLEVEL            = Int32(206027)
const     SPCM_STOPLVL_TRISTATE    = Int32(1) # 0x00000001
const     SPCM_STOPLVL_LOW         = Int32(2) # 0x00000002
const     SPCM_STOPLVL_HIGH        = Int32(4) # 0x00000004
const     SPCM_STOPLVL_HOLDLAST    = Int32(8) # 0x00000008
const     SPCM_STOPLVL_ZERO        = Int32(16) # 0x00000010
const     SPCM_STOPLVL_CUSTOM      = Int32(32) # 0x00000020

const SPC_DIFFMODE                 = Int32(206030)
const SPC_DACADJUST                = Int32(206040)

const SPC_CH0_CUSTOM_STOP          = Int32(206050)
const SPC_CH1_CUSTOM_STOP          = Int32(206051)
const SPC_CH2_CUSTOM_STOP          = Int32(206052)
const SPC_CH3_CUSTOM_STOP          = Int32(206053)
const SPC_CH4_CUSTOM_STOP          = Int32(206054)
const SPC_CH5_CUSTOM_STOP          = Int32(206055)
const SPC_CH6_CUSTOM_STOP          = Int32(206056)
const SPC_CH7_CUSTOM_STOP          = Int32(206057)

const SPC_AMP_MODE                 = Int32(207000)

const SPCM_FW_CTRL                 = Int32(210000)
const SPCM_FW_CTRL_GOLDEN          = Int32(210001)
const SPCM_FW_CTRL_ACTIVE          = Int32(210002)
const SPCM_FW_CLOCK                = Int32(210010)
const SPCM_FW_CONFIG               = Int32(210020)
const SPCM_FW_MODULEA              = Int32(210030)
const SPCM_FW_MODULEB              = Int32(210031)
const SPCM_FW_MODULEA_ACTIVE       = Int32(210032)
const SPCM_FW_MODULEB_ACTIVE       = Int32(210033)
const SPCM_FW_MODEXTRA             = Int32(210050)
const SPCM_FW_MODEXTRA_ACTIVE      = Int32(210052)
const SPCM_FW_POWER                = Int32(210060)
const SPCM_FW_POWER_ACTIVE         = Int32(210062)

const SPC_MULTI                    = Int32(220000)
const SPC_DOUBLEMEM                = Int32(220100)
const SPC_MULTIMEMVALID            = Int32(220200)
const SPC_BANK                     = Int32(220300)
const SPC_GATE                     = Int32(220400)
const SPC_RELOAD                   = Int32(230000)
const SPC_USEROUT                  = Int32(230010)
const SPC_WRITEUSER0               = Int32(230100)
const SPC_WRITEUSER1               = Int32(230110)
const SPC_READUSER0                = Int32(230200)
const SPC_READUSER1                = Int32(230210)
const SPC_MUX                      = Int32(240000)
const SPC_ADJADC                   = Int32(241000)
const SPC_ADJOFFS0                 = Int32(242000)
const SPC_ADJOFFS1                 = Int32(243000)
const SPC_ADJGAIN0                 = Int32(244000)
const SPC_ADJGAIN1                 = Int32(245000)
const SPC_READEPROM                = Int32(250000)
const SPC_WRITEEPROM               = Int32(250010)
const SPC_DIRECTIO                 = Int32(260000)
const SPC_DIRECT_MODA              = Int32(260010)
const SPC_DIRECT_MODB              = Int32(260020)
const SPC_DIRECT_EXT0              = Int32(260030)
const SPC_DIRECT_EXT1              = Int32(260031)
const SPC_DIRECT_EXT2              = Int32(260032)
const SPC_DIRECT_EXT3              = Int32(260033)
const SPC_DIRECT_EXT4              = Int32(260034)
const SPC_DIRECT_EXT5              = Int32(260035)
const SPC_DIRECT_EXT6              = Int32(260036)
const SPC_DIRECT_EXT7              = Int32(260037)
const SPC_MEMTEST                  = Int32(270000)
const SPC_NODMA                    = Int32(275000)
const SPC_NOCOUNTER                = Int32(275010)
const SPC_NOSCATTERGATHER          = Int32(275020)
const SPC_USER_RELAIS_OVERWRITE    = Int32(275030)
const     SPCM_URO_ENABLE              = Int32(-2147483648) # 0x80000000
const     SPCM_URO_INVERT_10TO1REL     = Int32(1) # 0x00000001
const SPC_RUNINTENABLE             = Int32(290000)
const SPC_XFERBUFSIZE              = Int32(295000)
const SPC_CHLX                     = Int32(295010)
const SPC_SPECIALCLOCK             = Int32(295100)
const SPC_PLL0_ICP                 = Int32(295105)
const     SPCM_ICP0             = Int32(0) # 0x00000000
# ...
const     SPCM_ICP7             = Int32(7) # 0x00000007
const SPC_STARTDELAY               = Int32(295110)
const SPC_BASISTTLTRIG             = Int32(295120)
const SPC_TIMEOUT                  = Int32(295130)
const SPC_SWL_INFO                 = Int32(295140)
const SPC_SWD_INFO                 = Int32(295141)
const SPC_SWD_DOWN                 = Int32(295142)
const SPC_SWL_EXTRAINFO            = Int32(295143)
const SPC_SPECIALCLOCK_ADJUST0     = Int32(295150)
const SPC_SPECIALCLOCK_ADJUST1     = Int32(295151)
const SPC_SPECIALCLOCK_ADJUST2     = Int32(295152)
const SPC_SPECIALCLOCK_ADJUST3     = Int32(295153)
const    SPCM_SPECIALCLOCK_ADJUST_SHIFT  = Int32(1000000)
const SPC_REGACC_CONTMEM           = Int32(299000)
const SPC_REGACC_MEMORYUSAGE       = Int32(299001)
const SPC_REINITLOGSETTINGS        = Int32(299998)
const SPC_LOGDLLCALLS              = Int32(299999)






# ----- PCK400 -----
const SPC_FREQUENCE                = Int32(300000)
const SPC_DELTAFREQUENCE           = Int32(300010)
const SPC_PINHIGH                  = Int32(300100)
const SPC_PINLOW                   = Int32(300110)
const SPC_PINDELTA                 = Int32(300120)
const SPC_STOPLEVEL                = Int32(300200)
const SPC_PINRELAIS                = Int32(300210)
const SPC_EXTERNLEVEL              = Int32(300300)



# ----- PADCO -----
const SPC_COUNTER0                 = Int32(310000)
const SPC_COUNTER1                 = Int32(310001)
const SPC_COUNTER2                 = Int32(310002)
const SPC_COUNTER3                 = Int32(310003)
const SPC_COUNTER4                 = Int32(310004)
const SPC_COUNTER5                 = Int32(310005)
const SPC_MODE0                    = Int32(310100)
const SPC_MODE1                    = Int32(310101)
const SPC_MODE2                    = Int32(310102)
const SPC_MODE3                    = Int32(310103)
const SPC_MODE4                    = Int32(310104)
const SPC_MODE5                    = Int32(310105)
const     CM_SINGLE                    = Int32(1)
const     CM_MULTI                     = Int32(2)
const     CM_POSEDGE                   = Int32(4)
const     CM_NEGEDGE                   = Int32(8)
const     CM_HIGHPULSE                 = Int32(16)
const     CM_LOWPULSE                  = Int32(32)



# ----- PAD1616 -----
const SPC_SEQUENCERESET            = Int32(320000)
const SPC_SEQUENCEADD              = Int32(320010)
const     SEQ_IR_10000MV               = Int32(0)
const     SEQ_IR_5000MV                = Int32(1)
const     SEQ_IR_2000MV                = Int32(2)
const     SEQ_IR_1000MV                = Int32(3)
const     SEQ_IR_500MV                 = Int32(4)
const     SEQ_CH0                      = Int32(0)
const     SEQ_CH1                      = Int32(8)
const     SEQ_CH2                      = Int32(16)
const     SEQ_CH3                      = Int32(24)
const     SEQ_CH4                      = Int32(32)
const     SEQ_CH5                      = Int32(40)
const     SEQ_CH6                      = Int32(48)
const     SEQ_CH7                      = Int32(56)
const     SEQ_CH8                      = Int32(64)
const     SEQ_CH9                      = Int32(72)
const     SEQ_CH10                     = Int32(80)
const     SEQ_CH11                     = Int32(88)
const     SEQ_CH12                     = Int32(96)
const     SEQ_CH13                     = Int32(104)
const     SEQ_CH14                     = Int32(112)
const     SEQ_CH15                     = Int32(120)
const     SEQ_TRIGGER                  = Int32(128)
const     SEQ_START                    = Int32(256)



# ----- Option CA -----
const SPC_CA_MODE                  = Int32(330000)
const     CAMODE_OFF                   = Int32(0)
const     CAMODE_CDM                   = Int32(1)
const     CAMODE_KW                    = Int32(2)
const     CAMODE_OT                    = Int32(3)
const     CAMODE_CDMMUL                = Int32(4)
const SPC_CA_TRIGDELAY             = Int32(330010)
const SPC_CA_CKDIV                 = Int32(330020)
const SPC_CA_PULS                  = Int32(330030)
const SPC_CA_CKMUL                 = Int32(330040)
const SPC_CA_DREHZAHLFORMAT        = Int32(330050)
const     CADREH_4X4                   = Int32(0)
const     CADREH_1X16                  = Int32(1)
const SPC_CA_KWINVERT              = Int32(330060)
const SPC_CA_OUTA                  = Int32(330100)
const SPC_CA_OUTB                  = Int32(330110)
const     CAOUT_TRISTATE               = Int32(0)
const     CAOUT_LOW                    = Int32(1)
const     CAOUT_HIGH                   = Int32(2)
const     CAOUT_CDM                    = Int32(3)
const     CAOUT_OT                     = Int32(4)
const     CAOUT_KW                     = Int32(5)
const     CAOUT_TRIG                   = Int32(6)
const     CAOUT_CLK                    = Int32(7)
const     CAOUT_KW60                   = Int32(8)
const     CAOUT_KWGAP                  = Int32(9)
const     CAOUT_TRDLY                  = Int32(10)
const     CAOUT_INVERT                 = Int32(16)


# ----- Option Sequence Mode (output cards) -----
const SPC_SEQMODE_STEPMEM0         = Int32(340000)
# ... 
const SPC_SEQMODE_STEPMEM8191      = Int32(348191)

# low part of 64 bit entry
const     SPCSEQ_SEGMENTMASK       = Int32(65535) # 0x0000FFFF
const     SPCSEQ_NEXTSTEPMASK      = Int32(-65536) # 0xFFFF0000

# high part of 64 bit entry
const     SPCSEQ_LOOPMASK          = Int32(1048575) # 0x000FFFFF
const     SPCSEQ_ENDLOOPALWAYS     = Int32(0) # 0x00000000
const     SPCSEQ_ENDLOOPONTRIG     = Int32(1073741824) # 0x40000000
const     SPCSEQ_END               = Int32(-2147483648) # 0x80000000

const SPC_SEQMODE_AVAILMAXSEGMENT  = Int32(349900)
const SPC_SEQMODE_AVAILMAXSTEPS    = Int32(349901)
const SPC_SEQMODE_AVAILMAXLOOP     = Int32(349902)
const SPC_SEQMODE_AVAILFEATURES    = Int32(349903)

const SPC_SEQMODE_MAXSEGMENTS      = Int32(349910)
const SPC_SEQMODE_WRITESEGMENT     = Int32(349920)
const SPC_SEQMODE_STARTSTEP        = Int32(349930)
const SPC_SEQMODE_SEGMENTSIZE      = Int32(349940)

const SPC_SEQMODE_STATUS           = Int32(349950)
const     SEQSTAT_STEPCHANGE           = Int32(-2147483648) # 0x80000000


# ----- netbox registers -----
const SPC_NETBOX_TYPE              = Int32(400000)
const     NETBOX_SERIES_MASK       = Int32(-16777216) # 0xFF000000
const     NETBOX_FAMILY_MASK       = Int32(16711680) # 0x00FF0000
const     NETBOX_SPEED_MASK        = Int32(65280) # 0x0000FF00
const     NETBOX_CHANNEL_MASK      = Int32(255) # 0x000000FF

const     NETBOX_SERIES_DN2        = Int32(33554432) # 0x02000000
const     NETBOX_SERIES_DN6        = Int32(100663296) # 0x06000000

const     NETBOX_FAMILY_20         = Int32(2097152) # 0x00200000
const     NETBOX_FAMILY_22         = Int32(2228224) # 0x00220000
const     NETBOX_FAMILY_44         = Int32(4456448) # 0x00440000
const     NETBOX_FAMILY_46         = Int32(4587520) # 0x00460000
const     NETBOX_FAMILY_47         = Int32(4653056) # 0x00470000
const     NETBOX_FAMILY_48         = Int32(4718592) # 0x00480000
const     NETBOX_FAMILY_49         = Int32(4784128) # 0x00490000
const     NETBOX_FAMILY_59         = Int32(5832704) # 0x00590000
const     NETBOX_FAMILY_60         = Int32(6291456) # 0x00600000
const     NETBOX_FAMILY_65         = Int32(6619136) # 0x00650000
const     NETBOX_FAMILY_66         = Int32(6684672) # 0x00660000
const     NETBOX_FAMILY_8X         = Int32(8388608) # 0x00800000
const     NETBOX_FAMILY_80         = Int32(8388608) # 0x00800000
const     NETBOX_FAMILY_81         = Int32(8454144) # 0x00810000
const     NETBOX_FAMILY_82         = Int32(8519680) # 0x00820000
const     NETBOX_FAMILY_83         = Int32(8585216) # 0x00830000

const     NETBOX_SPEED_1           = Int32(256) # 0x00000100
const     NETBOX_SPEED_2           = Int32(512) # 0x00000200
const     NETBOX_SPEED_3           = Int32(768) # 0x00000300
const     NETBOX_SPEED_4           = Int32(1024) # 0x00000400
const     NETBOX_SPEED_5           = Int32(1280) # 0x00000500
const     NETBOX_SPEED_6           = Int32(1536) # 0x00000600
const     NETBOX_SPEED_7           = Int32(1792) # 0x00000700
const     NETBOX_SPEED_8           = Int32(2048) # 0x00000800

const     NETBOX_CHANNELS_2        = Int32(2) # 0x00000002
const     NETBOX_CHANNELS_4        = Int32(4) # 0x00000004
const     NETBOX_CHANNELS_6        = Int32(6) # 0x00000006
const     NETBOX_CHANNELS_8        = Int32(8) # 0x00000008
const     NETBOX_CHANNELS_10       = Int32(10) # 0x0000000A
const     NETBOX_CHANNELS_12       = Int32(12) # 0x0000000C
const     NETBOX_CHANNELS_16       = Int32(16) # 0x00000010
const     NETBOX_CHANNELS_20       = Int32(20) # 0x00000014
const     NETBOX_CHANNELS_24       = Int32(24) # 0x00000018
const     NETBOX_CHANNELS_32       = Int32(32) # 0x00000020
const     NETBOX_CHANNELS_40       = Int32(40) # 0x00000028
const     NETBOX_CHANNELS_48       = Int32(48) # 0x00000030

const SPC_NETBOX_SERIALNO          = Int32(400001)
const SPC_NETBOX_PRODUCTIONDATE    = Int32(400002)
const SPC_NETBOX_HWVERSION         = Int32(400003)
const SPC_NETBOX_SWVERSION         = Int32(400004)

const SPC_NETBOX_FEATURES          = Int32(400005)
const     NETBOX_FEAT_DCPOWER          = Int32(1) # 0x1
const     NETBOX_FEAT_BOOTATPOWERON    = Int32(2) # 0x2
const     NETBOX_FEAT_EMBEDDEDSERVER   = Int32(4) # 0x4

const SPC_NETBOX_CUSTOM            = Int32(400006)

const SPC_NETBOX_WAKEONLAN         = Int32(400007)
const SPC_NETBOX_MACADDRESS        = Int32(400008)
const SPC_NETBOX_LANIDFLASH        = Int32(400009)
const SPC_NETBOX_TEMPERATURE       = Int32(400010)
const SPC_NETBOX_SHUTDOWN          = Int32(400011)
const SPC_NETBOX_RESTART           = Int32(400012)
const SPC_NETBOX_FANSPEED0         = Int32(400013)
const SPC_NETBOX_FANSPEED1         = Int32(400014)
const SPC_NETBOX_TEMPERATURE_K     = Int32(400010) # same SPC_NETBOX_TEMPERATURE
const SPC_NETBOX_TEMPERATURE_C     = Int32(400015)
const SPC_NETBOX_TEMPERATURE_F     = Int32(400016)
const SPC_NETBOX_TEMPERATURE1_K    = Int32(400017)
const SPC_NETBOX_TEMPERATURE1_C    = Int32(400018)
const SPC_NETBOX_TEMPERATURE1_F    = Int32(400019)
const SPC_NETBOX_TEMPERATURE2_K    = Int32(400020)
const SPC_NETBOX_TEMPERATURE2_C    = Int32(400021)
const SPC_NETBOX_TEMPERATURE2_F    = Int32(400022)

# ----- hardware monitor registers -----
const SPC_MON_V_PCIE_BUS           = Int32(500000)
const SPC_MON_V_CONNECTOR          = Int32(500001)
const SPC_MON_CARD_PWRSOURCE       = Int32(500002)
const     CARD_PWRSOURCE_BUS           = Int32(0)
const     CARD_PWRSOURCE_CONNECTOR     = Int32(1)
const SPC_MON_V_CARD_IN            = Int32(500003)
const SPC_MON_I_CARD_IN            = Int32(500004)
const SPC_MON_P_CARD_IN            = Int32(500005)
const SPC_MON_V_3V3                = Int32(500006)
const SPC_MON_V_2V5                = Int32(500007)
const SPC_MON_V_CORE               = Int32(500008)
const SPC_MON_V_AVTT               = Int32(500009)
const SPC_MON_V_AVCC               = Int32(500010)
const SPC_MON_V_MEMVCC             = Int32(500011)
const SPC_MON_V_MEMVTT             = Int32(500012)
const SPC_MON_V_CP_POS             = Int32(500013)
const SPC_MON_V_CP_NEG             = Int32(500014)

const SPC_MON_V_5VA                = Int32(500015)
const SPC_MON_V_ADCA               = Int32(500016)
const SPC_MON_V_ADCD               = Int32(500017)
const SPC_MON_V_OP_POS             = Int32(500018)
const SPC_MON_V_OP_NEG             = Int32(500019)
const SPC_MON_V_COMP_NEG           = Int32(500020)
const SPC_MON_V_COMP_POS           = Int32(500021)

# legacy temperature registers (Kelvin)
const SPC_MON_T_BASE_CTRL          = Int32(500022)
const SPC_MON_T_MODULE_0           = Int32(500023)
const SPC_MON_T_MODULE_1           = Int32(500024)

# new temperature registers for Kelvin (TK), Celsius (TC) or Fahrenheit (TF)
const SPC_MON_TK_BASE_CTRL          = Int32(500022)
const SPC_MON_TK_MODULE_0           = Int32(500023)
const SPC_MON_TK_MODULE_1           = Int32(500024)

const SPC_MON_TC_BASE_CTRL          = Int32(500025)
const SPC_MON_TC_MODULE_0           = Int32(500026)
const SPC_MON_TC_MODULE_1           = Int32(500027)

const SPC_MON_TF_BASE_CTRL          = Int32(500028)
const SPC_MON_TF_MODULE_0           = Int32(500029)
const SPC_MON_TF_MODULE_1           = Int32(500030)

# some more voltages (used on M2p)
const SPC_MON_V_1V8_BASE            = Int32(500031)
const SPC_MON_V_1V8_MOD             = Int32(500032)
const SPC_MON_V_MODA_0              = Int32(500033)
const SPC_MON_V_MODA_1              = Int32(500034)
const SPC_MON_V_MODB_0              = Int32(500035)
const SPC_MON_V_MODB_1              = Int32(500037)

# some more voltages and temperatures (used on M2p.65xx-hv)
const SPC_MON_TK_MODA_0            = Int32(500023) # same as SPC_MON_TK_MODULE_0
const SPC_MON_TK_MODA_1            = Int32(500038)
const SPC_MON_TK_MODA_2            = Int32(500039)
const SPC_MON_TK_MODA_3            = Int32(500040)
const SPC_MON_TK_MODA_4            = Int32(500041)
const SPC_MON_TK_MODB_0            = Int32(500024) # same as SPC_MON_TK_MODULE_1
const SPC_MON_TK_MODB_1            = Int32(500042)
const SPC_MON_TK_MODB_2            = Int32(500043)
const SPC_MON_TK_MODB_3            = Int32(500044)
const SPC_MON_TK_MODB_4            = Int32(500045)

const SPC_MON_TC_MODA_0            = Int32(500026) # same as SPC_MON_TC_MODULE_0
const SPC_MON_TC_MODA_1            = Int32(500046)
const SPC_MON_TC_MODA_2            = Int32(500047)
const SPC_MON_TC_MODA_3            = Int32(500048)
const SPC_MON_TC_MODA_4            = Int32(500049)
const SPC_MON_TC_MODB_0            = Int32(500027) # same as SPC_MON_TC_MODULE_1
const SPC_MON_TC_MODB_1            = Int32(500050)
const SPC_MON_TC_MODB_2            = Int32(500051)
const SPC_MON_TC_MODB_3            = Int32(500052)
const SPC_MON_TC_MODB_4            = Int32(500053)

const SPC_MON_TF_MODA_0            = Int32(500029) # same as SPC_MON_TF_MODULE_0
const SPC_MON_TF_MODA_1            = Int32(500054)
const SPC_MON_TF_MODA_2            = Int32(500055)
const SPC_MON_TF_MODA_3            = Int32(500056)
const SPC_MON_TF_MODA_4            = Int32(500057)
const SPC_MON_TF_MODB_0            = Int32(500030) # same as SPC_MON_TF_MODULE_1
const SPC_MON_TF_MODB_1            = Int32(500058)
const SPC_MON_TF_MODB_2            = Int32(500059)
const SPC_MON_TF_MODB_3            = Int32(500060)
const SPC_MON_TF_MODB_4            = Int32(500061)

const SPC_MON_I_MODA_0             = Int32(500062)
const SPC_MON_I_MODA_1             = Int32(500063)
const SPC_MON_I_MODA_2             = Int32(500064)
const SPC_MON_I_MODA_3             = Int32(500065)
const SPC_MON_I_MODB_0             = Int32(500066)
const SPC_MON_I_MODB_1             = Int32(500067)
const SPC_MON_I_MODB_2             = Int32(500068)
const SPC_MON_I_MODB_3             = Int32(500069)

const SPC_MON_MOD_FAULT            = Int32(500070)
const SPC_CLR_MOD_FAULT            = Int32(500071)

# power section temperature registers for Kelvin (TK), Celsius (TC) or Fahrenheit (TF)
const SPC_MON_TK_MODA_5            = Int32(500072)
const SPC_MON_TK_MODB_5            = Int32(500073)

const SPC_MON_TC_MODA_5            = Int32(500074)
const SPC_MON_TC_MODB_5            = Int32(500075)

const SPC_MON_TF_MODA_5            = Int32(500076)
const SPC_MON_TF_MODB_5            = Int32(500077)

# even more monitors (used on M5i)
const SPC_MON_V_MOD_0              = Int32(500078)
const SPC_MON_V_MOD_1              = Int32(500079)
const SPC_MON_V_MOD_2              = Int32(500080)
const SPC_MON_V_MOD_3              = Int32(500081)
const SPC_MON_V_MOD_4              = Int32(500082)
const SPC_MON_V_MOD_5              = Int32(500083)
const SPC_MON_V_MOD_6              = Int32(500084)
const SPC_MON_V_MOD_7              = Int32(500085)
const SPC_MON_V_MOD_8              = Int32(500086)
const SPC_MON_V_MOD_9              = Int32(500087)
const SPC_MON_V_MOD_10             = Int32(500088)

const SPC_MON_TK_MODULE_2          = Int32(500089)
const SPC_MON_TC_MODULE_2          = Int32(500090)
const SPC_MON_TF_MODULE_2          = Int32(500091)

const SPC_MON_RPM_FAN0             = Int32(500092)
const SPC_MON_RPM_FAN1             = Int32(500093)

const SPC_MON_V_MEMVTTA            = Int32(500094)
const SPC_MON_V_MEMVTTB            = Int32(500095)
const SPC_MON_V_VCCAUX             = Int32(500096)
const SPC_MON_V_MEMAUXA            = Int32(500097)
const SPC_MON_V_MEMAUXB            = Int32(500098)
const SPC_MON_TK_BASE_0            = Int32(500099)
const SPC_MON_TC_BASE_0            = Int32(500100)
const SPC_MON_TF_BASE_0            = Int32(500101)
const SPC_MON_TK_BASE_1            = Int32(500102)
const SPC_MON_TC_BASE_1            = Int32(500103)
const SPC_MON_TF_BASE_1            = Int32(500104)
const SPC_MON_I_CORE               = Int32(500105)
const SPC_MON_V_CORE_REMOTE        = Int32(500106)


# mask with available monitor registers
const SPC_AVAILMONITORS             = Int32(510000)
const     SPCM_MON_T_BASE_CTRL         = UInt64(1) # 0x0000000000000001
const     SPCM_MON_T_MODULE_0          = UInt64(2) # 0x0000000000000002
const     SPCM_MON_T_MODULE_1          = UInt64(4) # 0x0000000000000004
const     SPCM_MON_T_MODULE_2          = UInt64(8) # 0x0000000000000008

const     SPCM_MON_V_PCIE_BUS          = UInt64(16) # 0x0000000000000010
const     SPCM_MON_V_CONNECTOR         = UInt64(32) # 0x0000000000000020
const     SPCM_MON_CARD_PWRSOURCE      = UInt64(64) # 0x0000000000000040
const     SPCM_MON_V_CARD_IN           = UInt64(128) # 0x0000000000000080
const     SPCM_MON_I_CARD_IN           = UInt64(256) # 0x0000000000000100
const     SPCM_MON_P_CARD_IN           = UInt64(512) # 0x0000000000000200
const     SPCM_MON_V_3V3               = UInt64(1024) # 0x0000000000000400
const     SPCM_MON_V_2V5               = UInt64(2048) # 0x0000000000000800
const     SPCM_MON_V_CORE              = UInt64(4096) # 0x0000000000001000
const     SPCM_MON_V_AVTT              = UInt64(8192) # 0x0000000000002000
const     SPCM_MON_V_AVCC              = UInt64(16384) # 0x0000000000004000
const     SPCM_MON_V_MEMVCC            = UInt64(32768) # 0x0000000000008000
const     SPCM_MON_V_MEMVTT            = UInt64(65536) # 0x0000000000010000
const     SPCM_MON_V_CP_POS            = UInt64(131072) # 0x0000000000020000
const     SPCM_MON_V_CP_NEG            = UInt64(262144) # 0x0000000000040000
const     SPCM_MON_V_5VA               = UInt64(524288) # 0x0000000000080000
const     SPCM_MON_V_ADCA              = UInt64(1048576) # 0x0000000000100000
const     SPCM_MON_V_ADCD              = UInt64(2097152) # 0x0000000000200000
const     SPCM_MON_V_OP_POS            = UInt64(4194304) # 0x0000000000400000
const     SPCM_MON_V_OP_NEG            = UInt64(8388608) # 0x0000000000800000
const     SPCM_MON_V_COMP_NEG          = UInt64(16777216) # 0x0000000001000000
const     SPCM_MON_V_COMP_POS          = UInt64(33554432) # 0x0000000002000000
const     SPCM_MON_V_1V8_BASE          = UInt64(67108864) # 0x0000000004000000
const     SPCM_MON_V_1V8_MOD           = UInt64(134217728) # 0x0000000008000000

const     SPCM_MON_V_MODA_0            = UInt64(268435456) # 0x0000000010000000
const     SPCM_MON_V_MODA_1            = UInt64(536870912) # 0x0000000020000000
const     SPCM_MON_V_MODB_0            = UInt64(1073741824) # 0x0000000040000000
const     SPCM_MON_V_MODB_1            = UInt64(2147483648) # 0x0000000080000000

const     SPCM_MON_T_MODA_0            = UInt64(2) # 0x0000000000000002 # same as SPCM_MON_T_MODULE_0
const     SPCM_MON_T_MODA_1            = UInt64(4294967296) # 0x0000000100000000
const     SPCM_MON_T_MODA_2            = UInt64(8589934592) # 0x0000000200000000
const     SPCM_MON_T_MODA_3            = UInt64(17179869184) # 0x0000000400000000
const     SPCM_MON_T_MODA_4            = UInt64(34359738368) # 0x0000000800000000

const     SPCM_MON_T_MODB_0            = UInt64(4) # 0x0000000000000004 # same as SPCM_MON_T_MODULE_1
const     SPCM_MON_T_MODB_1            = UInt64(68719476736) # 0x0000001000000000
const     SPCM_MON_T_MODB_2            = UInt64(137438953472) # 0x0000002000000000
const     SPCM_MON_T_MODB_3            = UInt64(274877906944) # 0x0000004000000000
const     SPCM_MON_T_MODB_4            = UInt64(549755813888) # 0x0000008000000000

const     SPCM_MON_I_MODA_0            = UInt64(1099511627776) # 0x0000010000000000
const     SPCM_MON_I_MODA_1            = UInt64(2199023255552) # 0x0000020000000000
const     SPCM_MON_I_MODA_2            = UInt64(4398046511104) # 0x0000040000000000
const     SPCM_MON_I_MODA_3            = UInt64(8796093022208) # 0x0000080000000000

const     SPCM_MON_I_MODB_0            = UInt64(17592186044416) # 0x0000100000000000
const     SPCM_MON_I_MODB_1            = UInt64(35184372088832) # 0x0000200000000000
const     SPCM_MON_I_MODB_2            = UInt64(70368744177664) # 0x0000400000000000
const     SPCM_MON_I_MODB_3            = UInt64(140737488355328) # 0x0000800000000000

const     SPCM_MON_T_MODA_5            = UInt64(281474976710656) # 0x0001000000000000
const     SPCM_MON_T_MODB_5            = UInt64(562949953421312) # 0x0002000000000000

const     SPCM_MON_V_MEMVTTA           = UInt64(1125899906842624) # 0x0004000000000000
const     SPCM_MON_V_MEMVTTB           = UInt64(2251799813685248) # 0x0008000000000000
const     SPCM_MON_V_MEMAUXA           = UInt64(4503599627370496) # 0x0010000000000000
const     SPCM_MON_V_MEMAUXB           = UInt64(9007199254740992) # 0x0020000000000000
const     SPCM_MON_V_VCCAUX            = UInt64(18014398509481984) # 0x0040000000000000

const     SPCM_MON_T_BASE_0            = UInt64(36028797018963968) # 0x0080000000000000
const     SPCM_MON_T_BASE_1            = UInt64(72057594037927936) # 0x0100000000000000
const     SPCM_MON_RPM_FAN0            = UInt64(144115188075855872) # 0x0200000000000000
const     SPCM_MON_RPM_FAN1            = UInt64(288230376151711744) # 0x0400000000000000

const     SPCM_MON_I_CORE              = UInt64(576460752303423488) # 0x0800000000000000
const     SPCM_MON_V_CORE_REMOTE       = UInt64(1152921504606846976) # 0x1000000000000000


const SPC_AVAILMONITORS1            = Int32(510001)
const     SPCM_MON_V_MOD_0             = UInt64(1) # 0x0000000000000001
const     SPCM_MON_V_MOD_1             = UInt64(2) # 0x0000000000000002
const     SPCM_MON_V_MOD_2             = UInt64(4) # 0x0000000000000004
const     SPCM_MON_V_MOD_3             = UInt64(8) # 0x0000000000000008
const     SPCM_MON_V_MOD_4             = UInt64(16) # 0x0000000000000010
const     SPCM_MON_V_MOD_5             = UInt64(32) # 0x0000000000000020
const     SPCM_MON_V_MOD_6             = UInt64(64) # 0x0000000000000040
const     SPCM_MON_V_MOD_7             = UInt64(128) # 0x0000000000000080
const     SPCM_MON_V_MOD_8             = UInt64(256) # 0x0000000000000100
const     SPCM_MON_V_MOD_9             = UInt64(512) # 0x0000000000000200
const     SPCM_MON_V_MOD_10            = UInt64(1024) # 0x0000000000000400



# ----- re-located multi-purpose i/o related registers -----
const SPC_X0_READFEATURES          = Int32(600000)
const SPC_X1_READFEATURES          = Int32(600001)
const SPC_X2_READFEATURES          = Int32(600002)
const SPC_X3_READFEATURES          = Int32(600003)
const SPC_X4_READFEATURES          = Int32(600004)
const SPC_X5_READFEATURES          = Int32(600005)
const SPC_X6_READFEATURES          = Int32(600006)
const SPC_X7_READFEATURES          = Int32(600007)
const SPC_X8_READFEATURES          = Int32(600008)
const SPC_X9_READFEATURES          = Int32(600009)
const SPC_X10_READFEATURES         = Int32(600010)
const SPC_X11_READFEATURES         = Int32(600011)
const SPC_X12_READFEATURES         = Int32(600012)
const SPC_X13_READFEATURES         = Int32(600013)
const SPC_X14_READFEATURES         = Int32(600014)
const SPC_X15_READFEATURES         = Int32(600015)
const SPC_X16_READFEATURES         = Int32(600016)
const SPC_X17_READFEATURES         = Int32(600017)
const SPC_X18_READFEATURES         = Int32(600018)
const SPC_X19_READFEATURES         = Int32(600019)
const     SPCM_XFEAT_TERM              = Int32(1) # 0x00000001
const     SPCM_XFEAT_HIGHIMP           = Int32(2) # 0x00000002
const     SPCM_XFEAT_DCCOUPLING        = Int32(4) # 0x00000004
const     SPCM_XFEAT_ACCOUPLING        = Int32(8) # 0x00000008
const     SPCM_XFEAT_SE                = Int32(16) # 0x00000010
const     SPCM_XFEAT_DIFF              = Int32(32) # 0x00000020
const     SPCM_XFEAT_PROGTHRESHOLD     = Int32(64) # 0x00000040

const SPC_X0_TERM                 = Int32(600100)
const SPC_X1_TERM                 = Int32(600101)
const SPC_X2_TERM                 = Int32(600102)
const SPC_X3_TERM                 = Int32(600103)
const SPC_X4_TERM                 = Int32(600104)
const SPC_X5_TERM                 = Int32(600105)
const SPC_X6_TERM                 = Int32(600106)
const SPC_X7_TERM                 = Int32(600107)
const SPC_X8_TERM                 = Int32(600108)
const SPC_X9_TERM                 = Int32(600109)
const SPC_X10_TERM                = Int32(600110)
const SPC_X11_TERM                = Int32(600111)
const SPC_X12_TERM                = Int32(600112)
const SPC_X13_TERM                = Int32(600113)
const SPC_X14_TERM                = Int32(600114)
const SPC_X15_TERM                = Int32(600115)
const SPC_X16_TERM                = Int32(600116)
const SPC_X17_TERM                = Int32(600117)
const SPC_X18_TERM                = Int32(600118)
const SPC_X19_TERM                = Int32(600119)

const SPCM_X0_MODE                 = Int32(600200)
const SPCM_X1_MODE                 = Int32(600201)
const SPCM_X2_MODE                 = Int32(600202)
const SPCM_X3_MODE                 = Int32(600203)
const SPCM_X4_MODE                 = Int32(600204)
const SPCM_X5_MODE                 = Int32(600205)
const SPCM_X6_MODE                 = Int32(600206)
const SPCM_X7_MODE                 = Int32(600207)
const SPCM_X8_MODE                 = Int32(600208)
const SPCM_X9_MODE                 = Int32(600209)
const SPCM_X10_MODE                = Int32(600210)
const SPCM_X11_MODE                = Int32(600211)
const SPCM_X12_MODE                = Int32(600212)
const SPCM_X13_MODE                = Int32(600213)
const SPCM_X14_MODE                = Int32(600214)
const SPCM_X15_MODE                = Int32(600215)
const SPCM_X16_MODE                = Int32(600216)
const SPCM_X17_MODE                = Int32(600217)
const SPCM_X18_MODE                = Int32(600218)
const SPCM_X19_MODE                = Int32(600219)

const SPCM_X0_AVAILMODES           = Int32(600300)
const SPCM_X1_AVAILMODES           = Int32(600301)
const SPCM_X2_AVAILMODES           = Int32(600302)
const SPCM_X3_AVAILMODES           = Int32(600303)
const SPCM_X4_AVAILMODES           = Int32(600304)
const SPCM_X5_AVAILMODES           = Int32(600305)
const SPCM_X6_AVAILMODES           = Int32(600306)
const SPCM_X7_AVAILMODES           = Int32(600307)
const SPCM_X8_AVAILMODES           = Int32(600308)
const SPCM_X9_AVAILMODES           = Int32(600309)
const SPCM_X10_AVAILMODES          = Int32(600310)
const SPCM_X11_AVAILMODES          = Int32(600311)
const SPCM_X12_AVAILMODES          = Int32(600312)
const SPCM_X13_AVAILMODES          = Int32(600313)
const SPCM_X14_AVAILMODES          = Int32(600314)
const SPCM_X15_AVAILMODES          = Int32(600315)
const SPCM_X16_AVAILMODES          = Int32(600316)
const SPCM_X17_AVAILMODES          = Int32(600317)
const SPCM_X18_AVAILMODES          = Int32(600318)
const SPCM_X19_AVAILMODES          = Int32(600319)
# for definitions of the available modes see section at SPCM_LEGACY_X0_MODE above

const SPC_NUM_XIO_LINES            = Int32(600400)

const SPC_XIO_PULSEGEN0_MODE       = Int32(601000)
const SPC_XIO_PULSEGEN1_MODE       = Int32(601100)
const SPC_XIO_PULSEGEN2_MODE       = Int32(601200)
const SPC_XIO_PULSEGEN3_MODE       = Int32(601300)
const     SPCM_PULSEGEN_MODE_GATED          = Int32(1)
const     SPCM_PULSEGEN_MODE_TRIGGERED      = Int32(2)
const     SPCM_PULSEGEN_MODE_SINGLESHOT     = Int32(3)

const SPC_XIO_PULSEGEN0_LEN        = Int32(601001)
const SPC_XIO_PULSEGEN1_LEN        = Int32(601101)
const SPC_XIO_PULSEGEN2_LEN        = Int32(601201)
const SPC_XIO_PULSEGEN3_LEN        = Int32(601301)

const SPC_XIO_PULSEGEN0_HIGH       = Int32(601002)
const SPC_XIO_PULSEGEN1_HIGH       = Int32(601102)
const SPC_XIO_PULSEGEN2_HIGH       = Int32(601202)
const SPC_XIO_PULSEGEN3_HIGH       = Int32(601302)

const SPC_XIO_PULSEGEN0_DELAY     = Int32(601003)
const SPC_XIO_PULSEGEN1_DELAY     = Int32(601103)
const SPC_XIO_PULSEGEN2_DELAY     = Int32(601203)
const SPC_XIO_PULSEGEN3_DELAY     = Int32(601303)

const SPC_XIO_PULSEGEN0_LOOPS      = Int32(601004)
const SPC_XIO_PULSEGEN1_LOOPS      = Int32(601104)
const SPC_XIO_PULSEGEN2_LOOPS      = Int32(601204)
const SPC_XIO_PULSEGEN3_LOOPS      = Int32(601304)

const SPC_XIO_PULSEGEN0_MUX1_SRC   = Int32(601005)
const SPC_XIO_PULSEGEN1_MUX1_SRC   = Int32(601105)
const SPC_XIO_PULSEGEN2_MUX1_SRC   = Int32(601205)
const SPC_XIO_PULSEGEN3_MUX1_SRC   = Int32(601305)
const     SPCM_PULSEGEN_MUX1_SRC_UNUSED       = Int32(0)
const     SPCM_PULSEGEN_MUX1_SRC_RUN          = Int32(1)
const     SPCM_PULSEGEN_MUX1_SRC_ARM          = Int32(2)

const SPC_XIO_PULSEGEN0_MUX2_SRC   = Int32(601006)
const SPC_XIO_PULSEGEN1_MUX2_SRC   = Int32(601106)
const SPC_XIO_PULSEGEN2_MUX2_SRC   = Int32(601206)
const SPC_XIO_PULSEGEN3_MUX2_SRC   = Int32(601306)
const     SPCM_PULSEGEN_MUX2_SRC_UNUSED       = Int32(0)
const     SPCM_PULSEGEN_MUX2_SRC_SOFTWARE     = Int32(1)
const     SPCM_PULSEGEN_MUX2_SRC_CARDTRIGGER  = Int32(2)
const     SPCM_PULSEGEN_MUX2_SRC_PULSEGEN0    = Int32(3)
const     SPCM_PULSEGEN_MUX2_SRC_PULSEGEN1    = Int32(4)
const     SPCM_PULSEGEN_MUX2_SRC_PULSEGEN2    = Int32(5)
const     SPCM_PULSEGEN_MUX2_SRC_PULSEGEN3    = Int32(6)
const     SPCM_PULSEGEN_MUX2_SRC_XIO0         = Int32(7)
const     SPCM_PULSEGEN_MUX2_SRC_XIO1         = Int32(8)
const     SPCM_PULSEGEN_MUX2_SRC_XIO2         = Int32(9)
const     SPCM_PULSEGEN_MUX2_SRC_XIO3        = Int32(10)

const SPC_XIO_PULSEGEN0_CONFIG     = Int32(601007)
const SPC_XIO_PULSEGEN1_CONFIG     = Int32(601107)
const SPC_XIO_PULSEGEN2_CONFIG     = Int32(601207)
const SPC_XIO_PULSEGEN3_CONFIG     = Int32(601307)
const     SPCM_PULSEGEN_CONFIG_MUX1_INVERT  = Int32(1) # 0x1
const     SPCM_PULSEGEN_CONFIG_MUX2_INVERT  = Int32(2) # 0x2
const     SPCM_PULSEGEN_CONFIG_INVERT       = Int32(4) # 0x4
const     SPCM_PULSEGEN_CONFIG_HIGH         = Int32(8) # 0x8

const SPC_XIO_PULSEGEN_ENABLE     = Int32(601500)
const     SPCM_PULSEGEN_ENABLE0         = Int32(1) # 0x01
const     SPCM_PULSEGEN_ENABLE1         = Int32(2) # 0x02
const     SPCM_PULSEGEN_ENABLE2         = Int32(4) # 0x04
const     SPCM_PULSEGEN_ENABLE3         = Int32(8) # 0x08

const SPC_XIO_PULSEGEN_COMMAND     = Int32(601501)
const     SPCM_PULSEGEN_CMD_FORCE           = Int32(1) # 0x1

const SPC_XIO_PULSEGEN_CLOCK               = Int32(602000)
const SPC_XIO_PULSEGEN_AVAILLEN_MIN        = Int32(602001)
const SPC_XIO_PULSEGEN_AVAILLEN_MAX        = Int32(602002)
const SPC_XIO_PULSEGEN_AVAILLEN_STEP       = Int32(602003)
const SPC_XIO_PULSEGEN_AVAILHIGH_MIN       = Int32(602004)
const SPC_XIO_PULSEGEN_AVAILHIGH_MAX       = Int32(602005)
const SPC_XIO_PULSEGEN_AVAILHIGH_STEP      = Int32(602006)
const SPC_XIO_PULSEGEN_AVAILDELAY_MIN      = Int32(602007)
const SPC_XIO_PULSEGEN_AVAILDELAY_MAX      = Int32(602008)
const SPC_XIO_PULSEGEN_AVAILDELAY_STEP     = Int32(602009)
const SPC_XIO_PULSEGEN_AVAILLOOPS_MIN      = Int32(602010)
const SPC_XIO_PULSEGEN_AVAILLOOPS_MAX      = Int32(602011)
const SPC_XIO_PULSEGEN_AVAILLOOPS_STEP     = Int32(602012)

# DDS stuff
const SPC_DDS_CORE0_FREQ                   = Int32(603000)
const SPC_DDS_CORE1_FREQ                   = Int32(603001)
const SPC_DDS_CORE2_FREQ                   = Int32(603002)
const SPC_DDS_CORE3_FREQ                   = Int32(603003)
const SPC_DDS_CORE4_FREQ                   = Int32(603004)
const SPC_DDS_CORE5_FREQ                   = Int32(603005)
const SPC_DDS_CORE6_FREQ                   = Int32(603006)
const SPC_DDS_CORE7_FREQ                   = Int32(603007)
const SPC_DDS_CORE8_FREQ                   = Int32(603008)
const SPC_DDS_CORE9_FREQ                   = Int32(603009)
const SPC_DDS_CORE10_FREQ                  = Int32(603010)
const SPC_DDS_CORE11_FREQ                  = Int32(603011)
const SPC_DDS_CORE12_FREQ                  = Int32(603012)
const SPC_DDS_CORE13_FREQ                  = Int32(603013)
const SPC_DDS_CORE14_FREQ                  = Int32(603014)
const SPC_DDS_CORE15_FREQ                  = Int32(603015)
const SPC_DDS_CORE16_FREQ                  = Int32(603016)
const SPC_DDS_CORE17_FREQ                  = Int32(603017)
const SPC_DDS_CORE18_FREQ                  = Int32(603018)
const SPC_DDS_CORE19_FREQ                  = Int32(603019)
const SPC_DDS_CORE20_FREQ                  = Int32(603020)
const SPC_DDS_CORE21_FREQ                  = Int32(603021)
const SPC_DDS_CORE22_FREQ                  = Int32(603022)

const SPC_DDS_CORE0_FREQ_SLOPE             = Int32(604000)
const SPC_DDS_CORE1_FREQ_SLOPE             = Int32(604001)
const SPC_DDS_CORE2_FREQ_SLOPE             = Int32(604002)
const SPC_DDS_CORE3_FREQ_SLOPE             = Int32(604003)
const SPC_DDS_CORE4_FREQ_SLOPE             = Int32(604004)
const SPC_DDS_CORE5_FREQ_SLOPE             = Int32(604005)
const SPC_DDS_CORE6_FREQ_SLOPE             = Int32(604006)
const SPC_DDS_CORE7_FREQ_SLOPE             = Int32(604007)
const SPC_DDS_CORE8_FREQ_SLOPE             = Int32(604008)
const SPC_DDS_CORE9_FREQ_SLOPE             = Int32(604009)
const SPC_DDS_CORE10_FREQ_SLOPE            = Int32(604010)
const SPC_DDS_CORE11_FREQ_SLOPE            = Int32(604011)
const SPC_DDS_CORE12_FREQ_SLOPE            = Int32(604012)
const SPC_DDS_CORE13_FREQ_SLOPE            = Int32(604013)
const SPC_DDS_CORE14_FREQ_SLOPE            = Int32(604014)
const SPC_DDS_CORE15_FREQ_SLOPE            = Int32(604015)
const SPC_DDS_CORE16_FREQ_SLOPE            = Int32(604016)
const SPC_DDS_CORE17_FREQ_SLOPE            = Int32(604017)
const SPC_DDS_CORE18_FREQ_SLOPE            = Int32(604018)
const SPC_DDS_CORE19_FREQ_SLOPE            = Int32(604019)
const SPC_DDS_CORE20_FREQ_SLOPE            = Int32(604020)
const SPC_DDS_CORE21_FREQ_SLOPE            = Int32(604021)
const SPC_DDS_CORE22_FREQ_SLOPE            = Int32(604022)

const SPC_DDS_CORE0_AMP                    = Int32(605000)
const SPC_DDS_CORE1_AMP                    = Int32(605001)
const SPC_DDS_CORE2_AMP                    = Int32(605002)
const SPC_DDS_CORE3_AMP                    = Int32(605003)
const SPC_DDS_CORE4_AMP                    = Int32(605004)
const SPC_DDS_CORE5_AMP                    = Int32(605005)
const SPC_DDS_CORE6_AMP                    = Int32(605006)
const SPC_DDS_CORE7_AMP                    = Int32(605007)
const SPC_DDS_CORE8_AMP                    = Int32(605008)
const SPC_DDS_CORE9_AMP                    = Int32(605009)
const SPC_DDS_CORE10_AMP                   = Int32(605010)
const SPC_DDS_CORE11_AMP                   = Int32(605011)
const SPC_DDS_CORE12_AMP                   = Int32(605012)
const SPC_DDS_CORE13_AMP                   = Int32(605013)
const SPC_DDS_CORE14_AMP                   = Int32(605014)
const SPC_DDS_CORE15_AMP                   = Int32(605015)
const SPC_DDS_CORE16_AMP                   = Int32(605016)
const SPC_DDS_CORE17_AMP                   = Int32(605017)
const SPC_DDS_CORE18_AMP                   = Int32(605018)
const SPC_DDS_CORE19_AMP                   = Int32(605019)
const SPC_DDS_CORE20_AMP                   = Int32(605020)
const SPC_DDS_CORE21_AMP                   = Int32(605021)
const SPC_DDS_CORE22_AMP                   = Int32(605022)

const SPC_DDS_CORE0_AMP_SLOPE              = Int32(606000)
const SPC_DDS_CORE1_AMP_SLOPE              = Int32(606001)
const SPC_DDS_CORE2_AMP_SLOPE              = Int32(606002)
const SPC_DDS_CORE3_AMP_SLOPE              = Int32(606003)
const SPC_DDS_CORE4_AMP_SLOPE              = Int32(606004)
const SPC_DDS_CORE5_AMP_SLOPE              = Int32(606005)
const SPC_DDS_CORE6_AMP_SLOPE              = Int32(606006)
const SPC_DDS_CORE7_AMP_SLOPE              = Int32(606007)
const SPC_DDS_CORE8_AMP_SLOPE              = Int32(606008)
const SPC_DDS_CORE9_AMP_SLOPE              = Int32(606009)
const SPC_DDS_CORE10_AMP_SLOPE             = Int32(606010)
const SPC_DDS_CORE11_AMP_SLOPE             = Int32(606011)
const SPC_DDS_CORE12_AMP_SLOPE             = Int32(606012)
const SPC_DDS_CORE13_AMP_SLOPE             = Int32(606013)
const SPC_DDS_CORE14_AMP_SLOPE             = Int32(606014)
const SPC_DDS_CORE15_AMP_SLOPE             = Int32(606015)
const SPC_DDS_CORE16_AMP_SLOPE             = Int32(606016)
const SPC_DDS_CORE17_AMP_SLOPE             = Int32(606017)
const SPC_DDS_CORE18_AMP_SLOPE             = Int32(606018)
const SPC_DDS_CORE19_AMP_SLOPE             = Int32(606019)
const SPC_DDS_CORE20_AMP_SLOPE             = Int32(606020)
const SPC_DDS_CORE21_AMP_SLOPE             = Int32(606021)
const SPC_DDS_CORE22_AMP_SLOPE             = Int32(606022)

const SPC_DDS_CORE0_PHASE                  = Int32(607000)
const SPC_DDS_CORE1_PHASE                  = Int32(607001)
const SPC_DDS_CORE2_PHASE                  = Int32(607002)
const SPC_DDS_CORE3_PHASE                  = Int32(607003)
const SPC_DDS_CORE4_PHASE                  = Int32(607004)
const SPC_DDS_CORE5_PHASE                  = Int32(607005)
const SPC_DDS_CORE6_PHASE                  = Int32(607006)
const SPC_DDS_CORE7_PHASE                  = Int32(607007)
const SPC_DDS_CORE8_PHASE                  = Int32(607008)
const SPC_DDS_CORE9_PHASE                  = Int32(607009)
const SPC_DDS_CORE10_PHASE                 = Int32(607010)
const SPC_DDS_CORE11_PHASE                 = Int32(607011)
const SPC_DDS_CORE12_PHASE                 = Int32(607012)
const SPC_DDS_CORE13_PHASE                 = Int32(607013)
const SPC_DDS_CORE14_PHASE                 = Int32(607014)
const SPC_DDS_CORE15_PHASE                 = Int32(607015)
const SPC_DDS_CORE16_PHASE                 = Int32(607016)
const SPC_DDS_CORE17_PHASE                 = Int32(607017)
const SPC_DDS_CORE18_PHASE                 = Int32(607018)
const SPC_DDS_CORE19_PHASE                 = Int32(607019)
const SPC_DDS_CORE20_PHASE                 = Int32(607020)
const SPC_DDS_CORE21_PHASE                 = Int32(607021)
const SPC_DDS_CORE22_PHASE                 = Int32(607022)

const SPC_DDS_TRG_SRC                      = Int32(608000)
const     SPCM_DDS_TRG_SRC_NONE                = Int32(0)
const     SPCM_DDS_TRG_SRC_TIMER               = Int32(1)
const     SPCM_DDS_TRG_SRC_CARD                = Int32(2)
# set XIO with register below

const SPC_DDS_TRG_TIMER                    = Int32(608001)

const SPC_DDS_PHASE_BEHAVIOUR              = Int32(608002)
const     SPCM_DDS_PHASE_JUMP                  = Int32(0)
const     SPCM_DDS_PHASE_SHIFT                 = Int32(1)

const SPC_DDS_CMD                          = Int32(608003)
const     SPCM_DDS_CMD_RESET                   = Int32(1) # 0x01
const     SPCM_DDS_CMD_EXEC_AT_TRG             = Int32(2) # 0x02
const     SPCM_DDS_CMD_EXEC_NOW                = Int32(4) # 0x04
const     SPCM_DDS_CMD_WRITE_TO_CARD           = Int32(8) # 0x08
const     SPCM_DDS_CMD_NO_NOP_FILL             = Int32(16) # 0x10

const SPC_DDS_STATUS                       = Int32(608004) # read-only
const     SPCM_DDS_STAT_WAITING_FOR_TRG        = Int32(1) # 0x01
const     SPCM_DDS_STAT_QUEUE_UNDERRUN         = Int32(2) # 0x02

const SPC_DDS_QUEUE_CMD_MAX                = Int32(608005) # read-only
const SPC_DDS_QUEUE_CMD_COUNT              = Int32(608006) # read-only
const SPC_DDS_NUM_CORES                    = Int32(608007) # read-only

const SPC_DDS_FREQ_RAMP_STEPSIZE           = Int32(608008)
const SPC_DDS_AMP_RAMP_STEPSIZE            = Int32(608009)

const SPC_DDS_X_MANUAL_OUTPUT              = Int32(608010)
const     SPCM_DDS_X0                          = Int32(1) # 0x1
const     SPCM_DDS_X1                          = Int32(2) # 0x2
const     SPCM_DDS_X2                          = Int32(4) # 0x4

const SPC_DDS_NUM_QUEUED_CMD_IN_SW         = Int32(608011) # read-only

const SPC_DDS_DATA_TRANSFER_MODE           = Int32(608012)
const     SPCM_DDS_DTM_SINGLE                  = Int32(0)
const     SPCM_DDS_DTM_DMA                     = Int32(1)

const SPC_DDS_TRG_COUNT                    = Int32(608013) # read-only

const SPC_DDS_CORES_ON_CH0                 = Int32(608040)
const SPC_DDS_CORES_ON_CH1                 = Int32(608041)
const SPC_DDS_CORES_ON_CH2                 = Int32(608042)
const SPC_DDS_CORES_ON_CH3                 = Int32(608043)
const     SPCM_DDS_CORE_NONE                   = Int32(0) # 0x000000
const     SPCM_DDS_CORE0                       = Int32(1) # 0x000001
const     SPCM_DDS_CORE1                       = Int32(2) # 0x000002
const     SPCM_DDS_CORE2                       = Int32(4) # 0x000004
const     SPCM_DDS_CORE3                       = Int32(8) # 0x000008
const     SPCM_DDS_CORE4                       = Int32(16) # 0x000010
const     SPCM_DDS_CORE5                       = Int32(32) # 0x000020
const     SPCM_DDS_CORE6                       = Int32(64) # 0x000040
const     SPCM_DDS_CORE7                       = Int32(128) # 0x000080
const     SPCM_DDS_CORE8                       = Int32(256) # 0x000100
const     SPCM_DDS_CORE9                       = Int32(512) # 0x000200
const     SPCM_DDS_CORE10                      = Int32(1024) # 0x000400
const     SPCM_DDS_CORE11                      = Int32(2048) # 0x000800
const     SPCM_DDS_CORE12                      = Int32(4096) # 0x001000
const     SPCM_DDS_CORE13                      = Int32(8192) # 0x002000
const     SPCM_DDS_CORE14                      = Int32(16384) # 0x004000
const     SPCM_DDS_CORE15                      = Int32(32768) # 0x008000
const     SPCM_DDS_CORE16                      = Int32(65536) # 0x010000
const     SPCM_DDS_CORE17                      = Int32(131072) # 0x020000
const     SPCM_DDS_CORE18                      = Int32(262144) # 0x040000
const     SPCM_DDS_CORE19                      = Int32(524288) # 0x080000
const     SPCM_DDS_CORE20                      = Int32(1048576) # 0x100000
const     SPCM_DDS_CORE21                      = Int32(2097152) # 0x200000
const     SPCM_DDS_CORE22                      = Int32(4194304) # 0x400000

const SPC_DDS_X0_MODE                      = Int32(608060)
const SPC_DDS_X1_MODE                      = Int32(608061)
const SPC_DDS_X2_MODE                      = Int32(608062)
#          NONE                                     is not needed because one can simply switch off the XMODE
const     SPCM_DDS_XMODE_MANUAL                = Int32(1)
const     SPCM_DDS_XMODE_WAITING_FOR_TRG       = Int32(2)
const     SPCM_DDS_XMODE_EXEC                  = Int32(3)


const SPC_DDS_AVAIL_FREQ_MIN               = Int32(608500)
const SPC_DDS_AVAIL_FREQ_MAX               = Int32(608501)
const SPC_DDS_AVAIL_FREQ_STEP              = Int32(608502)

const SPC_DDS_AVAIL_FREQ_SLOPE_MIN         = Int32(608503)
const SPC_DDS_AVAIL_FREQ_SLOPE_MAX         = Int32(608504)
const SPC_DDS_AVAIL_FREQ_SLOPE_STEP        = Int32(608505)

const SPC_DDS_AVAIL_AMP_MIN                = Int32(608506)
const SPC_DDS_AVAIL_AMP_MAX                = Int32(608507)
const SPC_DDS_AVAIL_AMP_STEP               = Int32(608508)

const SPC_DDS_AVAIL_AMP_SLOPE_MIN          = Int32(608509)
const SPC_DDS_AVAIL_AMP_SLOPE_MAX          = Int32(608510)
const SPC_DDS_AVAIL_AMP_SLOPE_STEP         = Int32(608511)

const SPC_DDS_AVAIL_PHASE_MIN              = Int32(608512)
const SPC_DDS_AVAIL_PHASE_MAX              = Int32(608513)
const SPC_DDS_AVAIL_PHASE_STEP             = Int32(608514)


# ----- Hardware registers (debug use only) -----
const SPC_REG0x00                  = Int32(900000)
const SPC_REG0x02                  = Int32(900010)
const SPC_REG0x04                  = Int32(900020)
const SPC_REG0x06                  = Int32(900030)
const SPC_REG0x08                  = Int32(900040)
const SPC_REG0x0A                  = Int32(900050)
const SPC_REG0x0C                  = Int32(900060)
const SPC_REG0x0E                  = Int32(900070)

const SPC_DEBUGREG0                = Int32(900100)
const SPC_DEBUGREG15               = Int32(900115)
const SPC_DEBUGVALUE0              = Int32(900200)
const SPC_DEBUGVALUE15             = Int32(900215)

const SPC_MI_ISP                   = Int32(901000)
const     ISP_TMS_0                    = Int32(0)
const     ISP_TMS_1                    = Int32(1)
const     ISP_TDO_0                    = Int32(0)
const     ISP_TDO_1                    = Int32(2)


const SPC_EE_RWAUTH                = Int32(901100)
const SPC_EE_REG                   = Int32(901110)
const SPC_EE_RESETCOUNTER          = Int32(901120)

# ----- Test Registers -----
const SPC_TEST_BASE                = Int32(902000)
const SPC_TEST_LOCAL_START         = Int32(902100)
const SPC_TEST_LOCAL_END           = Int32(902356)
const SPC_TEST_PLX_START           = Int32(902400)
const SPC_TEST_PLX_END             = Int32(902656)

# 9012xx not usable
# 901900 not usable
# 903000 not usable
# 91xxxx not usable

# ----- used by GetErrorInfo to mark errors in other functions than SetParam/GetParam -----
const SPC_FUNCTION_DEFTRANSFER  = Int32(100000000)
